<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

if(!defined('LINPHA_DIR')) { exit(1); }

/**
 * Take care of translation
 * 
 * This simple method takes care of all translation related stuff.
 * If option tr_learn is set - all new entries will be autolearned.
 * @param string $text phrase/word to translate
 * @package Installation
 */
 function tr($text)
 {
	global $translate;
	$tr_learn = false; //autolearn new language entries
	
	/**
	 * translation starts here
	 */
	if(true == isset($translate[$text]))
	{
		$str = "$translate[$text]";
	}
	else
	{
		$str = "$text";
	}

	/**
	 * autolearn new language entries defined by tr() function.
	 */
	if($tr_learn == true)
	{
		$tmpfile = LINPHA_DIR."/var/lang.install.temp.txt";

		if(false == file_exists("$tmpfile"))
		{	
			if(false == touch("$tmpfile"))
			{
				echo "Unable to create ".$tmpfile." - please check permissions";
			}
		}
		
		$filedata = file("$tmpfile");

		if(false == in_array($text."\n", $filedata))
		{
			$filedata[] = $text."\n";
			natcasesort($filedata);	// Sort an array using a case insensitive "natural order" algorithm
			file_put_contents($tmpfile, $filedata);
		}
	}
	
	return $str;

} // end tr()

function installGetPATH($array_path)
{
	/**
	* use the path variable instead of own array
	*/
	if($array_path == 'PATH')
	{
		unset($array_path);
		if(!empty($_SERVER['PATH']))
		{
			$path = $_SERVER['PATH'];
		}
		elseif(!empty($_SERVER['Path']))
		{
			$path = $_SERVER['Path'];
		}
		else
		{
			$path = '';
		}
		$array_path = explode(PATH_SEPARATOR,$path);
	}
	// if we have open_basedir restrictions, take only the paths which are allowed from open_basedir to prevent errors
	$open_basedir = ini_get('open_basedir');
	if(!empty($open_basedir)) {
		$arr_open_basedir = explode(PATH_SEPARATOR,$open_basedir);
		
		$array_path = array_intersect($array_path, $arr_open_basedir);
	}
	
	// same with safemode
	if(ini_get('safe_mode'))
	{
		$safe_mode_exec_dir = ini_get('safe_mode_exec_dir');
		if(!empty($safe_mode_exec_dir)) {
			
			$arr_safe_mode_exec_dir = explode(PATH_SEPARATOR,$safe_mode_exec_dir);
			
			$array_path = array_intersect($array_path, $arr_safe_mode_exec_dir);
		}
	}
	
	return $array_path;
}

/**
 * delete directory recurisvely - same as "rm -rf"
 */
function installRm_rf($path,$followLinks=false)
{
	$dir = opendir($path) ;
	while (false !== ($entry = readdir($dir)))
	{
		if ( is_file( "$path/$entry" ) || ((!$followLinks) && is_link("$path/$entry")) )
		{
			unlink( "$path/$entry" );
		}
		elseif ( is_dir( "$path/$entry" ) && $entry!='.' && $entry!='..' )
		{
			installRm_rf( "$path/$entry" );
		}
	}
	closedir($dir) ;
	return @rmdir($path);
}

/**
* create directory recursively - same as "mkdir -p"
*/
function installMkdir_p($target,$mode='0700')
{
	if(is_dir($target) || empty($target)) {	// best case check first
		return true;
	}
	if(file_exists($target) && !is_dir($target)) {	// target exists bug isn't a directory..
		return false;
	}
	if( installMkdir_p( substr( $target,0,strrpos( $target,'/') ) ) )
	{
		return mkdir($target,intval($mode,8)); // crawl back up & create dir tree
	}
	else
	{
		return false;
	}
}

/**
 * realpath() doesnt work if path is absolute
 * installLinRealpath takes care of this
 */
function installLinRealpath( $path)
{
	if( ! installIsAbsolutePath( $path ) )
	{
		if(file_exists($path))
		{
			$path = realpath($path);
		}
		else
		{
			// remove first ../ because we are here in the install directory
			$path = substr($path,3,strlen($path)-3);
			$path = realpath(LINPHA_DIR).'/'.$path;
		}
	}
	
	/**
	 * remove ending '/' if exists
	 */
	$len = strlen( $path );
	$lastsign = substr( $path, $len-1);
	if($lastsign == '/' OR $lastsign == '\\')
	{
		$path = substr($path,0,$len-1);
	}
	
	/**
	 * unify slashes
	 */
	if(strpos( PHP_OS, 'WIN' ) !== FALSE)
	{
		$path = str_replace('/','\\',$path);
	}
	else
	{
		$path = str_replace('\\','/',$path);
	}
	
	return $path;
}

/**
* if $path is absolute, this is returned, if $path is relative, the relative path starting
* from the linpha rootdirectory is returned
*
* @author  flo
* @param  string  $path  a path
* @return  string  a path
* @uses  actions/submit_mod_data.php,photo_cache_cleanup_by_id(),photo_cache_delete_folder()
* @package  filesys
*/
function installGetFullPath($path)
{
	if(installIsAbsolutePath($path)) {
		$full_path = $path;
	} else {
		$full_path = LINPHA_DIR.'/'.$path;
	}
	
	return $full_path;
}

/**
* checks wether $path is absolute or relative
*
* @author  flo
* @param  string  $path  a path
* @return  ture|false  is an absolute path or not
* @uses  installGetFullPath()
* @package  filesys
*/
function installIsAbsolutePath($path)
{
	if(substr($path,0,1) == '/' OR		// string begins with a '/' -> absolut path
		((strpos($path, ':') !== false) AND (strpos( PHP_OS, 'WIN' ) !== FALSE))	
		// we are on windows and the string contains a ':' -> absolut path (c:/..)
		) {	// absolut path
		return true;
	} else {
		return false;
	}
}

/**
 * return an array with all language files found in the folder 'lang'
 */
function get_available_language_files()
{
	$folder = installGetFullPath('install/lang');

	$file_handle = opendir($folder);
	$file = readdir($file_handle);

	//$options = array();
	while($file != false)
	{
		$explode = explode(".",$file);

		/* Only files that starts with 'lang.' are language files. */
		if($explode[0] == "lang")
		{
			$options[$explode[1]] = $explode[1];
		}
		$file = readdir($file_handle);
	}

	/* Sort language files aphabetically */
	if(@is_array($options))
	{
		asort($options);
	}
	return @$options;
}

function get_http_accept_lang()
{
 /*##########################################################
## used in: header.php/install.php
## The magic of language auto-detection
## This alows us to welcome users with their natural language if already available
## returns:
## language to use
##########################################################*/

	$http_accept_language = trim($_SERVER["HTTP_ACCEPT_LANGUAGE"]);
	// Accept-Language Header detail, see RFC2616 
	$knownlangs = array( "" => "" );
						/*
						"dk" => "Danish",
						"nl"	=> "Dutch",
						"en"	=> "English",
						"de"	=> "German",
						"de-ch"	=> "German",	// for IE 6
						"de-at"	=> "German",	// for IE 6
						"ja"	=> "Japanese",
						"it"	=> "Italian",
						"es"	=> "Spanish",
						"co"	=> "Spanish",
						"fr"	=> "French",
						"no"	=> "Norwegian",
						"pt"	=> "BrazilianPortuguese",
						"br"	=> "BrazilianPortuguese",
						"zh"	=> "zh-tw",
						"tw"	=> "zh-tw",
						"cz"	=> "Czech",
						"cs"	=> "Czech",
						"ro"	=> "Romanian",
						"cn"	=> "zh-cn",
						"se"	=> "Swedish",
						"sv"	=> "Swedish",
						"ru"	=> "Russian" );
						*/

$accept_lang = 'English';
$lastquality = 0.0;

$langtag = '((?:[a-zA-Z]{1,8})(?:-[a-zA-Z]{1,8})*)';
$qvalue ='(0(?:\.[0-9]{1,3})?|1(?:\.0{1,3}))';
$eachbit = '^' . $langtag . '(?:;q=' . $qvalue . ')?(?:,\s*)?(.*)$';

while(strlen($http_accept_language))
{
	$m = Array();	// do not this directly in preg_match() !
	if(preg_match( "/$eachbit/", $http_accept_language, $m))
	{
		$tag = $m[1];
		$quality = $m[2];
		
		if(strlen($quality) == 0) $quality = 1;
      	$http_accept_language = $m[3];

		if(array_key_exists($tag, $knownlangs) && $quality > $lastquality)
		{
			$accept_lang = $knownlangs[$tag];
			$lastquality = $quality;
		}
		}
		else
		{
      		break;
		}
}

return $accept_lang;
}

/**
 * creates a random password
 */
function random_password($len)
{
	$pass = '';
	$lchar = 0;
	$char = 0;
	for($i = 0; $i < $len; $i++)
	{
		while($char == $lchar)
		{
			$char = rand(48, 109);
			if($char > 57) $char += 7;
			if($char > 90) $char += 6;
		}
		$pass .= chr($char);
		$lchar = $char;
	}
	return $pass;
}


/**
 * get a clean path
 * input:  ..//./../dir4//./dir5/dir6/..//dir7/
 * output: ../../dir4/dir5/dir7/
 * 
 * the php builtin function realpath does not work if the file does not exists
 * 
 * thanks to http://ch2.php.net/realpath
 */
function installCleanPath($path) {
	$result = array();
	// $pathA = preg_split('/[\/\\\]/', $path);
	$pathA = explode('/', $path);
	if (!$pathA[0])
		$result[] = '';
	foreach ($pathA AS $key => $dir) {
		if ($dir == '..') {
			if (end($result) == '..') {
				$result[] = '..';
			} elseif (!array_pop($result)) {
				$result[] = '..';
			}
		} elseif ($dir && $dir != '.') {
			$result[] = $dir;
		}
	}
	if (!end($pathA))
		$result[] = '';
	return implode('/', $result);
}



function failed_msg()
{
	$str = "<font color='red'>";
	$str .= "failed";
	$str .= "</font>";
	return $str;
}

function warning_msg()
{
	$str = "<font color='orange'>";
	$str .= "warning";
	$str .= "</font>";
	return $str;
}

function success_msg()
{
	$str = "<font color='green'>";
	$str .= "ok";
	$str .= "</font>";
	return $str;
}

/**
 * As we need this twice make it a method. This is because we skip steps 
 * 5+6 when using sqlite but still need to init values
 * @author  bzrudi
 * @param  none
 * @return  array $dirs
 * @package  filesys
 */
function initSessionValues()
{
/**
 * save settings from previous page into global $_SESSION var
 */
	$array = array('album','sql','cache','tmp');
	foreach($array AS $value)
	{
		/**
		 * remove ending slash if not already done
		 */
		$len = strlen( $_POST[$value.'_dir'] );
		$lastsign = substr( $_POST[$value.'_dir'], $len-1);
		if($lastsign == '/' && $lastsign == '\\')
		{
			$_POST[$value.'_dir'] = substr($_POST[$value.'_dir'],0,$len-1);
		}

		/**
		 * save current value and append correct dir names
		 */
		$_SESSION[$value.'_dir'] =  $_POST[$value.'_dir'];
		
		/**
		 * get correct dir names (take care of relativ and absolute paths, add
		 * LINPHA_DIR if necessary)
		 */
		$dirs[$value] = installGetFullPath( $_SESSION[$value.'_dir'] );
	}
return $dirs; 
}

/**
 * As we need this twice make it a method. This is because we skip steps 
 * 5+6 when using sqlite but still need to init directories below /var
 * @author  bzrudi
 * @param  none
 * @return  true|false
 * @package  filesys
 */
function initInitialDirectories($dirs)
{
	try
	{
		$init_ok = true;
		echo "<h2>".tr("Checking Directory Permissions")."</h2><br />";

		/**
		 * checking album directory
		 */
		echo tr("Album Directory")."... ";
		if( is_dir( $dirs['album'] ) )
		{
			if( is_writable( $dirs['album'] ) )
			{
				echo success_msg();
				echo '<br /> ('.$_SESSION['album_dir'].' => '.installLinRealpath( $dirs['album'] ).')';
			}
			else
			{
				echo warning_msg();
				echo '<br /> ('.$_SESSION['album_dir'].' => '.installLinRealpath( $dirs['album'] ).')';
				echo "<br />".tr("Album Directory Not Writable. Some Features May Not Work")."<br />";
			}
		}
		else
		{
			throw new Exception(failed_msg()."<br />".$_SESSION['album_dir']."<br />".tr("No Valid Directory Found"));
		}


		echo "<br /><br />";
		echo tr("Creating Required Directories")."... ";

		$array = array('sql','cache','tmp');
		foreach($array AS $value)
		{
			echo "<br /><br />".tr("Checking Directory").": ".$value."...";
	
			if( file_exists( $dirs[$value] ) )
			{
				/**
				 * oh oh, pay attention, only try to delete the folder on default values!
				 * if the users choose wrongly the albums folder, all images would be deleted.!!!!
				 */
				if( $_SESSION[$value.'_dir'] == 'var/'.$value)
				{
					echo warning_msg();
					echo "<br />".tr("Directory Already Exists - Trying To Delete")."... ";
					if( installRm_rf( $dirs[$value] ) )
					{
						echo success_msg();
					}
					else
					{
						throw new Exception(failed_msg()."<br />".tr("Please Delete Folder Manually"));
					}
				}
				else
				{
					throw new Exception(failed_msg()."<br />".tr("Directory Allready Exists, Please Delete Folder Manually"));
				}
			}
	
			echo "<br /> ".tr("Trying To Create Directory")."... ";
			if( installMkdir_p( $dirs[$value] ) )
			{
				echo success_msg();
	
				if( ! is_writable( $dirs[$value] ) )
				{
					echo "<br />".tr("No Write Permissions. Please Change Permissions Manually!");
				}
			}
			else
			{
				throw new Exception(failed_msg()."<br />".tr("Cannot create folder, please check permissions!"));
			}
	
			echo '<br /> ('.$_SESSION[$value.'_dir'].' => '.installLinRealpath( $dirs[$value] ).')<br />';
		}
	}
	catch(Exception $error)
	{
		echo $error -> getMessage();
		echo '<br /> ('.$_SESSION[$value.'_dir'].' => '.installLinRealpath( $dirs[$value] ).')<br />';
		$init_ok = false;
	}

return $init_ok;	
} 


?>