<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

if(!defined('LINPHA_DIR')) { exit(1); }

/**
* @package Installation
*/
function installCheckConvert()
{
	if( LIN_OS == 'win' ) {
		$convert = 'convert.exe';
	} else {
		$convert = 'convert';
	}
	$array_path = installGetPATH('PATH');
	$array_lookfor = installGetPATH(Array('/bin','/usr/bin','/usr/local/bin','/sw/bin'));
	
	/**
	* need a special check if safe_mode is on
	* because we can't check anymore with file_exists() since php 4.3.9 even if the file is in
	* allowed dirs (see http://bugs.php.net/bug.php?id=30259)
	*
	* try to execute in every location
	*/
	if(ini_get('safe_mode'))
	{
		foreach($array_path AS $value)
		{
			if( !isset($array) )
			{
				//echo $value.'<br />';
				$arr=array(); $return_var='';	// do not this directly in exec() !
				exec($value.'/'.$convert.' -help',$arr,$return_var);
			
				//print_r($arr);
				
				if(isset($return_var) && $return_var == 0)
				{
					$array = Array(1, '', '');
				}
			}
		}
		
		// search file in other locations
		foreach($array_lookfor AS $value)
		{
			if( !isset($array) )
			{
				$arr=array(); $return_var='';	// do not this directly in exec() !
				exec($value.'/'.$convert.' -help',$arr,$return_var);
			
				if(isset($return_var) && $return_var == 0)
				{
					$array = Array(1, '', '');
				}
			}
		}
	}
	else
	{
		/**
		* search file in PATH
		*
		* if convert is found in the path variable, don't store the full path
		* because if the path changes, we get problems (for example under windows:
		* c:/program files/imagemagick-5.8.... changes to c:/program files/imagemagick-6.0.0)
		*/
		foreach($array_path AS $value)
		{
			if( !isset($array) && file_exists($value.'/'.$convert) )
			{
				$array = Array(1, '','');
			}
		}
		
		// search file in other locations
		foreach($array_lookfor AS $value)
		{
			if( !isset($array) && file_exists($value.'/'.$convert) )
			{
				$array = Array(1, $value.'/','');
			}
		}
	}
	
	if(!isset($array) )
	{
		$array = Array(0,'','');
		return $array;
	}
	
	/**
	* now, we do an improved check
	*/
	$arr=array(); $return_var='';	// do not this directly in exec() !
	exec($array[1].$convert.' -help',$arr,$return_var);

	if(isset($return_var) && $return_var == 0)
	{
		$needle = 'ImageMagick ';

		$pos = strpos($arr[0],$needle);
		
		if($pos === false)
		{
			/**
			* wrong convert found ?!
			* @todo  we should now set use_convert to '0' ?!
			*/
			$version = '';
		}
		else
		{
			$version = substr($arr[0],$pos+strlen($needle),5);
		}
		
		$array[2] = $version;
	}
	else
	{
		/**
		* convert not found...
		*/
		$array = array(0,'','');
	}
	
	return $array;
}

/**
* this function checks if the posix functions are available
*
* @author  flo
* @return  bool  true if posix enabled, false if not
* @uses  sec_stage_install.php, plugins/ftp/index.php
* @package  requirements
*/
function installCheckPosix()
{
	$array_needed_functions = Array(
		'posix_getuid',
		'posix_getpwuid',
		'posix_getgrgid'
	);
	
	foreach($array_needed_functions AS $value)
	{
		if( !function_exists($value) )
		{
			return false;
		}
	}
	
    /**
    * can return true even if it fails...!!! seen on sf.net
    */
	if($uid = @posix_getuid())
	{
		if(@posix_getpwuid($uid))
		{
			return true;
		}
	}
    
    return false;
}

/**
* @package  requirements
*/
function installGetMemLimit()
{
	$memlimit=ini_get('memory_limit');

	/*
	 * if memory limit is disabled and we would have plenty of memory, get_cfg_var() gets 16MB...
	 * 
	 * found this on ch.php.net/get_cfg_var:
	 * As implemented, this function is an odd wart--it calls cfg_get_string, which returns the value originally set in the config file, if any.
	Unfortunately, you almost never want to know the original value in the config file. Instead, you want to know the value currently in effect. (These two values can be different if you have no php.ini file, or if you're setting options in httpd.conf or .htaccess files.)
	To get the current value of a configuration option you should use ini_get() instead
	 * if(empty($memlimit))
    {
        $memlimit = get_cfg_var('memory_limit');
    }
    */
    
    /**
     * mem limit disabled (-1) or not compiled in
     */
    if(empty($memlimit) OR $memlimit == -1)
    {
        return false;
    }
    else
    {
	   $memlimit=str_replace("M", "", $memlimit);
	   return $memlimit;
    }
}

/**
 * this function checks if the server supports the .htaccess directive to forbid
 * access to albums and var folder
 * 
 * @author  flo
 * @uses  step2_requirements.php
 * @package  requirements
 * @return  int  error number: 1: access forbidden, 2: access possible, 3: unknown
 * @todo  test with other webservers than apache2 (-> apache1, IIS, ..?)
 */
function installCheckProtectedFromOutside()
{
	$dir = installCleanPath( dirname($_SERVER['PHP_SELF']).'/'.LINPHA_DIR );	// same as LINPHA_CLIENT
	$dir .= '/albums/';
	$url = $_SERVER['SERVER_NAME'];
	$port = $_SERVER['SERVER_PORT'];
	
	$fp = fsockopen ($url, $port, $errno, $errstr, 5);
	if (!$fp)
	{
		return 3;
	}
	else
	{
		$get_data = "GET ".$dir." HTTP/1.1\r\nHost: ".$_SERVER['SERVER_NAME']."\r\nConnection: close\r\n\r\n";
		fputs ($fp, $get_data);
		
		$string = '';
		while (!feof($fp))
		{
			$string .= fgets($fp,128);
		}
		fclose($fp);
		
		if( preg_match('/^HTTP\/1.1 403 Forbidden/', $string)	// string starts with "HTTP/1.1 403 Forbidden"
			OR strpos($string, 'Error 403')!==false				// string contains "Error 403"
			OR strpos($string, '403 Forbidden') !== false)		// string contains "403 Forbidden"
		{
			return 1;
		}
		elseif( preg_match('/^HTTP\/1.1 200 OK/', $string) )	// string starts with "HTTP/1.1 200 OK"
		{
			return 2;
		}
		else
		{
			return 3;
		}
	}
}

?>