<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

if(!defined('LINPHA_DIR')) { define('LINPHA_DIR','..'); }

/**
 * linpha installer
 * @package Installation
 */
include_once(LINPHA_DIR.'/lib/include/common.php');
include_once(LINPHA_DIR.'/install/header.php');
include_once(LINPHA_DIR.'/install/lib.requirements.php');
include_once(LINPHA_DIR.'/install/header_html.php');
?>

<script language="JavaScript" type='text/javascript'>
<!--
function show_hide_tables(show_hide,max_lines)
{
	for(var i = 1; i <= max_lines; i++)
	{
		if(document.getElementById('table_line_' + i))
		{
			document.getElementById('table_line_' + i).style.display = show_hide;
		}
	}
}

function switch_visibility(line_nr)
{
	if(document.getElementById('table_line_' + line_nr).style.display == 'none')
	{
		document.getElementById('table_line_' + line_nr).style.display = '';
	}
	else
	{
		document.getElementById('table_line_' + line_nr).style.display = 'none';
	}
}
//-->
</script>

<?php
print_head();

/**
 * initialize table line variable
 */
$table_lines = 0;

/**
 * checking php version
 */
 	print_line_left("PHP Version");
 	print_line_middle(PHP_VERSION);
 	if(version_compare(PHP_VERSION, "5.0.0", ">="))
 	{
 		print_line_right(success_msg());
 	}
 	else
 	{
 		print_line_right(failed_msg());
	 	print_line_info("PHP Version >= 5.0.0 is required");
 		$error = 1;
 	}


/**
 * check for supported databases
 */
	print_line_left("Supported Databases (MySQL, PostgreSQL, SQLite)");
	if(function_exists('mysql_connect')) {
		$supported_db['mysql'] = 1;
	}
	if(function_exists('pg_connect')) {
		$supported_db['postgres'] = 1;
	}
	if(function_exists('sqlite_open')) {
		$supported_db['sqlite'] = 1;
	}

	if(!isset($supported_db))
	{
		print_line_middle("NONE");
		print_line_right(failed_msg());
		print_line_info("Enable at least support for one Database in " .
				"your PHP configuration file (php.ini)");
		$error = 1;
	}
	else
	{
		print_line_middle("Yes");
		print_line_right(success_msg());

		$str = "";
		foreach($supported_db AS $key=>$value)
		{
			$str .= $key.' ';
		}
		print_line_info("Supported Databases: ".$str);
	}

/**
 * checking imagemagick
 */
	print_line_left("ImageMagick");
	
	list($convert_avail, $convert_path, $convert_version) = installCheckConvert();
	if($convert_avail)
	{
	
	    /**
	     * the imagemagick  versions 6.1.1 - 6.1.3 contains a bug with the switch
	     * +profile "*", the "*" is taken as a wildcard and produces error messages
	     * like this: https://sourceforge.net/tracker/index.php?func=detail&aid=1073728&group_id=64772&atid=508614
	     *
	     * i tested these versions:
	     * 6.1.0	-	6.1.0-9
		 * 6.1.1	x	6.1.1-6
		 * 6.1.2	x	6.1.2-7
	     * 6.1.3	x	6.1.3-7
	     * 6.1.4	-	6.1.4-5
	     * 6.1.5	-	6.1.5-6
	     */
	    print_line_middle($convert_version);
	    
	    if($convert_version == "6.1.1" OR $convert_version == "6.1.2" OR $convert_version == "6.1.3")
	    {
	    	print_line_right(warning_msg());
	    	print_line_info("ImageMagick Versions 6.1.1 to 6.1.3 contain a bug " .
	    			"which makes it unusable. Please consider using another " .
	    			"Version or make use of GD library");
	    }
	    else
	    {
	    	print_line_right(success_msg());
	    	$convert_ok = true;
			
			/**
			 * Check if ImageMagick contains support for brackets
			 */
				/**
				 * this check occurs three times (2x in upgrade.php, 1x in sec_stage_install.
				 * php)
				 * 
				 * since version 6.1.3 (6.1.1 - 6.1.3 are unsupported anyway)
				 * 
				 * copy from http://redux.imagemagick.org/discussion-server/viewtopic.php?
				 * t=4701:
				 * since imagemagick 6.1 the resizing isn't working the same as it was with 6.0
				for example, i need to resize a big photo to 640x480, and want composite a watermark image
				i've done this with this command:
				      composite -size "640x480" "wm.jpg" "orig.jpg" -resize "640x480" "output.jpg"
				but since version 6.1 the watermark image (wm.jpg) is resized to 640x480 too, but it shouldn't
				is this a bug or a feature?
				i've tested this versions:
				6.0.8 watermark image isn't resized
				6.1.9 watermark image is resized
				6.2.3 watermark image is resized
				
				--------- Answer -------------
				This is a feature. composite is doing things to images, as the option appears.
				However surroungin orig.jpg and -resize operations with parenthesis should do the trick.
				
				----------
				but version below 6.1.3 doesn't support the "parenthesis", so we have to check which version we are using
				
				another solution would be to first resize the photo, save it to a file and then composite the watermark image
				over, but this would be much more dirty as the version depended commands because the function build_watermark_str()
				generates only the command, and doesn't execute anything this time, so we would have to change the whole procedure...
				 */
				if(version_compare($convert_version, "6.1.4", ">="))
				{
			    	$sys_im_bracket_support = 1;
			    } else {
			    	$sys_im_bracket_support = 0;
			    }
			    echo "\t".'<input type="hidden" name="sys_im_bracket_support" value="'.$sys_im_bracket_support.'">'."\n";
	    
	    	echo "\t".'<input type="hidden" name="sys_im_use_convert" value="1">'."\n";
	    	echo "\t".'<input type="hidden" name="sys_im_convert_path" value="'.$convert_path.'">'."\n";
	    }
	}
	else
	{
		print_line_right(warning_msg());
		print_line_info("ImageMagick was NOT found on this server. " .
				"This will result in lower quality thumbnails");
	}



/**
 * check GD only if IM not available
 */
	if(! isset($convert_ok))
	{
		print_line_left("GD Library");
		if(extension_loaded('gd'))
		{
			print_line_middle("Installed");
			print_line_right(success_msg());
		}
		else
		{
			print_line_middle("Not Found");
			print_line_right(failed_msg());
			$error = 1;
		}

		print_line_left("GD JPEG Support");
		if(!function_exists('imagecreatefromjpeg') || !function_exists('imagejpeg'))
		{
			print_line_middle("No");
			print_line_right(failed_msg());
			print_line_info("Your GD Library lacks JPEG support, " .
					"LinPHA will not support JPEG Images")."!";
			
			$error = 1;
		}
		else
		{
			print_line_middle("Yes");
			print_line_right(success_msg());
		}
	}

/**
 * checking posix extension
 * only if we are we are on unix
 */
	if( LIN_OS != "win" )
	{
		print_line_left("Posix Extension");
		if(installCheckPosix())
		{
			print_line_middle("Yes");
			print_line_right(success_msg());
		}
		else
		{
			print_line_middle("No");
			print_line_right(warning_msg());
			print_line_info("Missing POSIX Extension");
		}
	}

/**
 * checking safe mode
 */
	print_line_left("Safe Mode Activated");
	if(! ini_get('safe_mode'))
	{
		print_line_middle("No");
		print_line_right(success_msg());
	}
	else
	{
		print_line_middle("Yes");
		print_line_right(warning_msg());
		print_line_info("Safe mode is activated - Bad fun - " .
				"Some things may not work as expected");
	}

/**
 * checking open basedir
 */
	print_line_left("Open Basedir Activated");
	$basedir_name=ini_get("open_basedir");
	if(empty($basedir_name))
	{
		print_line_middle("No");
		print_line_right(success_msg());
	}
	else
	{
		print_line_middle("Yes");
		print_line_right(warning_msg());
		print_line_info("Open Basedir restrictions in effekt - " .
				"Some things may not work as expected");
	}


/**
* memory limit
*/
	print_line_left("PHP Memory limit");
	$limit = installGetMemLimit();
    if($limit === false)
    {
		print_line_middle("Unlimited");
		print_line_right(success_msg());
    }
	elseif($limit <= 12)
	{
		print_line_middle($limit.' MB');
		print_line_right(warning_msg());
		print_line_info("Your PHP memory limit is probably to low. If you " .
				"encounter problems with creating thumbnails, try to increase " .
				"the memory_limit in php.ini or resize your pictures to a " .
				"lower resolution and try again.");
	}
	else
	{
		print_line_middle($limit.' MB');
		print_line_right(success_msg());
	}

/**
 * checking session variables
 */
	print_line_left("Session Save Path Set Correctly");

	$session_path=session_save_path();
	if(!empty($session_path))
	{
		print_line_middle("Yes");
		print_line_right(success_msg());
		print_line_info("Session Save Path set to: ".$session_path);
	}
	else
	{
		print_line_middle("No");
		print_line_right(warning_msg());
		print_line_info("Session Save Path not set - Please set in " .
				"php.ini first.");
	}

	print_line_left("Session Save Handler Set Correctly");

	$session_save_handler = strtolower(ini_get('session.save_handler'));
	if($session_save_handler == 'files')
	{
		print_line_middle("Yes");
		print_line_right(success_msg());
	}
	else
	{
		print_line_middle("No");
		print_line_right(failed_msg());
		print_line_info("Session save handler NOT set correctly.<br />".
			"Open php.ini and change session.save_handler value " .
			"to \"files\"");
	}
/**
 * checking file permissions
 */
	print_line_left("File Permissions Set Correctly");
	
	if( is_writable(LINPHA_DIR.'/var') )
	{
		print_line_middle("Yes");
		print_line_right(success_msg());
	}
	else
	{
		print_line_middle("No");
		print_line_right(failed_msg());
		print_line_info("Please make sure the directory /linpha/var/ is writeable"); 
		
		$error = 1;
	}

/**
 * checking album and var folder are protected from outside
 */
	print_line_left("Album And System Folder Protected From Outside");
	
	$ret = installCheckProtectedFromOutside();
	if( $ret == 1 )
	{
		print_line_middle("Yes");
		print_line_right(success_msg());
		print_line_info("The /albums and /var folders are protected from outside. See <a href=\"http://linpha.sourceforge.net/wiki/index.php/Secure_LinPHA\">here</a> for more details.");
	}
	elseif( $ret == 2 )
	{
		print_line_middle("No");
		print_line_right(warning_msg());
		print_line_info("The /albums and /var folders are not protected from outside! Please see <a href=\"http://linpha.sourceforge.net/wiki/index.php/Secure_LinPHA\">here</a> for more details."); 
	}
	elseif( $ret == 3 )
	{
		print_line_middle("Unknown");
		print_line_right(warning_msg());
		print_line_info("Cannot determine wether the /albums and /var folders are protected from outside! Please see <a href=\"http://linpha.sourceforge.net/wiki/index.php/Secure_LinPHA\">here</a> for more details."); 
	}


/**
 * show footer of table requirements
 */
print_foot();

echo "<br />";
echo tr("For more detailed info - Please click on the status info text above")."<br />";

?>
<a href="#" onclick="show_hide_tables('',<?php echo $table_lines; ?>)">
(<?php echo tr("Show All Information"); ?></a>).

<script language="JavaScript" type='text/javascript'>
<!--
// hide all info tables on startup
show_hide_tables('none',<?php echo $table_lines; ?>);
//-->
</script>

<input type="hidden" name="language" value="<?php echo isset($_POST['language']) ? $_POST['language'] : ''; ?>">

<br /><br />

<?php

if(isset($error))
{
	echo "<h2>".tr("!!! INSTALLATION ABORTED !!!")."</h2><br />";
	echo tr("There was at least one serious error - continue at your own risk");
	echo "<br /><br />";
}

include_once(LINPHA_DIR.'/install/footer.php');

function print_head()
{
?>

<div align="center">
<table width="95%">
	<tr>
		<td>&nbsp;</td>
		<td><?php echo tr("Result"); ?></td>
		<td><?php echo tr("Status"); ?></td>
	</tr>
<?php
}

function print_line_left($str)
{
	$GLOBALS['table_lines']++;
?>
	<tr>
		<td style="text-align: left;"><?php echo tr("$str"); ?></td>
<?php
}

function print_line_middle($str)
{
?>
		<td width="50"><?php echo tr("$str"); ?></td>
<?php
}

function print_line_right($str)
{
?>
		<td width="50"><a href="#" onclick="switch_visibility(<?php echo $GLOBALS['table_lines']; ?>)"><?php
			echo tr("$str"); ?></a></td>
	</tr>
<?php
}

function print_line_info($str)
{
?>
	<tr id="table_line_<?php echo $GLOBALS['table_lines']; ?>" style="display:">
		<td colspan="3" style="text-align: left; border-left: none; border-right: none; border-bottom: none; ">
		<?php echo tr("$str"); ?><br /><br /></td>
	</tr>
<?php
}

function print_foot()
{
?>
</table>
</div>
<?php
}
?>
