<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

if(!defined('LINPHA_DIR')) { define('LINPHA_DIR','..'); }

/**
 * linpha installer
 * @package Installation
 */
include_once(LINPHA_DIR.'/lib/include/common.php');
include_once(LINPHA_DIR.'/install/header.php');
include_once(LINPHA_DIR.'/install/header_html.php');

/**
 * prevent php notices, as sqlite doesn't know about ports'
 */
if(!isset($_SESSION['sql_dbport']))
{
	$_SESSION['sql_dbport'] = "0";
}

/**
 * save settings from previous page
 */
	if(isset($_POST['sql_prefix']))
	{
		$_SESSION['sql_prefix'] = $_POST['sql_prefix'];
	}

/**
 * validate settings
 */
	if( empty($_SESSION['sql_prefix']) )
	{
		echo tr("Validating Information")."...  ";
		echo failed_msg();
		echo "<br />".tr("Please Fill In All Fields")."";
		echo "</form>";
		$show_next_button = false;
		include_once(LINPHA_DIR.'/install/footer.php');
		exit(1);
	}

/**
 * switching to AdoDB
 * connect to db and check if connection was succesfull
 */  
	echo tr("Switching To AdoDB")."... ";
	$_SESSION['installmode1'] = true;
	$_SESSION['installmode2'] = true;

	include_once(LINPHA_DIR.'/lib/classes/linpha.class.php');
	
	$linpha = new Linpha();
	$linpha->sql->dbConnect();
	
	echo success_msg();


echo '<br /><br /><hr /><br />';

/**
 * checking prefix if not any table already exists
 */
if($_SESSION['sql_dbtype'] != "sqlite")
{
	?>
	<h2><?php echo tr("Checking Table Prefix"); ?></h2><br />
	
	<?php echo tr("Checking For Existing Tables")."... "; ?>
	<?php
	$array_tables = $linpha->db->MetaTables(); 
	  
	if (! isset($array_tables) OR ! is_array($array_tables))
	{
		echo failed_msg();
  		echo "<br />".tr("Cannot Read Tables").": ".mysql_error();
  		$error_nr = 2;
	}
	else
	{
		include_once(LINPHA_DIR.'/install/sql/sql.tables.php');
		
		/**
		 * add prefix
		 */
		foreach($linpha_tables AS $key=>$value)
		{
			$linpha_tables[$key] = $_SESSION['sql_prefix'].$value;
		}
		
		$result = array_intersect ($linpha_tables, $array_tables);
		
		if(isset($result) && is_array($result) && (count($result) > 0) )
		{
			echo failed_msg();
			echo "<br />".tr("Table Prefix Already Exists, Please Choose Another One").".!";
			echo "</form>";
			$show_next_button = false;
			include_once(LINPHA_DIR.'/install/footer.php');
			exit(1);
		}
		else
		{
			echo success_msg();
		}
 	}
 	echo '<br /><hr /><br />';
}

/**
 * try to create a new db user
 */
$n_runs = 1;
if($_SESSION['sql_dbtype'] == "mysql")
{
	echo tr("Trying To Create New LinPHA Database User")."... ";
	$random_pass = random_password(8);
	$random_user = "linpha_".random_password(4);

	if($_SESSION['sql_hostname'] == 'localhost' OR $_SESSION['sql_hostname'] == '127.0.0.1')
	{
		$local_host = 'localhost';
	} else {
		$local_host = '%';
	}
	if(! $linpha->db->Execute("GRANT ALL ON ".$_SESSION['sql_dbname'].".* TO '".
				$random_user."'@'".$local_host."' ".
				"IDENTIFIED BY '".$random_pass."' WITH GRANT OPTION") )
	{
		echo warning_msg();
		echo "<br />".tr("Failed To Create New LinPHA User");
		echo "<br />".tr("Seems you are not allowed to create new Database users - " .
				"using given defaults (NOTE: Installation should still work)")."... ";
	}
	else
	{
		if(! $linpha->db->Execute("FLUSH PRIVILEGES") )
		{
			echo warning_msg();
		echo "<br />".tr("Failed To Create New LinPHA User");
		echo "<br />".tr("Seems you are not allowed to create new Database users - " .
				"using given defaults (NOTE: Installation should still work)")."... ";
		}
		else
		{
			/**
			 * close connection and connect with just created user
			 */
			$test_conn = NewADOConnection($_SESSION['sql_dbtype']);
			if(! $test_conn->Connect($_SESSION['sql_hostname'].":".$_SESSION['sql_dbport'], $random_user, $random_pass, $_SESSION['sql_dbname']) )
			{
				echo warning_msg();
				echo "<br />".tr("Failed To Create New LinPHA User");
				echo "<br />".tr("Seems you are not allowed to create new Database users - " .
				"using given defaults (NOTE: Installation should still work)")."... ";
			}
			else
			{
				/**
				 * backup other connection and replace with new one
				 */
				echo success_msg();
				$n_runs = 2;				
			}
		}
	}
	echo '<br /><br /><hr /><br />';
}

/**
 * running tests
 */
/**
 * prevent test running sqlite db
 * @todo  why?? -flo 
 * @todo - because at least alter will fail ;-) - bzrudi
 */	

if($_SESSION['sql_dbtype'] != 'sqlite')
{

for($i = 1; $i <= $n_runs; $i++)
{
	/**
	 * trying new user...
	 */
	if($i == 2)
	{
		echo "<br /><br />".tr("User Created, Now Running Tests")."...<br /><br />";
		$orig_conn = $linpha->db;
		$linpha->db = $test_conn;
	}

	/**
	 * Test table stuff
	 */
	echo "<h2>".tr("Testing Tables")."</h2>";
	$table_name = $_SESSION['sql_prefix']."test".time();

	echo "<br />".tr("Running Tests").": ";

	$array_tests = array(
		"CREATE" => "CREATE TABLE ".$table_name." ( id INT NOT NULL )",
		"INSERT" => "INSERT into ".$table_name." ( id  ) VALUES ( '5' )",
		"ALTER" => "ALTER TABLE ".$table_name." ADD md5sum VARCHAR(32)",
		"UPDATE" => "UPDATE ".$table_name." SET md5sum = '".md5('the life ist beautiful')."'",
		"SELECT" => "SELECT * FROM ".$table_name,
		"DELETE" => "DELETE FROM ".$table_name,
		"DROP" => "DROP TABLE ".$table_name	);
	
	foreach($array_tests AS $key=>$value)
	{

		echo " ".$key." ";
		$result = $linpha->db->Execute($value); 

		/**
		 * need special check with "SELECT"
		 */
		$error = false;
		if($key == "SELECT")
		{
			$data = $result->FetchRow(ADODB_FETCH_NUM);
			if( !$data OR @$data[1] != md5('the life ist beautiful'))
			{
				$error = true;
			}
		}
		else
		{
			if(!$result)
			{
				$error = true;
			}
		}
		
		/**
		 * something went wrong
		 */
		if($error)
		{
			echo failed_msg();
			echo '<br />'.$linpha->db->ErrorMsg();

			/**
			 * try to delete the temporary table to not leave tmp tables in
			 * database
			 */
			$linpha->db->Execute($array_tests['DROP']);

			if($i == 2)
			{
				$use_other_conn = true;
				break;
			}
			else
			{
				echo "</form>";
				$show_next_button = false;
				include_once(LINPHA_DIR.'/install/footer.php');
				exit(1);
			}
		}
		else
		{
			echo success_msg();
		}
	}
}
}
else
{	
	/**
	 * skip tests for sqlite
	 */	
	echo tr("Skipping Database Tests");
}
/**
 * new user failed, use old connection
 */
	if(isset($use_other_conn))
	{
		echo "<br /><br /><b>".tr("New User Tests Failed, Falling Back")."...</b>";
		$linpha->db = $orig_conn;
	}
	elseif($n_runs == 2)
	{
		$_SESSION['sql_username'] = $random_user;
		$_SESSION['sql_password'] = $random_pass;
	}

echo '<br /><br />';
include_once(LINPHA_DIR.'/install/footer.php');
?>