<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * Create watermark image with imagemagick
 * @package Image
 */

if(!defined('LINPHA_DIR')) { exit(1); }

class LinGDLib
{
public $imgType, $orgWidth, $orgHeight, $imgQuality;
public $img_thumbsize;

public $wm_config=NULL;

function createImage($input,$output,$img_width,$img_height,$rotate,$watermark)
{
	/**
	 * create image resources
	 */
		$src_image =& LinImage::GDimageCreateFrom($input,$this->imgType);
		$scaled_image = imagecreatetruecolor($img_width, $img_height);
	
	/**
	 * resize and crop image
	 */
		imagecopyresized(
				$scaled_image,$src_image,
				0,0,					//  int dstX, int dstY
				0,0,					//	int srcX, int srcY
				$img_width,$img_height,		//	int dstW, int dstH
				$this->orgWidth,$this->orgHeight		//	int srcW, int srcH
		);
	
	/**
	 * destroy source image
	 */
		imagedestroy($src_image);

	if( $watermark ) {
		$this->watermarkGD($scaled_image,$img_width,$img_height,$this->wm_config);
	}

	/**
	 * rotate image
	 */
		LinImage::GDimageRotate($scaled_image,$rotate);
	
	/**
	 * save image
	 */
		imagejpeg($scaled_image,$output,$this->imgQuality);
	
	/**
	 * destroy scaled image
	 */
		imagedestroy($scaled_image);
}

function createThumbnail($input,$output,$rotate)
{
	/**
	 * calculate crop
	 */
		$ratio = $this->orgWidth/$this->orgHeight;
		if($ratio==1)
		{
			$x = 0;
			$y = 0;
			
			$src_size = $this->orgWidth; // or $this->orgHeight, here it doesnt matter
		}
		elseif($ratio>1) // orgWidth > orgHeight
		{
			$x = ( $this->orgWidth - $this->orgHeight )/2;
			$y = 0;
	
			$src_size = $this->orgHeight; // i think it does matter...
		}
		elseif($ratio<1) // orgHeight > orgWidth
		{
			$y = ( $this->orgHeight - $this->orgWidth )/2;
			$x = 0;
			
			$src_size = $this->orgWidth; // i think it does matter...
		}
	
		$w = $this->img_thumbsize;
		$h = $this->img_thumbsize;
	
	/**
	 * create image resources
	 */
		$src_image =& LinImage::GDimageCreateFrom($input,$this->imgType);
	
		$scaled_image = imagecreatetruecolor($this->img_thumbsize, $this->img_thumbsize);
	
	/**
	 * resize and crop image
	 */
		imagecopyresized(
				$scaled_image,$src_image,
				0,0,					//  int dstX, int dstY
				$x,$y,					//	int srcX, int srcY
				$w,$h,					//	int dstW, int dstH
				$src_size,$src_size		//	int srcW, int srcH
		);
	
	/**
	 * destroy source image
	 */
		imagedestroy($src_image);
	
	/**
	 * rotate image
	 */
		LinImage::GDimageRotate($scaled_image,$rotate);
	
	/**
	 * save image
	 */
		imagejpeg($scaled_image,$output,$this->imgQuality);
	
	/**
	 * destroy scaled image
	 */
		imagedestroy($scaled_image);
}

function createThumbnailFolder($output,$array_thumbs)
{
	if( !function_exists('imagecreatefromgif')
		OR ! $im = @imagecreatefromgif (LINPHA_DIR.'/lib/graphics/thumb_folder.gif'))
	{
		/**
		 * on failure, do nothing
		 * -> if the output file doesnt exists, we will take a default file
		 * 
		 * //copy( $array_thumbs[0]['file'], $this->output_file );
		 */
		linLog(LOG_TYPE_IMPORT,LOG_ERR,'thumbnail',"Cannot create folder thumbnail");
	}
	else
	{
		for($i=1; list($key,$value) = each($array_thumbs); $i++ )
		{
			//echo $value['file'].' '.$value['type'].'<br />';
			$im2 =& LinImage::GDimageCreateFrom($value['file'],$value['type']);
			imagetruecolortopalette ( $im2, true, 256 );
			$src_width = ImageSX($im2);
			$src_height = ImageSY($im2);
			
			switch($i)
			{
			case 1:
				imagecopyresized(
						$im,$im2,
						15,15,					//  int dstX, int dstY
						0,0,					//	int srcX, int srcY
						80,80,					//	int dstW, int dstH
						$src_width,$src_height		//	int srcW, int srcH
				);
			break;
			case 2:
				imagecopyresized(
						$im,$im2,
						105,15,					//  int dstX, int dstY
						0,0,					//	int srcX, int srcY
						80,80,					//	int dstW, int dstH
						$src_width,$src_height		//	int srcW, int srcH
				);
			break;
			case 3:
				imagecopyresized(
						$im,$im2,
						15,105,					//  int dstX, int dstY
						0,0,					//	int srcX, int srcY
						80,80,					//	int dstW, int dstH
						$src_width,$src_height		//	int srcW, int srcH
				);
			break;
			case 4:
				imagecopyresized(
						$im,$im2,
						105,105,					//  int dstX, int dstY
						0,0,					//	int srcX, int srcY
						80,80,					//	int dstW, int dstH
						$src_width,$src_height		//	int srcW, int srcH
				);
			break;
			}
			
			imagedestroy($im2);
		}
		
		if (function_exists("imagegif"))
		{
			imagegif($im,$output);
		}
		elseif (function_exists("imagejpeg")) {
			imagejpeg($im, $output, 0.5);
			linLog(LOG_TYPE_IMPORT,LOG_ERR,'thumbnail',"Can only create a jpeg folder thumbnail because the php function 'imagegif' doesnt exists. Please enable the gdlib extension or update your php version.");
		}
		else
		{
			linLog(LOG_TYPE_IMPORT,LOG_ERR,'thumbnail',"Cannot create folder thumbnail because the php function 'imagegif' doesnt exists. Please enable the gdlib extension or update your php version.");
		}
		
		imagedestroy($im);
	}

}

/**
 * not supported with the gdlib, sorry...
 */
function createThumbnailVideo()
{
	return 0;
}

function watermarkGD(&$scaled_image,$w,$h,$wm_config=NULL)
{
	if($wm_config == NULL) {
		$wm_config = read_watermark();
	} 

	if($wm_config['wm_watermark']!=0)
	{
		switch($wm_config['wm_watermark'])
		{
		case 1:	//#### Text ####

			//## Font heigth and width
			$font_height = imagefontheight($wm_config['wm_fontsize']);
			$font_width = imagefontwidth($wm_config['wm_fontsize']);
			$string_width = strlen($wm_config['wm_text'])*$font_width;
			$string_height = $font_height;

			//## Create image
			$wm_img = imagecreate($wm_config['wm_width'], $wm_config['wm_height']);

			//## Colors
			$array_rgb = LinWatermark::getRgbFromAll($wm_config['wm_shadow_color']);
			$shadow_color  = ImageColorAllocate($wm_img, $array_rgb['r'], $array_rgb['g'], $array_rgb['b']);
			$array_rgb = LinWatermark::getRgbFromAll($wm_config['wm_fontcolor']);
			$text_color  = ImageColorAllocate($wm_img, $array_rgb['r'], $array_rgb['g'], $array_rgb['b']);

			//## Rectangle
			if($wm_config['wm_enable_rectangle']) {
				$array_rgb = LinWatermark::getRgbFromAll($wm_config['wm_rectangle_color']);
				$rect_col  = ImageColorAllocate($wm_img, $array_rgb['r'], $array_rgb['g'], $array_rgb['b']);
				imagefill($wm_img, 0, 0, $rect_col);
			}

			//## Text
			$pos_x = ($wm_config['wm_width']-$string_width)/2;
			$pos_y = ($wm_config['wm_height']-$string_height)/2;
			imagestring($wm_img, $wm_config['wm_shadow_fontsize'], $pos_x, $pos_y, $wm_config['wm_text'], $shadow_color);
			imagestring($wm_img, $wm_config['wm_fontsize'], $pos_x+($wm_config['wm_shadow_size']), $pos_y+($wm_config['wm_shadow_size']), $wm_config['wm_text'], $text_color);

			//## Position on the big image
			$src_w = $wm_config['wm_width'];
			$src_h = $wm_config['wm_height'];
		break;
		case 2:	//#### Image ####
			
			if(file_exists($wm_config['wm_img_img']))	// needed for creating video thumbnails
			{
				$wm_file = $wm_config['wm_img_img'];
			}
			else
			{
				$wm_file = LINPHA_DIR.'/lib/plugins/watermark/watermarks/'.$wm_config['wm_img_img'];
			}
			
			list($wm_img_width,$wm_img_height,$wm_img_type) = LinIdentify::linGetImageSize($wm_file);
			
			$wm_img = & LinImage::GDimageCreateFrom($wm_file, $wm_img_type);

			$src_w = $wm_img_width;
			$src_h = $wm_img_height;
		break;
		}

		/**
		 * Scale watermark always to X% of image size
		 * this will not look very nice with transparent images
		 */
		if($wm_config['wm_resize']!='no') {
			$arr_resized = LinWatermark::wmArrResized($w,$h,$src_w,$src_h,$wm_config['wm_resize']);

			$resized_image=imagecreate($arr_resized['w'],$arr_resized['h']); 

			imagecopyresized($resized_image,$wm_img,0,0,0,0,
				$arr_resized['w'],$arr_resized['h'],$src_w,$src_h);
			$src_w = $arr_resized['w'];
			$src_h = $arr_resized['h'];

			// Overwrite $wm_img with the resized image
			$wm_img = $resized_image;
		}
		
		/**
		 * Merge watermark image into scaled image
		 */
		$src_x = 0; $src_y = 0;
		$arr_pos = LinWatermark::calcAlign($wm_config['wm_align'],$src_w,$src_h, $w, $h, -$wm_config['wm_horizontal'], -$wm_config['wm_vertical']);
		imagecopymerge($scaled_image, $wm_img, $arr_pos['x'], $arr_pos['y'], $src_x, $src_y, $src_w, $src_h, $wm_config['wm_dissolve']);
	}
}

}
?>