<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * Create watermark image with imagemagick
 * @package Image
 */

if(!defined('LINPHA_DIR')) { exit(1); }

class LinImageMagick
{
public $imagemagickPath;

public $orgWidth, $orgHeight, $imgQuality;
public $img_thumbsize;

public $wm_config=NULL;

function createImage($input,$output,$img_width,$img_height,$rotate,$watermark)
{
	if( $watermark )
	{
		$convert_str = $this->buildWatermarkStr($input,$output,$this->imgQuality,$img_width,$img_height,$this->wm_config);
	}
	else
	{
		$convert_str = $this->imagemagickPath.'convert'.
			' '.linEscapeString($input).'[0]'.	// [0] -> take only first frame!!!
			' -resize "'.$img_width.'x'.$img_height.'"'.	// new size
			' -quality '.$this->imgQuality.
			' +profile "*"'.		// delete all profiles
			' -colorspace RGB'.		// change colorspace always to RGB because all browsers only can display RGB images
			' -rotate '.$rotate.
			' '.linEscapeString($output);
	}
	
	//echo $convert_str;
	//exit();
	
	LinFilesys::linExec( $convert_str );
}

function createThumbnail($input,$output,$rotate)
{
	/**
	 * crop image
	 * first, scaled down image to $w x $h
	 * keep ratio:
	 * if image is wider than heigh, the height is $thumbsize, and width is "empty"
	 * (-> imagemagick will set the correct width to keep the ratio)
	 * after the resize, the image is too wide
	 * -> crop it
	 */
		$ratio = $this->orgWidth/$this->orgHeight;
		if($ratio==1)
		{
			$x = 0;
			$y = 0;
			$w = $this->img_thumbsize;
			$h = $this->img_thumbsize;
		}
		elseif($ratio>1)
		{
			$x = (int) round( ( ($this->img_thumbsize*$ratio) - $this->img_thumbsize )/2 );
			$y = 0;
			$w = '';
			$h = $this->img_thumbsize;
		}
		elseif($ratio<1)
		{
			$y = (int) round( ( $this->img_thumbsize - ($this->img_thumbsize*$ratio) )/2 );
			$x = 0;
			$w = $this->img_thumbsize;
			$h = '';
		}
	
	$convert_str = $this->imagemagickPath.'convert'.
		' '.linEscapeString( $input ).'[0]'.	// [0] -> take only first frame!!!
		' -thumbnail "'.$w.'x'.$h.'"'.	// new size and delete profiles, no need anymore for ' +profile "*"'. // delete all profiles
		' -quality '.$this->imgQuality.
		' -crop '.$this->img_thumbsize.'x'.$this->img_thumbsize.'+'.$x.'+'.$y.	// crop
		' -colorspace RGB'.		// change colorspace always to RGB because all browsers only can display RGB images
		' -rotate '.$rotate.
		' '.linEscapeString( $output );
	
	//echo $convert_str;
	LinFilesys::linExec( $convert_str );
}

function createThumbnailFolder($output,$array_thumbs)
{
	/*
	roundrectangle 1,1  122,92  15,15
	
	# create the old style folder thumb
	convert -size 182x182 xc:lemonchiffon -mattecolor peru -frame 9x9+3+3 \
			-quality 70 output_file
	
	# create a rounded corner folder thumb with shadow
	only works with png as output format
	but this cause a too high file size (>100KB)
	and in IE the png transparency is shown correctly
	see also http://www.cit.gu.edu.au/~anthony/graphics/imagick6/thumbnails/
	 
	convert thumbnail.gif -border 2 -matte -channel RGBA -threshold -1 \
	        -background none -fill none -stroke black -strokewidth 3 \
	        -draw "roundrectangle 1,1  122,92  15,15"    rounded_corner_overlay.png
	convert thumbnail.gif -border 2 -matte -channel RGBA -threshold -1 \
	        -background none -fill white -stroke black -strokewidth 1 \
	        -draw "roundrectangle 1,1  122,92  15,15"    rounded_corner_mask.png
	
	
	
	# create the rounded corner folder thumb
	   - open gimp
	   - create new empty image, 200x200 transparent background
	   - use Auswahl - Alles, Auswahl - Abgerundetes Rechteck 20%
	   - fill with black color
	   - use Auswahl - Verkleinern - 2 pixel
	   - fill with color #fefaca
	   - save as gif
	
	
	to composite the four images to the folder image, we could do this with only one command
	but there seems to be a bug in imagemagick 6.2.4 (used in ubuntu 6.06 lts, see also http://www.imagemagick.org/pipermail/magick-users/2006-June/017707.html) 
	so we have to deal with it...
	    convert -size 100x100 xc:skyblue \
	            balloon.gif  -geometry 40x40+5+10   -composite \
	            medical.gif  -geometry      +35+30  -composite \
	            present.gif  -geometry 24x24+62+50  -composite \
	            shading.gif  -geometry 16x16+10+55  -composite \
	            compose_resize.gif
	*/
	

	/**
	 * create new unique tmp folder
	 */
		$tmp_folder = LinFilesys::createTmpFolder('thumb_folder');
	
	/**
	 * create smaller thumbs and set up the convert string at the same time for later use
	 */
	
		$str = $this->imagemagickPath."convert ".LINPHA_DIR."/lib/graphics/thumb_folder.gif ";
		reset($array_thumbs);
		for($i=1; list($key,$value) = each($array_thumbs); $i++ )
		{
			switch($value['type'])	// thumbnails are either jpg (files) or gifs (folders)
			{
			case '1': $fileext = 'gif'; break;
			case '2': $fileext = 'jpg'; break;
			default: $fileext = 'jpg'; break;
			}
			LinFilesys::linExec( $this->imagemagickPath."convert ". $value['file'] . " -quality 70 -resize 80x80 " . $tmp_folder . "/" . $i . "." . $fileext );
	
			switch($i)
			{
				case 1:
					$str .= $tmp_folder."/1.".$fileext."  -geometry +15+15   -composite ";
				break;
				case 2:
					$str .= $tmp_folder."/2.".$fileext."  -geometry +105+15  -composite ";
				break;
				case 3:
					$str .= $tmp_folder."/3.".$fileext."  -geometry +15+105  -composite ";
				break;
				case 4:
					$str .= $tmp_folder."/4.".$fileext."  -geometry +105+105 -composite ";
				break;
			}
		}
	
		/**
		 * filesizes and time:
		 * with -colors 64: 18.32KB		1.1s
		 * with -colors 128: 22.29KB	1.07s
		 * with -colors 256: 27.19KB	3.9s	!!
		 * with -colors 512: 27.09KB	4.5s	!!
		 * 
		 * it seems that the -quality switch doesnt matter with gif images
		 */
		$str .= "-colors 512 ".$output;
		
		LinFilesys::linExec( $str );
		
	/**
	 * delete tmp folder
	 */
	 	reset($array_thumbs);
		for($i=1; list($key,$value) = each($array_thumbs) ; $i++)
		{
			switch($value['type'])	// thumbnails are either jpg (files) or gifs (folders)
			{
			case '1': $fileext = 'gif'; break;
			case '2': $fileext = 'jpg'; break;
			default: $fileext = 'jpg'; break;
			}
			unlink( $tmp_folder .'/'. $i . '.' . $fileext );
		}
		rmdir($tmp_folder);
	
	}

function createThumbnailVideo($input,$output,$dst_w,$dst_h)
{
	/**
	* redirect stderr to get all error messages to see if convert fails
	* on divx videos, convert outputs 'convert: AVI chunk vprp
	* not yet supported.' to stderr but exists with return value
	* 0
	*
	* "2>&1" produces no error under windows
	* 
	* $input[0] -> take only first frame!!!
	*/

	$convert_str = $this->imagemagickPath.'convert '.
		linEscapeString($input.'[0]').
		' -resize '.$dst_w.'x'.$dst_h.' '.
		linEscapeString($output).' 2>&1';

	$array_output = array(); $return_value = '';	// do not this inside exec() !!!!!!!!!! it will overwrite the returned content !!
	LinFilesys::linExec( $convert_str,$array_output,$return_value);
}

/**
 * 		'wm_active' => "0",
		'wm_watermark' => "0",
		'wm_text' => "Copyright 2004",
		'wm_font' => "",
		'wm_fontsize' => "20",
		'wm_fontcolor' => "white",
		'wm_align' => "southwest",
		'wm_horizontal' => "0",
		'wm_vertical' => "0",
		'wm_enable_shadow' => "1",
		'wm_shadow_size' => "1",
		'wm_shadow_fontsize' => "20",
		'wm_shadow_color' => "black",
		'wm_enable_rectangle' => "1",
		'wm_rectangle_color' => "darkgrey",
		'wm_height' => "30",
		'wm_width' => "145",
		'wm_img_img' => "",
		'wm_dissolve' => "75%",
		'wm_resize' => "30"
 */
function buildWatermarkStr($input,$output,$q,$w,$h,$wm_config=NULL)
{
	$convert_path = $GLOBALS['linpha']->sql->config->value['sys_im_imagemagick_path'];

	if($wm_config == NULL) {
		$wm_config = LinWatermark::readWatermark();
	} 

	$align_str = ' -gravity '.$wm_config['wm_align'];
	$dissolve_str = ' -dissolve '.$wm_config['wm_dissolve'];
	
	/**
	 * check wether brackets are needed or not
	 * details see sec_stage_install.php
	 */
	if($GLOBALS['linpha']->sql->config->value['sys_im_bracket_support'])
	{
		if( LIN_OS == "win" ) {
			$bracket_escape = '';
		} else {
			$bracket_escape = '\\';
		}

		$bracket_start = $bracket_escape.'( ';
		$bracket_end = ' '.$bracket_escape.')';
	} else {
		$bracket_start = '';
		$bracket_end = '';
	}
	
	switch($wm_config['wm_watermark'])
	{
	/*case 0:	//#### Disabled ####

		// if making any changes, change it in the function convert_image_convert() too!
		$convert_str = $convert_path.'convert'.
			' -quality '.$q.
			' -size "'.$w.'x'.$h.'"'.	// new size
			' '.linEscapeString($input).'[0]'.	// [0] -> take only first frame!!!
			' -resize "'.$w.'x'.$h.'"'.	// new size
			' +profile "*"'.		// delete all profiles
			' -colorspace RGB'.		// change colorspace always to RGB because all browsers only can display RGB images
			' '.linEscapeString($output);
		
	break;*/
	case 1:	//#### Text #####
	
		//## Rectangle
		if($wm_config['wm_enable_rectangle']==1) {
			$rect_str = ' -fill "'.$wm_config['wm_rectangle_color'].'"'.					// rectangle color
				' -draw "rectangle 0,0,'.$wm_config['wm_width'].','.$wm_config['wm_height'].'"';	// rectangle itself
		} else {
			$rect_str = "";
		}
		
		//## Font
		if($wm_config['wm_font']=="no") {	
			$font_str = '';		// no ttf files found, try without setting the font parameter (on my linux server it has produced an error, on windows it has worked...)
		} else {
			$font_str = ' -font '.LINPHA_DIR.'"/lib/plugins/watermark/font/'.$wm_config['wm_font'].'.ttf"';
		}
		
		//## Text and Shadow
		$fontsize_str = ' -pointsize '.$wm_config['wm_fontsize'];

			// normal or no shadow
		if($wm_config['wm_enable_shadow'] == 1)
		{
			$draw_str = ' -pointsize '.$wm_config['wm_shadow_fontsize'].	// Shadow fontsize
				' -draw "fill '.$wm_config['wm_shadow_color'].		// Shadow color
				' text 0,0 \''.$wm_config['wm_text'].'\'"'.		// Shadow itself
				$fontsize_str.						// Text fontsize
				' -draw "fill '.$wm_config['wm_fontcolor'].		// Text color
				' text '.$wm_config['wm_shadow_size'].','.$wm_config['wm_shadow_size'].' \''.$wm_config['wm_text'].'\'"';	// Text itself
		}
			// extended shadow
		elseif($wm_config['wm_enable_shadow'] == 2)
		{
			$draw_str = $fontsize_str.					// Text fontsize
				' -stroke '.$wm_config['wm_shadow_color'].		// Shadow color
				' -strokewidth '.$wm_config['wm_shadow_size'].		// Shadow fontsize
				' -draw "text 0,0 \''.$wm_config['wm_text'].'\'"'.	// Shadow itself
				' -gaussian 0x3 -stroke none'.				// Extended Shadow
				' -fill '.$wm_config['wm_fontcolor'].			// Text color
				' -draw "text 0,0 \''.$wm_config['wm_text'].'\'"';	// Text itself
		}

		//## Scale watermark always to X% of image size - Stuff
		if($wm_config['wm_resize']!="no") {
			$arr_resized = LinWatermark::wmArrResized($w,$h,$wm_config['wm_width'],$wm_config['wm_height'],$wm_config['wm_resize']);
			
			$convert_str_part1 = $convert_path.'convert'.
				' -quality 100'.
				' "-"'.
				' -resize "'.$arr_resized['w'].'x'.$arr_resized['h'].'"'.
				'  miff:- | ';
		} else {
			$convert_str_part1 = '';
		}

		//## The magic convert - stuff
		$convert_str = $convert_path.'convert'.
			' -quality 100'.		// quality 100, this is only for the small rectangle
			$font_str.
			' -size '.$wm_config['wm_width'].'x'.$wm_config['wm_height'].
			' xc:none'.			// ? (this slows down watermark..?! (maybe))
			$rect_str.			// if rectangle is enabled, it will be shown here
			' -gravity center'.		// align of text in the smell rectangle is always center
			$draw_str.			// draw the text with shadow
			' miff:- | '.			// not to output, but to pipe '|' so that is for the input for the program 'composite' which is also part of the convert program, this program can merge two images

			$convert_str_part1.		// Scale watermark always to X% of image size - Stuff

			$convert_path.'composite'.
			' -quality '.$q.		// here is the real quality string
			' -size "'.$w.'x'.$h.'"'.	// new size
			$dissolve_str.
			$align_str.
			' -geometry +'.$wm_config['wm_horizontal'].'+'.$wm_config['wm_vertical'].	// adjust the image with the specified paramters
			' -type TruecolorMatte'.
			' "-" '.			// watermark image "input from pipe"
			' '.$bracket_start.linEscapeString($input).'[0]'.	// [0] -> take only first frame!!!	// input file
			' -resize "'.$w.'x'.$h.'"'.$bracket_end.	// new size (the values are the same like $size_str)
			' +profile "*"'.		// delete all profiles (-strip isn't available in imagemagick < 6.0)
			' -colorspace RGB'.		// change colorspace always to RGB because all browsers only can display RGB images
			' '.linEscapeString($output);		// output
	break;
	case 2:	//#### Image ####
		//## Watermark image
		if($wm_config['wm_img_img']=="no") {	// no image found, to prevent an error, take the same image as watermark and as input
			$img_path = $input;
			$wm_img_str = ' '.$img_path;		// no '"' before and after $img_path because it is already escaped!
		} else {
			if(file_exists($wm_config['wm_img_img']))	// needed for creating video thumbnails
			{
				$img_path = $wm_config['wm_img_img'];
				$wm_img_str = ' "'.$img_path.'"';
			}
			else
			{
				$img_path = LINPHA_DIR.'/lib/plugins/watermark/watermarks/'.$wm_config['wm_img_img'];
				$wm_img_str = ' "'.$img_path.'"';
			}
		}

		//## Scale watermark always to X% of image size
		if($wm_config['wm_resize']!="no") {
			list($org_width,$org_height,$org_type) = LinIdentify::linGetImageSize($img_path);
			
			$arr_resized = LinWatermark::wmArrResized($w,$h,$org_width,$org_height,$wm_config['wm_resize']);	// calculate the size of the resized watermark image

			$convert_str_part1 = $convert_path.'convert'.
				' -quality 100'.						// quality 100, this is only for the small image
				$wm_img_str.							// input
				' -resize "'.$arr_resized['w'].'x'.$arr_resized['h'].'"'.	// resize watermark image
				'  miff:- | ';							// output to 'pipe'
			$wm_img_str = ' "-"';							// new input from 'pipe'
		} else {
			$convert_str_part1 = '';						// no resize
			
		}

		//## The magic convert - stuff
		$convert_str = $convert_str_part1.
			$convert_path.'composite'.
			' -quality '.$q.							// here is the real quality string
			' -size "'.$w.'x'.$h.'"'.						// new size
			$dissolve_str.
			$align_str.
			' -geometry +'.$wm_config['wm_horizontal'].'+'.$wm_config['wm_vertical'].	// adjust the image with the specified paramters
			$wm_img_str.								// if the wm image is resized, it's "-"
			' '.$bracket_start.linEscapeString($input).'[0]'.	// [0] -> take only first frame!!!					// input
			' -resize "'.$w.'x'.$h.'"'.$bracket_end.						// new size
			' +profile "*"'.		// delete all profiles
			' -colorspace RGB'.		// change colorspace always to RGB because all browsers only can display RGB images
			' '.linEscapeString($output);							// output
	break;
	}
	//linLog("thumbnail","notice","watermark","convert_str: $convert_str");

	return $convert_str;
}


}
?>