<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class provides functions for edit configs etc. 
 * @package Admin
 */
class LinAdmin
{

public $options;
public $options_value_system;
public $description_array;

/**
 * constructor
 */
function __construct()
{
	/**
	 * get system option names and values and save in array for easier access
	 */
	$query = $GLOBALS['linpha']->db->Execute("SELECT option_name, option_value FROM ".LIN_PREFIX."config WHERE user_id = '0' ORDER by option_name");
	while($data = $query->FetchRow())
	{
		$this->options[] = $data['option_name'];
		$this->option_value_system[$data['option_name']] = $data['option_value'];
	}

	$this->description_array = Array(
		'sys_image_exif' => "Use EXIF",
		'sys_image_iptc' => "Use IPTC",
		'sys_image_xmp' => "Use XMP",
		'sys_lang' => "Language",
		'sys_lang_autolang' => "Autodetection of language",

		'sys_style_home_nrrandomimages' => "Nr Of Random Images (0 to disable)",
		'sys_style_home_showbrowsebydate' => "Show Browse By Date",
		'sys_style_home_showalbums' => "Show Albums On Home Site",
		'sys_style_home_firstsortorder' => "Sort Order Of First Level Albums",

		'sys_style_image_nrprevnextthumbs' => "Nr Of Previous/Next Thumbnails",
		'sys_style_image_width' => "Max Image Width",
		'sys_style_image_height' => "Max Image Height",

		'sys_style_thumb_showsubfoldersseparate' => "Show Subfolders Separate",
		'sys_style_thumb_nojsnrcols' => "Nr Cols if Javascript deactivated",
		'sys_style_thumb_nojsnrrows' => "Nr Rows if Javascript deactivated",
		'sys_style_thumb_size_display' => "Thumb size",

		'sys_style_others_sortorder' => "Sort order",

		'sys_style_layout_title' => "Linpha Title",
		'sys_style_layout_template' => "Template",
		'sys_style_layout_theme' => "Theme",
		'sys_style_layout_color_bodybg' => "Main Background Color",
		'sys_style_layout_color_elembg' => "Elements Background Color",
		'sys_style_layout_color_font' => "Fonts Color",
		'sys_style_layout_color_albumsbg' => "Albums Background Color",
		'sys_style_layout_color_links' => "Links Color",
		'sys_style_layout_color_linkshover' => "Links Hover Color",

		'plugins_log_filename' => "Log to this filename",
		'plugins_log_syslog_enable' => "Send important messages to syslog",
		'plugins_log_syslog_add_events' => "Additional log events sent to syslog",
		'plugins_log_email_enable' => "Send important messages by email",
		'plugins_log_email_add_events' => "Additional log events sent by email",
	    'plugins_log_email_to' => "Recipient",
	    'plugins_log_email_headers' => "Extra headers",
	    'plugins_log_email_subject' => "Subject",
	    
	    'plugins_filemanager_enable' => "Enable Filemanager For Non-Admin Users",
	);
}

function printAdminMenu($array_menu,$active)
{
	?>
	<ul class="linUlMenu">
		<?php
		foreach($array_menu AS $key=>$value)
		{
			if($value['link']=='linpha_home')
			{
				echo '<li><a href="../">'.i18n($value['name']).'</a></li>';
			}
			else
			{
				if($active==$key)
				{
					echo '<li><a href="./?cat='.$value['link'].'" class="current">'.i18n($value['name']).'</a></li>';
				}
				else
				{
					echo '<li><a href="./?cat='.$value['link'].'">'.i18n($value['name']).'</a></li>';
				}
			}
		}
		?>
	</ul>
	<?php
}

function printAdminConfig($type,$text,$id,$value,$settings=array())
{
	?>
	<div style="width: 600px; border: 1px solid black; padding: 5px; margin-bottom: 5px;">
		<div align="left" style="float: left;">
	<?php
	switch($type)
	{
	case 'radio':
		if($value)
		{
			$str_enabled = ' checked="checked"';
			$str_disabled = '';
		}
		else
		{
			$str_enabled = '';
			$str_disabled = ' checked="checked"';
		}
		echo $text;
		?>
		</div>
		<div align="right">
		<input type="radio" id="<?php echo $id; ?>01" name="<?php echo $id; ?>" value="1"<?php echo $str_enabled; ?> /><label for="<?php echo $id; ?>01"><?php echo i18n("Enable"); ?></label>
		<input type="radio" id="<?php echo $id; ?>02" name="<?php echo $id; ?>" value="0"<?php echo $str_disabled; ?>/><label for="<?php echo $id; ?>02"><?php echo i18n("Disable"); ?></label>
		<?php
		break;
	case 'text':
	case 'password':
		echo $text;
		?>
		</div>
		<div align="right">
		<input type="<?php echo $type; ?>" name="<?php echo $id; ?>" value="<?php echo htmlspecialchars($value, ENT_QUOTES); ?>" class="linForms" maxlength="255" style="width: 200px;" />
		<?php
  
		break;
	case 'select':
		echo $text;
		?>
		</div>
		<div align="right">
		<select name="<?php echo $id; ?>" size="1"<?php echo (isset($settings['input_settings']) ? $settings['input_settings'] : '' ); ?> class="linForms">
			<?php
			foreach($settings['options'] AS $option_key=>$option_value)
			{
				if($option_key == $value)
				{
					$selected = ' selected="selected"';
				}
				else
				{
					$selected = '';
				}
				echo '<option value="'.$option_key.'"'.$selected.'>'.$option_value.'</option>';
			}
			?>
		</select>
  		<?php
		break;
	}
	?>
		</div>
	</div>
	<?php
}

function saveConfig($array,$userid=0)
{
	foreach($array AS $value)
	{
		/**
		 * does this config name exists?
		 */
		if(isset($GLOBALS['linpha']->sql->config->value[$value]) &&
			isset($_POST[$value]))
		{
			$GLOBALS['linpha']->sql->config->updateConfig($value,$_POST[$value],$userid);
		}
	}
	
	linSysLog(i18n("Settings saved."));
	
	/**
	 * update config array
	 */
	$GLOBALS['linpha']->sql->config->reloadConfig();

	/**
	 * update system config array, used in admin/settings*.php
	 */	
	$query = $GLOBALS['linpha']->db->Execute("SELECT option_name, option_value FROM ".LIN_PREFIX."config WHERE user_id = '0' ORDER by option_name");
	while($data = $query->FetchRow())
	{
		$this->option_value_system[$data['option_name']] = $data['option_value'];
	}
}

/**
 * thanks to http://www.ilovejackdaniels.com/php/email-address-validation/
 */
function checkValidEmailAddress($email)
{
	// First, we check that there's one @ symbol, and that the lengths are right
	if (!ereg("^[^@]{1,64}@[^@]{1,255}$", $email)) {
	  // Email invalid because wrong number of characters in one section, or wrong number of @ symbols.
	  return false;
	}
	// Split it into sections to make life easier
	$email_array = explode("@", $email);
	$local_array = explode(".", $email_array[0]);
	for ($i = 0; $i < sizeof($local_array); $i++) {
	   if (!ereg("^(([A-Za-z0-9!#$%&'*+/=?^_`{|}~-][A-Za-z0-9!#$%&'*+/=?^_`{|}~\.-]{0,63})|(\"[^(\\|\")]{0,62}\"))$", $local_array[$i])) {
	    return false;
	  }
	}  
	if (!ereg("^\[?[0-9\.]+\]?$", $email_array[1])) { // Check if domain is IP. If not, it should be valid domain name
	  $domain_array = explode(".", $email_array[1]);
	  if (sizeof($domain_array) < 2) {
	      return false; // Not enough parts to domain
	  }
	  for ($i = 0; $i < sizeof($domain_array); $i++) {
	    if (!ereg("^(([A-Za-z0-9][A-Za-z0-9-]{0,61}[A-Za-z0-9])|([A-Za-z0-9]+))$", $domain_array[$i])) {
	      return false;
	    }
	  }
	}
	return true;
}

/*function getOptions()
{
	return array(
    'sys_db_version',
    'sys_im_bracket_support',
	'sys_im_imagemagick_path',
	'sys_im_use_imagemagick',
	'sys_im_video_thumbnail',

	'sys_import_autoimport',
	'sys_import_exif',
	'sys_import_exif_autorot',

	'sys_image_exif',
	'sys_image_iptc',
	'sys_image_xmp',
	'sys_lang',
	'sys_lang_autolang',
	'sys_path_album_dir',
	'sys_path_cache_dir',
	'sys_path_tmp_dir',

	'sys_basket_mail_max_size',
	'sys_basket_download_limit',

	'sys_style_sortorder',
	'sys_style_template',
	'sys_style_home_showbrowsebydate',
	'sys_style_home_nrrandomimages',
	'sys_style_home_showalbums',
	'sys_style_home_usedefaultwelcometext',
	'sys_style_home_firstsortorder',
	'sys_style_thumb_size_max',
	'sys_style_thumb_size_display',
	'sys_style_thumb_showsubfoldersseparate',
	'sys_style_thumb_selectsizes',
	'sys_style_thumb_selectnrimages',
	'sys_style_thumb_nojsnrrows',
	'sys_style_thumb_nojsnrcols',
	'sys_style_image_quality',
	'sys_style_image_width',
	'sys_style_image_height',
	'sys_style_image_nrprevnextthumbs',

	'sys_user_autologin'
	);
}*/

/**
 * a list of all settings which are enable/disable
 * to show radio buttons instead of text field
 */
function getRadioOptions()
{
	return array(
    'sys_im_bracket_support',
	'sys_im_use_imagemagick',
	'sys_im_video_thumbnail',
	'sys_import_autoimport',
	'sys_import_exif',
	'sys_import_exif_autorot',
	'sys_image_exif',
	'sys_image_iptc',
	'sys_image_xmp',
	'sys_lang_autolang',
	'sys_style_home_showbrowsebydate',
	'sys_style_home_showalbums',
	'sys_style_home_usedefaultwelcometext',
	'sys_style_thumb_showsubfoldersseparate',
	'sys_style_image_useeffects',
	'sys_user_autologin',
	'plugins_log_syslog_enable',
	'plugins_log_email_enable',
	'plugins_filemanager_enable'
	);
}

function getDescriptionByOptionName( $option_name )
{
	if( isset( $this->description_array[ $option_name ] ) )
	{
		return i18n( $this->description_array[ $option_name ] );
	}
}

function getTemplates()
{
	$templatesdir = LINPHA_DIR."/templates";
	
	$file_handle = opendir($templatesdir);
	$all_themes = array();

	while( ($file = readdir($file_handle)) !== false )
	{
		if($file != "." && $file != ".." && $file != ".svn" &&
			is_dir($templatesdir.'/'.$file) &&
			file_exists($templatesdir.'/'.$file.'/global.html.php') &&	// do at least this check
			file_exists($templatesdir.'/'.$file.'/themes') 				// do at least this check
			)
		{
			$all_themes[$file] = $file;
		}
	}
	
	closedir($file_handle);

	/* Sort theme files aphabetically */
	asort($all_themes);
	
	return $all_themes;
}

function getThemes($template)
{
	$templatesdir = LINPHA_DIR."/templates/".$template."/themes";
	
	$file_handle = opendir($templatesdir);
	$all_themes = array();

	while( ($file = readdir($file_handle)) !== false )
	{
		if($file != "." && $file != ".." && $file != ".svn" && is_dir($templatesdir.'/'.$file))
		{
			$all_themes[$file] = $file;
		}
	}
	
	closedir($file_handle);

	/* Sort theme files aphabetically */
	asort($all_themes);
	
	return $all_themes;
}

} // end class LinAdmin

?>