<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * Just a couple of useful functions
 * @package Functions
 */

/**
* get content of the systems PATH variable and remove dirs not in open_basedir and safe_mode_exec_dir (only if activated)
*/
function linGetPATH($array_path)
{
	/**
	* use the path variable instead of own array
	*/
	if($array_path == 'PATH')
	{
		unset($array_path);
		if(!empty($_SERVER['PATH']))
		{
			$path = $_SERVER['PATH'];
		}
		elseif(!empty($_SERVER['Path']))
		{
			$path = $_SERVER['Path'];
		}
		else
		{
			$path = '';
		}
		$array_path = explode(PATH_SEPARATOR,$path);
	}
	// if we have open_basedir restrictions, take only the paths which are allowed from open_basedir to prevent errors
	$open_basedir = ini_get('open_basedir');
	if(!empty($open_basedir)) {
		$arr_open_basedir = explode(PATH_SEPARATOR,$open_basedir);
		
		$array_path = array_intersect($array_path, $arr_open_basedir);
	}
	
	// same with safemode
	if(ini_get('safe_mode'))
	{
		$safe_mode_exec_dir = ini_get('safe_mode_exec_dir');
		if(!empty($safe_mode_exec_dir)) {
			
			$arr_safe_mode_exec_dir = explode(PATH_SEPARATOR,$safe_mode_exec_dir);
			
			$array_path = array_intersect($array_path, $arr_safe_mode_exec_dir);
		}
	}
	
	return $array_path;
}

/**
 * get the memory limit set in php.ini
 * if php was compiled without memory_limit
 */
function linGetMemoryLimit()
{
	$memlimit=ini_get('memory_limit');
	if(empty($memlimit))
    {
        $memlimit = get_cfg_var('memory_limit');
    }
    
    /**
     * mem limit disabled (-1) or not compiled in
     */
    if(empty($memlimit) OR $memlimit == -1)
    {
        return false;
    }
    else
    {
	   $memlimit=str_replace("M", "", $memlimit);
	   return $memlimit;
    }
}

/**
 * cut a string to the given size
 */
function linCutString($string,$len,$with_span_title)
{
	$append = '...';

	if( strlen($string) > ($len+strlen($append)) )
	{
		$str_cutted = substr_replace($string, $append, $len);
		if($with_span_title)
		{
			return '<span title="'.$string.'">'.$str_cutted.'</span>';
		}
		else
		{
			return $str_cutted;
		}
	}
	else
	{
		return $string;
	}
}

/**
 * cut string where are words longer than $len not to break page design 
 */
function linCutOverloadedStrings($str,$len)
{
	if(strlen($str)>$len) //only check long strings
	{
		if(eregi(" ", $str)) // do we have a space in str
		{
			$words = explode(" ", $str);
			$cut = false;

			while(list($key,$value) = each($words)) // check wordlength of ALL words in string
			{
				if(strlen($value)>$len) {
					$cut = true;
					$cut_word = substr_replace($value, '...', ($len-3));
					$translate = array($value => $cut_word);
					$verified_string = strtr($str, $translate);
				}
			}

			if(!$cut)
			{
				
				$verified_string = $str; 
			}
		}
		else
		{
			$verified_string=substr_replace($str, '...', $len); //cut
		}
	}
	else
	{
		$verified_string=$str;
	}
	return $verified_string;
}

/**
 * format user inputs and autoformat urls and mailaddresses
 */
function linHtmlTag($post)
{
	$post = htmlspecialchars($post, ENT_QUOTES);
	
	//###### URL
	// the next two parts converts http://linpha.sf.net to [url]http://linpha.sf.net[/url]
	// and www.sourceforge.net to [url]http://www.sourceforge.net[/url]
	// im the third part converts [url]http://linpha.sf.net[/url] to <a href="http://linpha.sf.net">http://linpha.sf.net</a>

	// 1. part
	// detect url if it is after one of the following signs: ' ', '\r' or '\n'
	$post=eregi_replace("([ \r\n])http://([^ ,\r\n]*)","\\1[url]http://\\2[/url]",$post);
	$post=eregi_replace("([ \r\n])https://([^ ,\r\n]*)","\\1[url]https://\\2[/url]",$post);
	$post=eregi_replace("([ \r\n])ftp://([^ ,\r\n]*)","\\1[url]ftp://\\2[/url]",$post);
	$post=eregi_replace("([ \r\n])www\\.([^ ,\r\n]*)","\\1[url]http://www.\\2[/url]",$post);

	// 2. part
	// detect url if the url is in the beginning of the string
	$post=eregi_replace("^http://([^ ,\r\n]*)","[url]http://\\1[/url]",$post);
	$post=eregi_replace("^https://([^ ,\r\n]*)","[url]https://\\1[/url]",$post);
	$post=eregi_replace("^ftp://([^ ,\r\n]*)","[url]ftp://\\1[/url]",$post);
	$post=eregi_replace("^www\\.([^ ,\r\n]*)","[url]http://www.\\1[/url]",$post);

	// 3. part
	$post=preg_replace("/\[url\](.*)\[\/url\]/iUms","<a href=\"\\1\" target=\"_blank\">\\1</a>",$post);
	$post=preg_replace("/\[url=(.*)\](.*)\[\/url\]/iUms","<a href=\"\\1\" target=\"_blank\">\\2</a>",$post);
	//#####


	// e-mail	
	$post= eregi_replace('([_a-z0-9-]+(\.[_a-z0-9-]+)*@[_a-z0-9-]+(\.[_a-z]+)+)','<a href="mailto:\\1">\\1</a>',$post);
	
	// formatting
	$post = str_replace("[hr]","<hr>",$post);
	$post=preg_replace("/\[b\](.*)\[\/b\]/iUms","<b>\\1</b>",$post);
	$post=preg_replace("/\[i\](.*)\[\/i\]/iUms","<i>\\1</i>",$post);
	$post=preg_replace("/\[s\](.*)\[\/s\]/iUms","<strike>\\1</strike>",$post);
	$post=preg_replace("/\[u\](.*)\[\/u\]/iUms","<u>\\1</u>",$post);
	$post=preg_replace("/\[h1\](.*)\[\/h1\]/iUms","<h1>\\1</h1>",$post);
	$post=preg_replace("/\[h2\](.*)\[\/h2\]/iUms","<h2>\\1</h2>",$post);
	$post=preg_replace("/\[h3\](.*)\[\/h3\]/iUms","<h3>\\1</h3>",$post);
	$post=preg_replace("/\[h4\](.*)\[\/h4\]/iUms","<h4>\\1</h4>",$post);
	$post=preg_replace("/\[h5\](.*)\[\/h5\]/iUms","<h5>\\1</h5>",$post);
	
	
	//code
	$post=preg_replace("/\[code\](.*)\[\/code\]/msiU", "<blockquote><pre><smallfont><hr>\\1<hr></smallfont></pre></blockquote>", $post);
	
	// image
	$post=preg_replace("/\[img\](.*)\[\/img\]/iUms","<img src=\"\\1\" border=\"0\">",$post);
	
	// center
	$post=preg_replace("/\[center\](.*)\[\/center\]/iUms","<center>\\1</center>",$post);
	
	// colors
	$post=preg_replace("/\[green\](.*)\[\/green\]/iUms","<span style=\"color:green\">\\1</span>",$post);
	$post=preg_replace("/\[yellow\](.*)\[\/yellow\]/iUms","<span style=\"color:yellow\">\\1</span>",$post);
	$post=preg_replace("/\[red\](.*)\[\/red\]/iUms","<span style=\"color:red\">\\1</span>",$post);
	$post=preg_replace("/\[blue\](.*)\[\/blue\]/iUms","<span style=\"color:blue\">\\1</span>",$post);
	$post=preg_replace("/\[white\](.*)\[\/white\]/iUms","<span style=\"color:white\">\\1</span>",$post);
	$post=preg_replace("/\[black\](.*)\[\/black\]/iUms","<span style=\"color:black\">\\1</span>",$post);
	
	$post=preg_replace("/\[color=(.*)\](.*)\[\/color\]/iUms","<span style=\"color:\\1\">\\2</span>",$post);
	
	// <br />
	$post= nl2br($post);

	return $post;
}

/**
 * print a nice navigation line with links on each entry
 */
function linSetNavigationLine( $id, $link )
{
	$str = '<a href="'.$link.'0">&gt;</a> ';

	$array_files = $GLOBALS['linpha']->sql->getFullFilenameFromId( $id );
	foreach($array_files AS $key=>$value)
	{
		if($value != "")
		{
			$array_values[] = '<a href="'.$link.$key.'">'.htmlspecialchars($value,ENT_QUOTES).'</a>';
		}
	}
	
	if(isset($array_values))
	{
		$str .= implode(' &gt; ',$array_values);
	}

	
	return $str;
}

/**
 * functions to create a html select form with all albums
 */
function linBuildAlbumSelect($with_all_albs_entry)
{
    ?>
	
	<?php
	if($with_all_albs_entry)
	{
    	if(!isset($_REQUEST['album_select']) OR $_REQUEST['album_select'][0]=='all') {
    		$select = ' selected';
            $_REQUEST['album_select'] = Array('all'=>'all');
    	} else {
    		$select = '';
    	}
	    echo '<option value="all"'.$select.'>'.i18n("All Albums").'</option>'."\n";
    }
	
	linBuildAlbumSelectSubEntry(0,'');
}

function linBuildAlbumSelectSubEntry($id,$text)
{
	$query = $GLOBALS['linpha']->db->Execute("SELECT id, name FROM ".LIN_PREFIX."photos WHERE parent_id = '".$id."' AND img_type = '0' ORDER by name");
	while($data = $query->FetchRow())
	{
		if(!isset($_REQUEST['album_select'])) { 
			$_REQUEST['album_select'] = array();
		}		
		if(in_array($data['id'],$_REQUEST['album_select'])) {
			$select = ' selected';
		} else {
			$select = '';
		}
		echo '<option value="'.$data['id'].'"'.$select.'>'.$text.'/'.htmlspecialchars($data['name'],ENT_QUOTES).'</option>'."\n";
		
		linBuildAlbumSelectSubEntry($data['id'],$text.'/'.htmlspecialchars($data['name'],ENT_QUOTES));
	}
}

/**
 * returns an array with the sort orders
 */
function linGetSortOrders()
{
	return Array(
		'nameasc' => i18n("Name Asc"),
		'namedesc' => i18n("Name Desc"),
		'dateasc' => i18n("Date Asc"),
		'datedesc' => i18n("Date Desc")
	);
}

/**
 * return the correct sql order string
 */
function linGetSqlSortOrder($order)
{
	switch($order)
	{
	case 'nameasc':		return "name ASC";
	case 'namedesc':	return "name DESC";
	case 'dateasc':		return "time_add ASC";
	case 'datedesc':	return "time_add DESC";
	}
}

/**
 * explode string to array and remove first and last entry of array because they are empty
 */
function linExplodeAndSlice($del,$string)
{
    $array = explode($del,$string);
    $array = array_slice($array,1,-1);
    
    return $array;
}

/**
 * we have to use &amp; in links (W3C), but in javascript we cannot use &amp;!
 */
function linConvertAmp($url)
{
	return str_replace('&amp;','&',$url);
}

/**
 * some functions to calculate the page rendering time
 * this is very useful while trying to speed up LinPHA :-)
 */
function linStartTimer($string) 
{
	global $timer;
	$timer[$string]=linGetRenderTime();
}

/**
 * some functions to calculate the page rendering time
 * this is very useful while trying to speed up LinPHA :-)
 */
function linStopTimer($string)
{
	global $timer;
	return number_format(linGetRenderTime()-$timer[$string],4);
}

/**
 * some functions to calculate the page rendering time
 * this is very useful while trying to speed up LinPHA :-)
 */
function linGetRenderTime()
{
	list ($usec, $sec)=explode(" ", microtime());
	return ((float)$usec + (float)$sec);
}


/**
* this function extends the php strftime with the localizations of linpha
* because the respective locales aren't installed on most systems.
*
*	%a - abbreviated weekday name according to the current locale
*	%A - full weekday name according to the current locale
*	%b - abbreviated month name according to the current locale
*	%B - full month name according to the current locale
*	
*	%c - preferred date and time representation for the current locale
*	%x - preferred date representation for the current locale without the time
*	%X - preferred time representation for the current locale without the date
*	
*	%w - day of the week as a decimal, Sunday being 0
*	%m - month as a decimal number (range 01 to 12)
*
* @author  flo
* @param  string  $str_format  optional, date and time format string, if empty default time format is used
* @param  int  $timestamp  optional, unix timestamp, if empty time() is used
* @return  string  formatted date and time string
* @package  time
*/
/*
If the time and date format in your country is different with these formats,
please set it to the right format. Otherwise, leave it as it is.
See http://www.php.net/manual/en/function.strftime.php for definitions.
(Don't apply this to the english language file, the default date and time format are
set in the options page of linpha. This is because the date format is different within
the english spoken countries.)
*/
function linStrftime($timestamp='now',$str_format='')
{
	/**
	 * if making any changes in these arrays, all language files needs updating
	 */
	$arr_month_short = Array('1' => i18n('Jan'),'2' => i18n('Feb'),'3' => i18n('Mar'),'4' => i18n('Apr'),'5' => i18n('May'),'6' => i18n('Jun'),'7' => i18n('Jul'),'8' => i18n('Aug'),'9' => i18n('Sep'),'10' => i18n('Oct'),'11' => i18n('Nov'),'12' => i18n('Dec')); /* abrevations of months */
	$arr_month_long = Array('1' => i18n('January'),'2' => i18n('February'),'3' => i18n('March'),'4' => i18n('April'),'5' => i18n('May'),'6' => i18n('June'),'7' => i18n('July'),'8' => i18n('August'),'9' => i18n('September'),'10' => i18n('October'),'11' => i18n('November'),'12' => i18n('December')); /* months */
	$arr_day_short = Array(i18n('Sun'),i18n('Mon'),i18n('Tue'),i18n('Wed'),i18n('Thu'),i18n('Fri'),i18n('Sat')); /* abrevations of weekdays */
	$arr_day_long = Array(i18n('Sunday'),i18n('Monday'),i18n('Tuesday'),i18n('Wednesday'),i18n('Thursday'),i18n('Friday'),i18n('Saturday')); /* weekdays */
	
	if(empty($str_format)) {
		$str_format = linGetDateFormat().' '.linGetTimeFormat();
	}

	if($timestamp == 'now' OR empty($timestamp)) {
		$timestamp = time();
	}

	$weekday_in_decimal = strftime("%w", $timestamp);
	$month_in_decimal = strftime("%m", $timestamp);	// pay attention: %m is '01 to 12', and not '1 to 12'
	if($month_in_decimal < 10) {
		$month_in_decimal = $month_in_decimal[1];
	}
	
	$str_format = str_replace("%c", linGetDateFormat().' '.linGetTimeFormat(), $str_format);
	$str_format = str_replace("%x", linGetDateFormat(), $str_format);
	$str_format = str_replace("%X", linGetTimeFormat(), $str_format);

	$str_format = str_replace("%a", $arr_day_short[$weekday_in_decimal], $str_format);
	$str_format = str_replace("%A", $arr_day_long[$weekday_in_decimal], $str_format);
	
	if(!empty($month_in_decimal))
	{
	    $str_format = str_replace("%b", $arr_month_short[$month_in_decimal], $str_format);
	    $str_format = str_replace("%B", $arr_month_long[$month_in_decimal], $str_format);
    }
	
	return strftime($str_format,$timestamp);
}

/**
* Get date format string
* if it isn't defined in the current language file (for example in the english language file)
* the date format string is read from the linpha_config table
* 
* @author  flo
* @return  string  time format string for use in strftime()
* @package  time
*/
function linGetDateFormat()
{
	if( i18n('special_date_format') == 'special_date_format') {
		return $GLOBALS['linpha']->sql->config->value['sys_style_datetime_dates'];
	} else {
		return i18n('special_date_format');
	}
}

/**
* Get time format string
* if it isn't defined in the current language file (for example in the english language file)
* the time format string is read from the linpha_config table
* 
* @author  flo
* @return  string  time format string for use in strftime()
* @package  time
*/
function linGetTimeFormat()
{
	if( i18n('special_time_format') == 'special_time_format') {
		return $GLOBALS['linpha']->sql->config->value['sys_style_datetime_times'];
	} else {
		return i18n('special_time_format');
	}
}

/**
 * switch between Ajax and static view
 * takes care if we explicit disable ajax
 * even if we could use it
 */
function linUseAjax()
{
	if( isset($_SESSION['use_js']) && ! isset($_SESSION['disable_ajax']) )
	{
		return true;
	}
	else
	{
		return false;
	}
}


/**
 * LinPHA internal logging function
 * 
 * Types:
 * LOG_TYPE_IMPORT
 * LOG_TYPE_UPDATE
 * LOG_TYPE_USER
 * LOG_TYPE_DB
 * LOG_TYPE_META
 * LOG_TYPE_FILEMANAGER
 * LOG_TYPE_GUESTBOOK
 * 
 * Severity:
 * LOG_EMERG	1	System ist unbrauchbar.
 * LOG_ALERT	1	Aktion ist sofort erforderlich.
 * LOG_CRIT		1	Kritischer Zustand.
 * LOG_ERR		4	Fehler aufgetreten.
 * LOG_WARNING	5	Warnung aufgetreten.
 * LOG_NOTICE	6	Normale, aber wichtige, Meldung.
 * LOG_INFO		6	Informative Meldung.
 * LOG_DEBUG	6	Debug-Level-Meldung.
 * 
 * Events:
 * login, logout, rotate
 * fm_others, fm_upload, fm_delete, fm_move, fm_copy, fm_rename, fm_create_folder, fm_perm
 * thumbnail, image
 * meta_iptc
 * accessdenied
 * 
 * 
 * @param  string  $type  log types
 * @param  string  $severity  from notice - fatal error
 * @param  string  $text
 */
/**
 * defines for linLog()
 */
define('LOG_TYPE_IMPORT',		0);
define('LOG_TYPE_UPDATE',		1);
define('LOG_TYPE_USER',			2);
define('LOG_TYPE_DB',			3);
define('LOG_TYPE_META',			4);
define('LOG_TYPE_FILEMANAGER',	5);
define('LOG_TYPE_GUESTBOOK',	6);
define('LOG_TYPE_BASKET',		7);
function linLog($type,$severity,$event,$text)
{
	switch($type)
	{
	case LOG_TYPE_IMPORT:			$type_msg = 'IMPORT  ';				break;
	case LOG_TYPE_UPDATE:			$type_msg = 'UPDATE  ';				break;
	case LOG_TYPE_USER:				$type_msg = 'USER    ';				break;
	case LOG_TYPE_DB:				$type_msg = 'DATABASE';				break;
	case LOG_TYPE_META:				$type_msg = 'COMMENTS';				break;
	case LOG_TYPE_FILEMANAGER:		$type_msg = 'FILEMANAGER';			break;
	case LOG_TYPE_GUESTBOOK:		$type_msg = 'GUESTBOOK';			break;
	case LOG_TYPE_BASKET:			$type_msg = 'BASKET';				break;
	}

	/**
	 * Set Log Severity Output Message
	 */
	switch($severity)
	{
	case LOG_EMERG:					$severity_msg = 'EMERGENCY ERROR';	break;
	case LOG_ALERT:					$severity_msg = 'ALERT  ';			break;
	case LOG_CRIT:					$severity_msg = 'CRITICAL ERROR';	break;
	case LOG_ERR:					$severity_msg = 'ERROR  ';			break;
	case LOG_WARNING:				$severity_msg = 'WARNING';			break;
	case LOG_NOTICE:				$severity_msg = 'NOTICE ';			break;
	case LOG_INFO:					$severity_msg = 'INFO   ';			break;
	case LOG_DEBUG:					$severity_msg = 'DEBUG  ';			break;
	}

	/**
	 * Setup Log Time Output Message & Remote IP
	 */
	$time = linStrftime(time());
	$ip   = $_SERVER["REMOTE_ADDR"];
	if( !isset($_SESSION['user_name'])) {
		$user = 'anonymous';
	} else {
		$user = $_SESSION['user_name'];
	}
	
	/**
	 * don't trust $text (contains $_GET and $_POST data)
	 * see also linpha advisory http://secunia.com/advisories/18808/
	 * and for more details http://retrogod.altervista.org/linpha_10_local.html 
	 */
	$text = htmlspecialchars($text, ENT_QUOTES);

	/**
	 * Process Log
	 */
	if( !isset($GLOBALS['linpha']->sql->config->value) )	// database not connected!
	{
		error_log("$time | $severity_msg | $type_msg | $ip | $user | $event | $text".LIN_NL, 3, LINPHA_DIR.'/var/db_error.log' );
	}
	else
	{
		/**
		 * filename
		 */
	    error_log("$time | $severity_msg | $type_msg | $ip | $user | $event | $text".LIN_NL, 3, LinFilesys::getFullPath($GLOBALS['linpha']->sql->config->value['plugins_log_filename']) );
	    
		/**
		 * email
		 */
		if( $GLOBALS['linpha']->sql->config->value['plugins_log_email_enable'] )
		{
		    if( $severity <= LOG_WARNING OR
		    	strpos($GLOBALS['linpha']->sql->config->value['plugins_log_email_add_events'], $event) !== false )
		    {
			    mail(
			    	$GLOBALS['linpha']->sql->config->value['plugins_log_email_to'],
			    	$GLOBALS['linpha']->sql->config->value['plugins_log_email_subject'],
			    	"$time | $severity_msg | $type_msg | $ip | $ip | $user | $text",
			    	$GLOBALS['linpha']->sql->config->value['plugins_log_email_headers']
			    );
		    }
		}

	    /**
	     * syslog
	     */
		if( $GLOBALS['linpha']->sql->config->value['plugins_log_syslog_enable'] &&
			$severity <= LOG_WARNING )
		{
			/**
		     * In Windows NT, use the following values of priority:
			 * 1 = error,
			 * 2-5 = warning
			 * 6 = info
			 */
			
			if( LIN_OS == 'win' )
			{
				if( $severity <= LOG_ERR )
				{
					$severity = 1;
				}
			}
		    
		    syslog($severity, "LinPHA $severity_msg | $type_msg | $ip | $user | $event | $text");
		}
	}
}

/**
 * LinPHA user logging function
 * shows up system info in the browser like "images added"
 */
function linSysLog($str)
{
	if( ! isset( $GLOBALS['linpha']->template->output['sys_log'] ) )
	{
		$GLOBALS['linpha']->template->output['sys_log'] = $str;
	}
	else
	{
		$GLOBALS['linpha']->template->output['sys_log'] .= '<br />'.$str;
	}
}

/**
 * Take care of translation
 * 
 * This simple method takes care of all translation related stuff.
 * If option tr_learn is set - all new entries will be autolearned.
 * @param string $text phrase/word to translate
 */
function i18n($text)
{
	global $translate;
	$tr_learn = true; //autolearn new language entries
	
	/**
	 * translation starts here
	 */
	if(true == isset($translate[$text]))
	{
		$str = "$translate[$text]";
	}
	else
	{
		$str = "$text";
	}

	/**
	 * autolearn new language entries defined by tr() function.
	 */
	if($tr_learn == true)
	{
		$tmpfile = LINPHA_DIR."/var/lang.temp.txt";

		if(false == file_exists("$tmpfile"))
		{	
			if(false == touch("$tmpfile"))
			{
				echo "Unable to create ".$tmpfile." - please check permissions";
			}
		}
		
		$filedata = file("$tmpfile");

		if(false == in_array($text."\n", $filedata))
		{
			$filedata[] = $text."\n";
			natcasesort($filedata);	// Sort an array using a case insensitive "natural order" algorithm
			file_put_contents($tmpfile, $filedata);
		}
	}
	
	return $str;

} // end tr()

/**
* escape string for use in exec() (especially for convert), not in any other 
* function like fopen etc.
*
* @author  flo
* @param  string  $str  string to exec
* @return  string  escape string
* @package  filesys
*/
function linEscapeString($str)
{
	if( LIN_OS == 'win' )
	{
		/**
		 * replace forward slash with backslash, need this for powerarchiver for
		 * downloading albums, because powerarchiver cannot handle forward
		 * slashes in paths, does this disturb somewhere??
		 */
		$str = str_replace("/","\\",$str);

		/**
		* escape string with ("), -> call exec always without (")
		* for example: exec('convert "'.escape_string($input).'" "'.escape_string($output).'") is FALSE !!!
		* escape with (") because under windows we cannot have double quotes in filenames, in contrast to single quotes!
		*/

		$str = '"'.$str.'"';
	}
	else
	{
        /**
         * do not escape string with safemode, because if safemode is on,
         * strings are escaped automatically
         *
         * see http://ch.php.net/manual/en/features.safe-mode.functions.php
         * exec() 	You can only execute executables within the
         * safe_mode_exec_dir. For practical reasons it's currently not allowed
         * to have .. components in the path to the executable. escapeshellcmd()
         * is executed on the argument of this function.
         */

			/**
			* done: maybe we should this replace with the php builtin function
			* escapeshellcmd()
            * 
            * now we should get the same results wether safe mode is on or off 
			*/
			/*$str = str_replace("\\","\\\\",$str);
            
            //$str = escapeshellcmd($str);
			
			$array = Array('"','\'','(',')','[',']','{','}','*','?','!','`','=','&','@',' ',':',';');
		
			foreach($array AS $value)
			{
				$str = str_replace($value,'\\'.$value,$str);
			}*/
		//}
        /**
         * what is the better solution:
         * 1. convert albums/dir\ with\ spaces/test.jpg tmp/test.jpg
         * or
         * 2. convert "albums/dir with spaces/test.jpg" "tmp/test.jpg"
         * 
         * 1. negativ: if safemode is on it doesn't work anymore because the
         * backslashes are escaped too
         */
        $str = str_replace('"','\"',$str);
        $str = '"'.$str.'"';
	}	
	return $str;
}


?>