<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class takes care of image and file identifying due
 * a couple of possible methods
 * @package Image
 */

class LinIdentify
{

/**
 * constructor
 */
function __construct()
{
}


/**
 * returns true if this is a panorama image
 * all panorama images have 2/1 ratio (width/height)
 *
 * @param int $org_width width of images
 * @param int $org_height height of image
 * @return  true|false
 */
function isPanoramaImage($org_width,$org_height)
{
	if( ($org_width/2) >= $org_height &&
		$org_width > 0 && $org_height > 0 && !empty($org_width) && !empty($org_height) )
	{
		return true;
	} else {
		return false;
	}
}


/**
 * returns true if we can handle this kind of image
 *
 * @author  flo
 * @param  int  $org_type imagetype in the format of getimagesize() "JPEG", ...
 * @return  int  true|false
 */
function isSupportedImage($org_type)
{
	if( $GLOBALS['linpha']->sql->config->value['sys_im_use_imagemagick'] ) {
		if($org_type >= 1 && $org_type < 1000) {
			return true;
		} else {
			return false;
		}
	} else {	// gdlib
		if($org_type >= 1 && $org_type <= 3) {	// gdlib supports only jpg, gif and png!
			return true;
		} else {
			return false;
		}
	}
}


/**
 * checks if the given file is a video
 *
 * @author  flo
 * @param  int $org_type imagetype in the format of getimagesize()
 * @return  true|false
 */
function isVideo($org_type)
{
	if($org_type >= 1000 && $org_type < 2000)
	{
		return true;
	} else {
		return false;
	}
}

function getShortFileType( $img_type )
{
	if( LinIdentify::isSupportedImage( $img_type )) {
		return 1;
	} elseif( LinIdentify::isVideo( $img_type )) {
		return 2;
	} else {
		return 0;
	}
}


/**
 * Get image properties.
 * This function extends PHP's getimagesize() function. This include's
 * support for TIFF like files with PHP < 4.2.0 and others
 * 
 * @todo bzrudi: is this still necesarry using php > 5.x ???
 * @todo flo: yes i think so, imagemagick supports much more images than the gdlib
 *            but i don't now about the detectTiffType() stuff 
 *  
 * @author  bzrudi,flo
 * @param  string  $path  path to image
 * @return  array  array(width,height,type,img_string)
 */
function linGetImageSize($path)
{
	$g = @getimagesize( $path );
	
	$ext = LinFilesys::getFileExtFromPath($path);
	
	/**
	* $g[2] doesn't exists: -> image type is for example TIFF, but it also can be AVI etc.
	* -> use identify from imagemagick to determine the imagetype
	*
	* but first, check if it is a video, because imagemagick's identify really sucks in detecting videos
	* (segmentation faults and wait times about 2 minutes for each file!)
	* test results:
	* version 5.4.4, 5.5.6, 5.5.7 slow (for one AVI file 2 minutes)
	* 6.0.0 works fine
	* 6.0.7 segmentation fault
	*/
	if( !$g[2] )
	{
		$org_type = LinIdentify::convertImagetype($ext);
		if(LinIdentify::isVideo($org_type))
		{
			$g[2] = $org_type;
		}
		else
		{
			$identify_path = $GLOBALS['linpha']->sql->config->value['sys_im_imagemagick_path'] ."identify";
			if( $GLOBALS['linpha']->sql->config->value['sys_im_use_imagemagick'] )
			{
				/**
				* $array_output[0] contains something like '2272 1704 JPEG'
				*/
				LinFilesys::linExec ( $identify_path.' -format "%w %h %m" '.linEscapeString($path), $array_output = Array(), $return_value = 0);
				
				if($return_value == 0)	// identify succeed
				{
					if(isset($array_output[0]))
					{
						print_r($array_output);
						exit();

						$g = explode( " ", $array_output[0] );
				
						$g[2] = LinIdentify::convertImagetype($g[2],$path);
					}
				}
			}
		}
	}
	else	//  getimagesize detects mpg and mp4 as type 15, but type 15 is wbmp (look at the mimetype returned by getimagesize), pay attention...
	{
		if($g[2] == 15)
		{
			if(LinIdentify::convertImagetype($ext) == 1000)	// 1000 = mpg, mpeg, mpe, ...
			{
				$g[2] = LinIdentify::convertImagetype('mpg');	// get imagetype of mpg
			}elseif($ext == 'mov'){
				$g[2] = LinIdentify::convertImagetype('mov');
			}elseif($ext == 'mp4'){
				$g[2] = LinIdentify::convertImagetype('mp4');
			}
		}
	}
	
	/**
	* still don't have an imagetype, take imagetype by fileextension
	* @todo  what should we do now, getimagesize and identify doesn't handle this files as images and we doesn't know the image height and width, i think they therefore can't convert/handle this file, so we should declare it as unsupported. But we need still a check if it is a video or something other, so we can't remove get_imagetype_from_filename(). That means we should check if get_imagetype_from_filename() returned a value lesser than 1000 and if so, delete it! what do others mean?! flo
	*/
	if( !$g[2] )
	{
		$type = LinIdentify::getImagetypeFromFilename($path);
		if($type < 1000)
		{
			$g[2] = LinIdentify::convertImagetype('unsupported');
		}
		else
		{
			$g[2] = $type;
		}
	}
	
	
	/**
	* set to not empty to prevent erros on commands like
	* list($org_width,$org_height,$org_type) = linpha_getimagesize($img2rotate);
	*/
	if(!isset($g[0]))
	{
		$g[0] = 0;
	}

	if(!isset($g[1]))
	{
		$g[1] = 0;
	}
	
	$g[3] = 'width="'.$g[0].'" height="'.$g[1].'"';

	return $g;
}


/**
 * converts the image type from imagemagick to the format of getimagesize()
 * for example: "TIFF" (imagemagick) -> "7" (getimagesize())
 *
 * @author  flo
 * @param  string  $type  imagetype from identify, for example "TIFF", "JPEG", ...
 * @param  string  $path  optional, path to image, needed for tifftype detection
 * @return  int  imagetype in the format of getimagesize()
 */
function convertImagetype($type,$path='')
{
	$type = trim($type);
	$type = strtolower($type);

	$imagetypes = LinIdentify::getImagetypeTranslationArray();

	if(array_key_exists($type,$imagetypes)) {
		$orgtype = $imagetypes[$type];
	} else {
		$orgtype = $imagetypes['unsupported'];
	}

	/**
	* there are two different tiff formats
	* check which tiff file it is
	*/
	if( $orgtype == 7 )
	{
 		if( file_exists( $path ) ) {	// valid
			$orgtype = LinIdentify::detectTiffType($path);
		} else {
			$orgtype = 7;	// set default to number 7
		}
	}
	
	return $orgtype;
}


/**
 * detects the tiff type of the image
 * there are two different TIFF formats:
 * intel byte order (little endian) and motorola byte order (big endian))
 * 
 * The second word of TIFF files is the TIFF version number, 42, which has
 * never changed. The TIFF specification recommends testing for it
 *
 * @author  bzrudi?
 * @param  string  $path  path to tiffimage
 * @return  int  7 or 8 (imagetype in the format of getimagesize())
 */
function detectTiffType($path)
{
	$tfh = fopen( $path, "r" );
	$magic = fread( $tfh, 4 );
	fclose( $tfh );
	
	// big-endian=8, little-endian=7 according to getimagesize() of PHP4.2
	if(ord($magic[2])==42 && ord($magic[3])==0) {
		$type = 7;
	} else {
		$type = 8;
	}
	return $type;
}


/**
 * gets the imagetype from a path/filename.ext, filename.ext, .ext or ext* 
 *
 * @author  flo
 * @param  string  $filename  path to image
 * @return  int  imagetype in the format of getimagesize()
 */
function getImagetypeFromFilename($filename)
{
	$ext = LinFilesys::getFileExtFromPath($filename);
	$imagetypes = LinIdentify::getImagetypeTranslationArray();
	if(array_key_exists($ext,$imagetypes)) {
		return $imagetypes[$ext];
	} else {
		return $imagetypes['unsupported'];
	}
}


/**
 * gets an array with all imagetypes, fileext are the keys, imagetype the values
 * see http://ch.php.net/manual/de/function.exif-imagetype.php
 *
 * @author  flo
 * @return  array  keys=>value, fileext=>imagetype
 * @uses  get_imagetype_from_filename(),convert_imagetype()
 */
function getImagetypeTranslationArray()
{
	$array = Array(
// getimagesize starts here

	'gif' => 1,
	'jpg' => 2,
	'jpeg' => 2,
	'png' => 3,
	'swf' => 4, 	//not supported by imagemagick?
	'psd' => 5,
	'bmp' => 6,
	'tiff' => 7,
	'tiff_ii' => 7,	// IMAGETYPE_TIFF_II (intel byte order) little endian
	'tiff_mm' => 8,	// IMAGETYPE_TIFF_MM (motorola byte order) big endian
	'jpc' => 9,
	'jp2' => 10,
	'jpx' => 11,	//not supported by imagemagick?
	'jb2' => 12,	//not supported by imagemagick?
	'swc' => 13,	//not supported by imagemagick?
	'iff' => 14,	//not supported by imagemagick?
	'wbmp' => 15,	// getimagesize detects mpg as type 15, but type 15 is wbmp (look at the mimetype returned by getimagesize), pay attention...
	'xbm' => 16,


// imagemagick starts here
	'art' => 100,
//	'avi' => 101,	video
	'avs' => 102,
//	'bmp' => 103,	already
	'cgm' => 104,
	'cin' => 105,
	'cmyk' => 106,
	'cur' => 107,
	'cut' => 108,
	'dcm' => 109,
	'dcx' => 110,
	'dib' => 111,
	'dng' => 112,
	'dot' => 113,
	'dpx' => 114,
	'emf' => 115,
	'epdf' => 116,
	'epi' => 117,
	'eps' => 118,
//	'eps2' => 119,	only write support
//	'eps3' => 120,	only write support
	'epsf' => 121,
	'epsi' => 122,
	'ept' => 123,
	'fax' => 124,
	'fig' => 125,
	'fits' => 126,
	'fpx' => 127,
//	'gif' => 128,	already
	'gplt' => 129,
	'gray' => 130,
	'hpgl' => 131,
//	'html' => 132,	this type shouldn't be handled as image
	'ico' => 133,
	'jbig' => 134,
	'jng' => 135,
//	'jp2' => 136,	already
//	'jpc' => 137,	already
//	'jpeg' => 138,	already
//	'man' => 139,	this type shouldn't be handled as image
	'mat' => 140,
	'miff' => 141,
	'mono' => 142,
	'mng' => 143,
//	'mpeg' => 144,	video
//	'm2v' => 145,	video
	'mpc' => 146,
	'msl' => 147,
	'mtv' => 148,
	'mvg' => 149,
	'otb' => 150,
	'p7' => 151,
	'palm' => 152,
//	'pam' => 153,	only write support
	'pbm' => 154,
	'pcd' => 155,
	'pcds' => 156,
	'pcl w' => 157,
	'pcx' => 158,
	'pdb' => 159,
//	'pdf' => 160,	this type shouldn't be handled as image
	'pfa' => 161,
	'pfb' => 162,
	'pgm' => 163,
	'picon' => 164,
	'pict' => 165,
	'pix' => 166,
//	'png' => 167,	already
	'pnm' => 168,
	'ppm' => 169,
	'ps' => 170,
	'ps2' => 171,
	'ps3' => 172,
//	'psd' => 173,	 already
	'ptif' => 174,
	'pwp' => 175,
	'rad' => 176,
	'rgb' => 177,
	'rgba' => 178,
	'rla' => 179,
	'rle' => 180,
	'sct' => 181,
	'sfw' => 182,
	'sgi' => 183,
//	'shtml' => 184,	only write support
	'sun' => 185,
	'svg' => 186,
	'tga' => 187,
//	'tiff' => 188,	already
	'tim' => 189,
	'tif' => 190,
//	'txt' => 191,	this type shouldn't be handled as image
//	'uil' => 192,	only write support
	'uyvy' => 193,
	'vicar' => 194,
	'viff' => 195,
//	'wbmp' => 196,	already
	'wmf' => 197,
	'wpg' => 198,
//	'x' => 199,		?
//	'xbm' => 200,	already
	'xcf' => 201,
	'xpm' => 202,
	'xwd' => 203,
	'ycbcr' => 204,
	'ycbcra' => 205,
	'yuv' => 206,

// video
	'mpg' => 1000,	// getimagesize detects mpg as type 15, but type 15 is wbmp (look at the mimetype returned by getimagesize), pay attention...
	'mpeg' => 1000,
	'mp4' => 1001,
	'avi' => 1002,
	'mov' => 1003,
	'm2v' => 1004,
	'wmv' => 1005,
	'mpe' => 1000,
/*
// sound
	'mp3' => 2000,
	'wav' => 2001,
	'ogg' => 2002,
	'midi' => 2003,
*/

/*
//text
	'html' => 3000,
	'htm' => 3001,
	'php' => 3002,
	'asp' => 3003,

	'txt' => 3050,
	'pdf' => 3051,
	'rtf' => 3052,
	'man' => 3053,
	
// OpenOffice
	'sxw' => 3100,	// OpenOffice Textdocument
	'stw' => 3101,	// OpenOffice Textdocument
	'sxc' => 3102,	// OpenOffice Spreadsheet
	'stc' => 3103,	// OpenOffice Spreadsheet
	'sxi' => 3104,	// OpenOffice Presentation
	'sti' => 3105,	// OpenOffice Presentation
	'sxd' => 3106,	// OpenOffice Draw
	'std' => 3107,	// OpenOffice Draw
	'sxm' => 3108,	// OpenOffice Formula
	
	'sdw' => 3120,	// Starwriter Textdocument
	'sdc' => 3121,	// Starcalc
	'sda' => 3122,	// StarImpress
	'sdd' => 3123,	// StarImpress
	'sdp' => 3124,	// StarImpress

// MS Office
	'doc' => 3200,	// MS Word
	'dot' => 3201,	// MS Word
	'xls' => 3202,	// MS Excel
	'xlt' => 3203,	// MS Excel
	'ppt' => 3204,	// MS Powerpoint
	'pps' => 3205,	// MS Powerpoint
	'pot' => 3206,	// MS Powerpoint
*/

	'unsupported' => 9999999

	);
	return $array;
} // end function getImagetypeTranslationArray()

} // end class linIdentify
?>
