<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class provides a couple methods for image manipulation/creation
 * @package Image
 */
 
if(!defined('LINPHA_DIR')) { exit(1); }

class LinImage
{
public $error_msg;
public $output_file;

public $photo_id, $cached_id;
public $isCached;
public $mode;
public $img_type, $org_width, $org_height, $rotate;

private $src_file;
private $img_width, $img_height;
private $img_quality;
private $imageTool;

/**
 * constructor
 */
function __construct()
{
}

function setFileInformation()
{
	/**
	 * check for valid id
	 * and get img_type and rotate informations
	 */
		$query = $GLOBALS['linpha']->db->Execute("SELECT id, img_type, rotate, width, height " .
				"FROM ".LIN_PREFIX."photos " .
				"WHERE id = '".LinSql::linAddslashes( $this->photo_id )."'");

		if( $query->EOF ) // not a single value returned
		{
			$this->error_msg = "wrong id supplied!";
			return false;
		}
	
		$data = $query->FetchRow();
		
		if( ! LinSql::photoIsAllowed( $data['id'] ) )
		{
			$this->error_msg = "not permitted!";
			return false;
		}
		
		if($this->mode == 'img' && ! LinIdentify::isSupportedImage( $data['img_type'] ) )
		{
			$this->error_msg = "only images possible!";
			return false;
		}

	/**
	 * checke force parameter
	 */
		if( isset($_GET['force'] ) ) {
			$this->force = true;
		} else {
			$this->force = false;
		}

	/**
	 * set additional informations
	 */
	$this->rotate = $data['rotate'];
	$this->img_type = $data['img_type'];
	$this->org_width = $data['width'];
	$this->org_height = $data['height'];
	$this->setCachedInfos();
	
	/**
	 * check if file already exists OR if forced
	 */
	if( file_exists( $this->output_file ) AND !$this->force )
	{
		$this->isCached = true;
	}
	else // !file_exists() OR $this->force
	{
		$this->isCached = false;

		/**
		 * create subfolder if needed
		 */
		if( ! file_exists( dirname($this->output_file) ) )
		{
			if( ! @mkdir( dirname($this->output_file) ) )
			{
				$this->error_msg = "Cannot create folder ".dirname($this->output_file)." check permissions!";
				return false;
			}
		}

		/**
		 * use imagemagick or gdlib
		 */
		if($GLOBALS['linpha']->sql->config->value['sys_im_use_imagemagick'])
		{
			include_once(LINPHA_DIR.'/lib/classes/image/linpha.imagemagick.class.php');
			
			$this->imageTool = new LinImageMagick();
			$this->imageTool->imagemagickPath = $GLOBALS['linpha']->sql->config->value['sys_im_imagemagick_path'];
		}
		else
		{
			include_once(LINPHA_DIR.'/lib/classes/image/linpha.gdlib.class.php');
			
			$this->imageTool = new LinGDLib();
			$this->imageTool->imgType = $this->img_type;
		}

		$this->imageTool->imgQuality = $this->img_quality;
		$this->imageTool->orgWidth = $this->org_width;
		$this->imageTool->orgHeight = $this->org_height;
	}
	
	return true;
}

/**
 * set cached_id and output_file
 * in image mode set also rotate, watermark, quality, ...
 *  and add an empty entry in the cache database
 */
function setCachedInfos()
{
	if($this->mode == 'thumb')
	{
		$this->cached_id = $this->photo_id;

		$this->output_file = LinImage::getCachePath($this->mode,$this->cached_id,$this->img_type);
	}
	elseif($this->mode == 'img')
	{
		/**
		 * check additional settings
		 */
			/**
			 * max_width and max_height
			 */
			if(isset($this->overrideSettings['max_width']) && isset($this->overrideSettings['max_height']))
			{
				$this->max_width = $this->overrideSettings['max_width'];
				$this->max_height = $this->overrideSettings['max_height'];
			}
			else
			{
				$this->max_width = $GLOBALS['linpha']->sql->config->value['sys_style_image_width'];
				$this->max_height = $GLOBALS['linpha']->sql->config->value['sys_style_image_height'];
			}
			
			/**
			 * update rotate (printing, ...) if applicable (no else() !! )
			 * the rotate value from the database is read earlier
			 */
			if(isset($this->overrideSettings['rotate']))
			{
				$this->rotate = $this->overrideSettings['rotate'];
			}
			
			/**
			 * watermark (true|false)
			 */
			if(isset($this->overrideSettings['watermark']))
			{
				$this->watermark = $this->overrideSettings['watermark'];
			}
			else
			{
				/**
				 * @todo  place need_watermark() here
				 */
				$this->watermark = false;
			}
	
			/**
			 * quality
			 */
			if(isset($this->overrideSettings['quality']))
			{
				$this->img_quality = intval($this->overrideSettings['quality']);		// oh, do intval() for security, comes from $_GET and will be included in exec() commands
			}
			else
			{
				$this->img_quality = intval($GLOBALS['linpha']->sql->config->value['sys_style_image_quality']);
			}

		/**
		 * in image mode this is done earlier, because we need already org_width and org_height
		 * to determine if image is cached or not
		 * in thumb mode this can only be done later, because there are also folders and videos
		 */
		if( ! $this->setSourceFileInformation() )
		{
			return false;
		}

		/**
		 * this results in fliped img_width and img_height for later photos_cache
		 * compare, we take care of it below... 
		 */
		$array = $this->scaleToFit($this->org_width,$this->org_height,$this->max_width,$this->max_height);
		$this->img_width = $array['w'];
		$this->img_height = $array['h'];			

		/**
		 * check if entry in linpha_photos_cache exists
		 */
		/**
		 * Take care of fliped img_width and img_height, or it will result in a 
		 * cache hit miss (scaleToFit() (above) takes the width and height from 
		 * photos table, where values are wrong for images which need rotate)
		 */	
		$rot = $GLOBALS['linpha']->db->GetRow("SELECT is_rotated, img_width, img_height " .
				"FROM ".LIN_PREFIX."photos_cache " .
				"WHERE photo_id = '".$this->photo_id."' ");

		if(isset($rot['is_rotated']) && $rot['is_rotated'] != 0)
		{
			
			$dummy_height = $this->img_height;
			$this->img_height = $this->img_width;
			$this->img_width = $dummy_height;
			linLog(LOG_TYPE_IMPORT,LOG_INFO,'debug',"Flipping width and height to get cache hit for image # ".$this->photo_id);
		}
		 
		$data = $GLOBALS['linpha']->db->GetRow("SELECT id FROM ".LIN_PREFIX."photos_cache WHERE " .
				"photo_id = '".$this->photo_id."' AND " .
				"img_width = '".$this->img_width."' AND " .
				"img_height = '".$this->img_height."' AND " .
				"img_quality = '".$this->img_quality."' AND " .
				"is_rotated = '".$this->rotate."' AND " .
				"has_watermark = '".$this->watermark."'");

		if(isset($data['id']) && !empty($data['id']))
		{
			linLog(LOG_TYPE_IMPORT,LOG_INFO,'debug',"Cache HIT for image # ".$this->photo_id);

			$this->cached_id = $data['id'];
			$this->output_file = LinImage::getCachePath('img',$this->cached_id,$this->img_type);
		}
		else
		{
			linLog( LOG_TYPE_IMPORT,LOG_INFO,'image',"Cache MISS for image # ".$this->photo_id);
			
		
			/**
			 * insert an empty entry and get cached_id
			 * we will update correct values only if create cached image has been successfull
			 */
			$GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."photos_cache " .
					"(photo_id) VALUES ('".$this->photo_id."')");
	
			$this->cached_id = $GLOBALS['linpha']->db->GetOne("SELECT MAX(id) " .
					"FROM ".LIN_PREFIX."photos_cache ");			
			$this->output_file = LinImage::getCachePath('img',$this->cached_id,$this->img_type);
			
			if(file_exists($this->output_file))
			{
				/**
				 * o-oh, we're leaking old files in cache path, -> delete it
				 */
				unlink($this->output_file);
			}
		}
	}

	
}

/**
 * set source file informations
 * 
 * @todo  save values to database and read from there
 */
function setSourceFileInformation()
{
	if($this->img_type != 0 && $this->img_type != 9999999)
	{
		$this->src_file = LinSql::getFullImagePath( $this->photo_id );

		if(! file_exists($this->src_file))
		{
			echo $this->src_file.' ';
			echo i18n("File doesnt exist anymore").'. ';
			echo i18n("You should consider start a re-import of your files/folders").'<br />';
			return false;
		}
	}
	
	return true;
}


/**
 * thumbnail functions
 * 
 * @return  true on success, false on failure (checks for valid id, valid image, valid permission!)
 */
	function createThumbnail($photo_id)
	{
		$this->mode = 'thumb';
		$this->photo_id = $photo_id;

		if( ! $this->setFileInformation() )
		{
			return false;
		}
		
		if( ! $this->isCached )
		{
			if( ! $this->setSourceFileInformation() )
			{
				return false;
			}
	
			linLog(LOG_TYPE_IMPORT,LOG_INFO,'thumbnail',"Creating thumbnail ".$this->photo_id);

			/**
			 * image thumbnail
			 */
			if( LinIdentify::isSupportedImage( $this->img_type ) )
			{
				$this->createThumbnailImage();
			}
			/**
			 * folder thumbnail
			 */
			elseif($this->img_type == 0)
			{
				if( ! $this->createThumbnailFolder() )
				{
					$this->output_file = LINPHA_DIR.'/lib/graphics/subfolder_image.png';
				}
			}
			/**
			 * video thumbnail
			 */
			elseif( LinIdentify::isVideo( $this->img_type ) )
			{
				$this->createThumbnailVideo();
			}

			if( ! file_exists( $this->output_file ) )
			{
				$this->error_msg = "Output File Doesnt Exists";
				return false;
			}
		}	
	
		return true;
	}
	
	function createThumbnailImage()
	{
		/**
		 * include the file which contains the imagemagick strings and exec() / gdlib instructions
		 */
	    $this->imageTool->img_thumbsize = $GLOBALS['linpha']->sql->config->value['sys_style_thumb_size_max'];
	    $this->imageTool->imgQuality = 75;

		$this->imageTool->createThumbnail($this->src_file,$this->output_file,$this->rotate);
	}
	
	function createThumbnailFolder()
	{
		/**
		 * check if we have already some thumbnails in this folder
		 * and create array_ids which holds 4 image ids
		 */
		$query = $GLOBALS['linpha']->db->Execute("SELECT id, img_type FROM ".LIN_PREFIX."photos WHERE parent_id = '".LinSql::linAddslashes( $this->photo_id )."'");
		$array_thumbs = Array();
		$i=0;
		while($data = $query->FetchRow())
		{
			if($i >= 4)
			{
				break;
			}
			
			$file = LinImage::getCachePath('thumb',$data['id'],$data['img_type']);

			if( file_exists( $file ) )
			{
				if($data['1']=='0') {	// folders are gif files, all others are jpg
					$type = '1';
				} else {
					$type = '2';
				}

				$array_thumbs[] = array('id' => $data['id'],'file' => $file, 'type' => $type);
				$i++;
			}
		}
	
		/**
		 * no images found in this folder
		 */
		if(count($array_thumbs) == 0)
		{
			return false;
		}

		$this->imageTool->createThumbnailFolder($this->output_file,$array_thumbs);
		
		if( !file_exists($this->output_file))
		{
			return false;
		}
		else
		{
			return true;
		}
	}
	
	function createThumbnailVideo()
	{
		$dummy_thumbnail = LINPHA_DIR.'/lib/graphics/avi_mov.gif';
	    $this->img_thumbsize = $GLOBALS['linpha']->sql->config->value['sys_style_thumb_size_max'];
	    $this->img_quality = 75;
	
		/**
		* check if we can get a thumbnail
		*/
			$found_thumbnail = false;
	
			/**
			* check if there exists a .thm file
			* (video thumbnail of Canon videos)
			*/
			$pos = strrpos($this->src_file,'.');
			$start = substr($this->src_file,0,$pos);
			$thm_file = $start.'.thm';
			if(file_exists($thm_file))
			{
				list($this->org_width, $this->org_height, $file_type) = LinIdentify::linGetImageSize($thm_file);
				$this->src_file = $thm_file;

				$this->imageTool->orgWidth = $this->org_width;
				$this->imageTool->orgHeight = $this->org_height;
				$this->imageTool->imgType = $file_type;
				$this->imageTool->img_thumbsize = $this->img_thumbsize;
				$this->imageTool->imgQuality = 75;
				$this->imageTool->createThumbnail($this->src_file,$this->output_file,$this->rotate);
				$found_thumbnail = true;
			}
			
			if(!$found_thumbnail)
			{
				/**
				* try to get the tumbnail with convert
				*/
				if($GLOBALS['linpha']->sql->config->value['sys_im_use_imagemagick'] &&
					$GLOBALS['linpha']->sql->config->value['sys_im_video_thumbnail'])
				{
					/**
					* only if codec is mjpg, because otherwise we get an access
					* violation message box on the servers desktop of convert.exe
					* (already reported to imagemagick http://studio. imagemagick.
					* org/magick/viewtopic.php?t=2868)
					*/
					include_once(LINPHA_DIR.'/lib/classes/getid3/getid3.php');
					define('GETID3_HELPERAPPSDIR', LINPHA_DIR); // needs to be set to a valid dir, otherwise it doesn't work under windows
					$getID3 = new getID3;
					$file_info = $getID3->analyze($this->src_file);
	
					if(isset($file_info['video']['codec']))
	                {
	                    $codec = strtolower($file_info['video']['codec']);
	                    if(strpos($codec, "mjpeg") !== false 
	                        OR strpos($codec, "motion jpeg") !== false
	                        OR strpos($codec, "motion jpg") !== false
	                        OR strpos($codec, "mjpg") !== false)
	                    {
							$return_value = $this->imageTool->createThumbnailVideo($this->src_file,$this->output_file,$this->img_thumbsize,$this->img_thumbsize);
							
							if($return_value == 0 && file_exists($this->output_file)) /* && empty($array_output[0]) */
							{
								$found_thumbnail = true;
							}

	                    }
	                }
				}
			}
		
		/**
		* copy the video icon in the lower right corner
		*/
		if($found_thumbnail)
		{
			include_once(LINPHA_DIR.'/lib/plugins/watermark/func.watermark.php');
			$this->imageTool->wm_config = Array(
				'wm_watermark' => 2,
				'wm_align' => 'southeast',
				'wm_dissolve' => 100,
				'wm_img_img' => $dummy_thumbnail,
				'wm_resize' => 'no',	// use 'no' because the gdlib cannot handle transparent images with imagecopyresized() !!
				'wm_horizontal' => 5,
				'wm_vertical' => 5
			);

	        /**
	         * warning: session will be closed!
	         */
			$this->imageTool->orgWidth = $this->img_thumbsize; // only used with gdlib
			$this->imageTool->orgHeight = $this->img_thumbsize; // only used with gdlib

			$this->imageTool->createImage($this->output_file,$this->output_file,$this->img_thumbsize,$this->img_thumbsize,$this->rotate,$watermark=1);
		}
		else
		{
			$this->output_file = $dummy_thumbnail;
		}
	}


/**
 * image functions
 */
	function createImage($photo_id, $overrideSettings)
	{
		$this->mode = 'img';
		$this->photo_id = $photo_id;
		
		$this->overrideSettings = $overrideSettings;

		if( ! $this->setFileInformation() )	// will also do setCachedInfos() and setSourceFileInformation()
		{
			return false;
		}
		
		if( ! $this->isCached OR $this->force )
		{
			linLog(LOG_TYPE_IMPORT,LOG_INFO,'image',"Creating image ".$this->photo_id);
			
			/**
			 * include the create image script
			 */
			linStartTimer("createimage");
			
			$this->imageTool->createImage($this->src_file,$this->output_file,$this->img_width,$this->img_height,$this->rotate,$this->watermark);

			$time = linStopTimer("createimage") * 1000000;
			
			//echo $time; exit();

			if( ! file_exists( $this->output_file ) )
			{
				$this->error_msg = "Output File Doesnt Exists";
				return false;
			}

			/**
			 * finish cache entry
			 */
			$filesize = filesize( $this->output_file );
			if($filesize <= 0)
			{
				$this->error_msg = "Incorrect filesize (".$filesize.")";
				return false;
			}

			/**
			 * If image needs rotating (by EXIF value) flip width and height 
			 * to have the right values in photos_cache
			 */
			if($this->rotate != 0)
			{
			$dummy_height = $this->img_height;
			$this->img_height = $this->img_width;
			$this->img_width = $dummy_height;
			linLog(LOG_TYPE_IMPORT,LOG_INFO,'image',"Height/width flip during storage of image # ".$this->photo_id." in photos_cache" );
				
			}
			$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos_cache SET " .
					"img_width = '".$this->img_width."', " .
					"img_height = '".$this->img_height."', " .
					"img_quality = '".$this->img_quality."', " .
					"img_size = '".$filesize."', " .
					"is_rotated = '".$this->rotate."', " .
					"has_watermark = '".$this->watermark."', " .
					"time_add = '".time()."', " .
					"time_creating = '".$time."' " .
				"WHERE id = '".$this->cached_id."'");
		}
		else
		{
			/**
			 * got a cached copy, increment counter and time_use
			 */
			$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos_cache " .
					"SET hits = hits + 1, time_use = '".time()."' " .
					"WHERE id = '".$this->cached_id."'");
		}

		return true;
	}


/**
 * Rotates images when called from menu direction left/right.
 * Insert rotation value into linpha_photos and flip image width 
 * and height in table.
 * Later do a force recreate
 * 
 * @param string $direction either 90 (right turn) or -90 (left turn)
 * @param int $linID
 *   
 */
  function rotateImageByMenuCall($direction, $linId)
 {
 	/**
 	 * Insert ration value in linpha_photos
 	 */
	 $rotate = $GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos " .
					"SET rotate='".LinSql::linAddslashes($direction)."' " .
					"WHERE id='".$linId."'");
 
 	/**
 	 * Flip image width and height
 	 */
 	 $imagedata = $GLOBALS['linpha']->db->GetRow("SELECT width AS width, height AS height " .
 	 				"FROM ".LIN_PREFIX."photos " .
					"WHERE id='".LinSql::linAddslashes($linId)."' ");
	 
 	 $flip = $GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos SET " .
					"width = '".LinSql::linAddslashes($imagedata['height'])."', " .
					"height = '".LinSql::linAddslashes($imagedata['width'])."' " .
					"WHERE id='".LinSql::linAddslashes($linId)."'");
 	
 	/**
 	 * force recreate with new values
 	 */
 	 $thumbnail = new LinImage();
	 $_GET['force'] = true;
	 $thumbnail->createThumbnail( $linId );

 }


/**
 * returns the full path to the cached image
 * in thumb mode, $id is the photoid
 * in image mode, $id is the cached photoid
 * 
 * @param string $what 'thumb' or 'img'
 * @param int $id image id
 * @return string full path to cached image
 * 
 * @todo  use img_type as parameter and decide in this function
 *        which file extension to use
 */
function getCachePath($what,$id,$img_type)
{
	/**
	 * thumbnails of folders are gif files, all others are (currently at least) jpg files
	 */
	if($what == 'thumb' && $img_type == '0') {
		$ext = 'gif';
	} else {
		$ext = 'jpg';
	}
	
	$path = LinSql::getPath('cache');
	
	if(strlen($id)==1)
	{
		$subdir = $id.'0';	// dont use foldernames 1-9 because it would only contain one image per folder
	}
	else
	{
		$subdir = substr($id,0,2);
	}
	
	return $path . '/' . $what . '/' . $subdir . '/' . $id . '.' . $ext;
}

/**
 * deleteCached thumbnail by id
 * 
 * @param int $id id of thumb in database
 * @return boolean false if there was no image, true if there was an image to delete
 */
function deleteCachedThumbnail($id,$img_type)
{
	$output = LinImage::getCachePath('thumb',$id,$img_type);
	
	if(file_exists($output))
	{
		if( ! @unlink( $output ) )
		{
			linSysLog( i18n("Warning while deleting file/folder, check permissions!").'<br />'.$output );
		}
		
		return true;
	}
	else
	{
		return false;
	}
}

/**
 * delete all cached images by this photo id
 * (there can be more than one image with the same photo id in different sizes etc.)
 * 
 * @param int $id id of image in database
 */
function deleteCachedImage($id,$img_type)
{
	$query = $GLOBALS['linpha']->db->Execute("SELECT id FROM ".LIN_PREFIX."photos_cache WHERE photo_id = '".$id."'");
	while($data = $query->FetchRow(ADODB_FETCH_NUM))
	{
		$output = LinImage::getCachePath('img',$data[0],$img_type);
		
		if(file_exists($output))
		{
			if( ! @unlink( $output ) )
			{
				linSysLog( i18n("Warning while deleting file/folder, check permissions!").'<br />'.$output );
			}
		}
	}
}

/**
 * This returns returns width and height of the scaled image
 * to fit in $dst_h and $dst_w, it keeps the image ratio
 * calculated from $src_h and $src_w
 * if $no_increase is set, the image is only decreased
 */
function scaleToFit($src_w,$src_h,$dst_w,$dst_h,$no_increase=1)
{
	if($src_h == 0 OR $src_w == 0)
	{
		return array('w' => 0, 'h' => 0);
	}

	$img_relation = $src_w/$src_h;

	// Image is smaller than screen, no resize required
	if (($src_w <= $dst_w) && ($src_h <= $dst_h) && $no_increase)
	{
		$array['w'] = $src_w;
		$array['h'] = $src_h;
	}
	else
	{
		/*
		The image is way bigger than the screen, resize maintaining aspect ratio

		$src_w, $src_h: original image sizes
		$dst_w, $dst_h: screen width and height
		$img_relation = $src_w/$src_h;
	
		Either $dst_w decisive or $dst_h, usually $dst_h 
		*/

		$tmp_height = $dst_w / $img_relation;		// calc the new height with screen width
		if ($tmp_height > $dst_h) {			// we were wrong, it's still widther than screen -> $dst_h is decisive
			$array['w'] = round($img_relation*$dst_h);
			$array['h'] = intval($dst_h);	// oh, do intval() for security, comes from $_GET and will be included in exec() commands
		} else {
			$array['w'] = intval($dst_w);	// oh, do intval() for security, comes from $_GET and will be included in exec() commands
			$array['h'] = round($tmp_height);
		}
	}
	
	return $array;
}

/**
 * send correct header and file
 */
function printFileToScreen()
{
	if( $this->img_type == 0 )		// folder are gif images, all other are jpg
	{
		header("Content-type: image/gif");
	}
	else
	{
		header("Content-type: image/jpeg");
	}
	
	/**
	 * show correct image filename wenn selecting "save as" in browser
	 * it works on firefox, but it does not on my IE
	 */
	header("Content-Disposition: inline; filename=".basename($this->src_file).";" );
	
	readfile( $this->output_file );
}

/**
 * rotates an image with a given angle using GDLib
 * do nothing if imagerotate() doesn't exists
 * 
 * 
 * @param imageressource $src_img pointer to the source image
 * @param int $angle angle to rotate
 * @author flo
 */
function GDimageRotate(& $src_img, $angle)
{
	if($angle != "0")
	{
		if(function_exists('imagerotate'))
		{
		    /**
		    * flip $angle, because with this function, left means right, and right means left :-)
		    */
		    $angle = 360 - $angle;
			$angle = $angle % 360;
			$src_img = imagerotate($src_img,$angle,0);
		}
		else
		{
			linLog(LOG_TYPE_META,LOG_WARNING,'rotate',"Cannot rotate image becuase the function imagerotate() doesnt exists, consider upgrading your php version");
		}
	}
}

function &GDimageCreateFrom($src_file,$img_type)
{
	switch($img_type)
	{
		case 1: $src_image = imagecreatefromgif($src_file); break;
		case 2: $src_image = imagecreatefromjpeg($src_file); break;
		case 3: $src_image = imagecreatefrompng($src_file); break;
		default: $foobar = false; return $foobar; break;
	}
	
	return $src_image;
}
	
} // end class linImage
?>