<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class takes automagic care of added/deleted images and albums 
 * @package Image
 */
class LinImport
{

/**
 * update selected directory
 * add new dirs/files, delete old ones
 * if recursive, all sub directories are checked too!
 * @param int $parent_id Parent id of directory in database
 * @param bool $recursive scan directory recursive if true
 * @param bool $dryrun do actually nothing, just report
 */
function updateDir($parent_id, $recursive, $dryrun)
{
	try
	{
		if($parent_id == 0)
		{
			$dirname = LinSql::getPath('album'); 
		}
		else
		{
			$dirname = LinSql::getFullImagePath( $parent_id );
		}

		if( ! file_exists( $dirname ) )
		{
			throw new Exception("Directory ".$dirname." doesnt exists!<br />" .
					"Please start a complet re-importing of your files/folders");
		}

		/**
		 * read current directory in array
		 */	
			$importarray_folder = Array();
			if( $d = @dir( $dirname ) )
			{
				while( false !== ($entry = $d->read() ) )
				{
					if($entry != '.' AND $entry != '..')
					{
						$importarray_folder[] = $entry;
					}
				}
				$d->close();
			}
		
		/**
		 * read directory from database in array
		 */
			$importarray_db = Array();
			$query = $GLOBALS['linpha']->db->Execute("SELECT id, name " .
					"FROM ".LIN_PREFIX."photos WHERE parent_id = '".$parent_id."'");
			while($data = $query->FetchRow())
			{
				$importarray_db[$data['id']] = $data['name'];
			}
		
		/**
		 * compare arrays
		 */
			$array_new = array_diff($importarray_folder, $importarray_db);
			$array_delete = array_diff($importarray_db, $importarray_folder);
			$array_update = array_intersect($importarray_db,$importarray_folder);

		/**
		 * parse deleted entries
		 */
		foreach($array_delete AS $key=>$value)
		{
			if($dryrun)
			{
				linSysLog(i18n("Delete Entry").": ".htmlspecialchars($dirname.'/'.$value,ENT_QUOTES));
				$deleted_keys[] = $key;
			}
			else
			{
				LinImport::deleteEntry( $key, $dirname.'/'.$value);
			}
		}

		/**
		 * parse new entries
		 */
		foreach($array_new AS $value)
		{
			if($dryrun)
			{
				linSysLog(i18n("New Entry").": ".htmlspecialchars($dirname.'/'.$value,ENT_QUOTES));
			}
			else
			{
				LinImport::addNewEntry( $parent_id , $dirname , $value, $recursive );
			}			
			
		}
		
		/**
		 * "detect file changes", parse updated entries
		 * not by default! (performance issues)
		 * and only admin can do that
		 */
		if( isset($_REQUEST['force_update']) && $GLOBALS['linpha']->sql->isAdmin() )
		{
			foreach($array_update AS $key=>$value)
			{
				LinImport::updateEntry( $key , $dirname , $value );
			}
		}
		
		/**
		 * if recursive, parse all existing directories too
		 */
		if($recursive)
		{
			$query = $GLOBALS['linpha']->db->Execute("SELECT id " .
					"FROM ".LIN_PREFIX."photos WHERE img_type = '0' " .
					"AND parent_id = '".$parent_id."'");
			while($data = $query->FetchRow(ADODB_FETCH_NUM))
			{
				/**
				 * on dryrun, the delete process isn't done completley
				 * so it would appear here again, prevent this!
				 */
				if( !isset($deleted_keys) OR !in_array($data['0'],$deleted_keys) )
				{
					LinImport::updateDir($data['0'], $recursive, $dryrun);
				}
			}
		}
	}
	catch(Exception $error)
	{
		linSysLog(i18n("Error during import files/folders").": ".$error -> getMessage());
		linLog(LOG_TYPE_IMPORT,LOG_ERR,'import_others',"Error during import files/folders: ".$error -> getMessage());
	}
}


/**
 * import new file, calc md5sum, delete thumbnail if one already exists 
 * with the same id, update number of photo of parent folder
 * @param int $parent_id parent id of folder in database
 * @param string $dirname name of current directory
 * @param string $filename filename of file currently processed
 */
function addNewEntry( $parent_id , $dirname , $filename , $recursive )
{
	linSysLog(i18n("Import").': '.i18n("New").': '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES));
	linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_new',"New: ".$dirname.'/'.$filename);


	list($md5sum,$file_type,$org_width,$org_height,$stats_number,$time_exif,$rotate,$filesize) =
		LinImport::getFileInformation($dirname,$filename);

    $GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."photos ( " .
		"parent_id, name, img_type, md5sum, width, height, filesize, time_add, time_mod, time_exif, rotate, stats_numbers ) " .
		"VALUES ( '".$parent_id."' , '".$GLOBALS['linpha']->sql->linAddslashes($filename)."', '".$file_type."'," .
		"'".$md5sum."' , '".$org_width."' , '".$org_height."' , '".$filesize."', '".time()."', " .
		"'".filemtime($dirname.'/'.$filename)."', '".$time_exif."', '".$rotate."', '".$stats_number."' )");

	/**
	 * delete thumbnail if one exists with the same id
	 */
	$inserted_id = $GLOBALS['linpha']->db->GetOne("SELECT MAX(id) FROM ".LIN_PREFIX."photos ");

	LinImage::deleteCachedThumbnail($inserted_id,$file_type);

	/**
	 * update number of photos
	 */
	LinImport::updateNumberPhotos( $parent_id );


	if($recursive && $file_type == 0)
	{
		LinImport::addNewSubDir( $inserted_id , $dirname.'/'.$filename , $recursive);
	}
}

/**
 * add new subdir/subfile
 * only called from addNewDir()
 * @param int $parent_id parent id of folder in database
 * @param string $dirname name of current directory
 * @param bool $recursive scan directory recursive if true 
 */
function addNewSubDir( $parent_id , $dirname , $recursive)
{
	$d = dir( $dirname );
	while( false !== ($entry = $d->read() ) )
	{
		if($entry != '.' AND $entry != '..')
		{
			LinImport::addNewEntry( $parent_id , $dirname , $entry, $recursive );
		}
	}
	$d->close();
}

/**
 * recursive delete entries
 * important: first delete deepest entry -> on errors or user aborts the database keeps consistent
 * do not delete comments and meta_data, they keep in database
 * 
 * @param int $photo_id id of current image in database
 * @param string $full_filename full path to file, can directly be used in unlink() etc.
 */
function deleteEntry( $photo_id, $full_filename)
{
	/**
	 * delete subentries
	 */
	$query = $GLOBALS['linpha']->db->Execute("SELECT id, name " .
			"FROM ".LIN_PREFIX."photos WHERE parent_id = '".$photo_id."' ");
	while($data = $query->FetchRow())
	{
		LinImport::deleteEntry( $data['id'], $full_filename.'/'.$data['name']);
	}

	$data = $GLOBALS['linpha']->db->GetRow("SELECT img_type, md5sum FROM ".LIN_PREFIX."photos WHERE id = '".$photo_id."'");


	linSysLog(i18n("Import").': '.i18n("Delete").': '.htmlspecialchars($full_filename,ENT_QUOTES));
	linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_delete',"Delete: ".$full_filename);

	
	/**
	 * delete thumbnail and image in cache folder
	 * even delete folder thumbnail
	 */
	LinImage::deleteCachedThumbnail($photo_id,$data['img_type']);
	LinImage::deleteCachedImage($photo_id,$data['img_type']);

	/**
	 * delete metadata (exif,iptc,xmp)
	 */
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_exif WHERE md5sum = '".$data['md5sum']."'");
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_iptc WHERE md5sum = '".$data['md5sum']."'");
	//$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_xmp WHERE md5sum = '".$data['md5sum']."'");

	/**
	 * delete permission
	 */
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."permissions WHERE photos_id = '".$photo_id."'");

	/**
	 * and at last: delete the entry (keeps database consistent if we do this at last!)
	 */
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."photos " .
			"WHERE id = '".$photo_id."' ");

}

/**
 * update entry "detect file changes"(check md5sum and file_type)
 * this has nothing todo with the updateDir() method
 * 
 * (the file_type changes if a file changes to readable)
 */
function updateEntry( $id , $dirname , $filename )
{
	list($md5sum,$file_type,$org_width,$org_height,$stats_number,$time_exif,$rotate,$filesize)
		= LinImport::getFileInformation($dirname,$filename);
	
	$data = $GLOBALS['linpha']->db->GetRow("SELECT img_type, md5sum " .
			"FROM ".LIN_PREFIX."photos WHERE id = '".$id."'");
	
	/**
	 * file type changed (not the file content) -> update file_type
	 * 
	 * on change of file_type, change:
	 * - img_type
	 * - width and height
	 * - time_exif
	 * - stats_numbers
	 */
	if($file_type != $data['img_type'])
	{
		linSysLog(i18n("Import").': '.i18n("Detected a change of file type, update entry of:").' '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_update',"Detected a change of file type, update entry of: ".$dirname.'/'.$filename);

		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos " .
				"SET img_type = '".$file_type."', width = '".$org_width."', height = '".$org_height."', time_exif = '".$time_exif."', stats_numbers = '".$stats_number."' " .
			"WHERE id = '".$id."'");
	}
	
	/**
	 * file data change -> update md5sum
	 * 
	 * on change of md5sum, change:
	 * - width and height
	 * - filesize
	 * - time_exif, time_mod
	 * - rotate (maybe the file got an orientation tag, or the file has been autorotated with an external program)
	 */
	/**
	 * @todo  hey, thats a nice feature! we are now able to detect
	 * changes on images and update the md5sum of the images in the comments etc.!!
	 */
	if($md5sum != $data['md5sum'])
	{
		linSysLog(i18n("Import").': '.i18n("Detected a change of file data, update entry of:").' '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_update',"Detected a change of file data, update entry of: ".$dirname.'/'.$filename);

		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos SET " .
			"md5sum = '".$md5sum."', width = '".$org_width."', height = '".$org_height."', " .
			"filesize='".$filesize."', time_mod = '".filemtime($dirname.'/'.$filename)."', " .
			"time_exif = '".$time_exif."', rotate = '".$rotate."' " .
			"WHERE id = '".$id."'");
		
		LinImport::updateMd5sum( $id, $data['md5sum'], $md5sum );

		LinImage::deleteCachedImage($id, $data['img_type']);	// they will automatically be recreated
		LinImage::deleteCachedThumbnail($id, $data['img_type']);
	}
}

/**
 * if the md5sum changes, update the md5sum in linpha_stats, linpha_meta_data
 * occurs while rotating images, or we edited the iptc data,
 * or if LinImport detects a change of the file
 * (for example: image has been edited with an external program)
 * 
 * problem: if there are two images with the same md5sum, and one image gets changed
 * linpha will update the md5sum of the changed image for the comments (for example)
 * to the new value, but if we then do a "detect file changes" on the old image,
 * the old image will do the same and change the md5sum of the comment back to this
 * this will create a race condition 
 * -> thats not true ?! the "detect file changes" on the old image will not detect
 *    any changes, because there are no changes!
 */
function updateMd5sum( $id, $old_md5sum, $new_md5sum )
{
	$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."meta_comments SET " .
		"md5sum = '".$new_md5sum."' WHERE md5sum = '".$old_md5sum."'");

	$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."meta_data SET " .
		"md5sum = '".$new_md5sum."' WHERE md5sum = '".$old_md5sum."'");


	/**
	 * if the file changed, it is best to re-index image
	 * -> delete/update exif/iptc/xmp data!
	 */
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_exif WHERE md5sum = '".$old_md5sum."'");
	$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_iptc WHERE md5sum = '".$old_md5sum."'");
	
}

/**
 * update recursively number of photos in parent directories
 * @param int $id id of current image in database
 */
function updateNumberPhotos( $id )
{
	if( $id != 0 )
	{
		$data = $GLOBALS['linpha']->db->GetRow("SELECT sum(stats_numbers) " .
				"FROM ".LIN_PREFIX."photos WHERE parent_id = '".$id."'");
		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."photos " .
				"SET stats_numbers = '".$data[0]."' WHERE id = '".$id."'");    	
		
		$data = $GLOBALS['linpha']->db->GetRow("SELECT parent_id " .
				"FROM ".LIN_PREFIX."photos WHERE id = '".$id."'");
		LinImport::updateNumberPhotos( $data['parent_id'] );
	}
}

/**
 * returns an array containing md5sum, file_type and number of images
 * number of images: 0 default, 1 if is_video or is_supported_image
 */
function getFileInformation($dirname,$filename)
{
	/**
	 * initial settings
	 * will be overwritten if necessary
	 */
	$md5sum = '';
	$stats_number = 0;
	$time_exif = "0";
	$rotate = "0";
	$org_width = "0";
	$org_height = "0";
	$filesize = 0;

	$ext = LinFilesys::getFileExtFromPath($filename);
	if($filename{0} == '.')
	{
		linSysLog(i18n("Import").': '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES)." ".i18n("Is Hidden"));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_fileinfo',$dirname.'/'.$filename." Is Hidden");
		$file_type = 9999999;
	}
	elseif( in_array( $filename, LinImport::getIgnoreFilelist() ) )
	{
		linSysLog(i18n("Import").': '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES)." ".i18n("Is In The Ignore List"));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_fileinfo',$dirname.'/'.$filename." Is In The Ignore List");
		$file_type = 9999999;
	}
	elseif( in_array($ext, LinImport::getIgnoreFileext() ) )
	{
		linSysLog(i18n("Import").': '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES)." ".i18n("Is In The Ignore List Of File Types"));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_fileinfo',$dirname.'/'.$filename." Is In The Ignore List Of File Types");
		$file_type = 9999999;
	}
	elseif( !is_readable( $dirname.'/'.$filename) )
	{
		linSysLog(i18n("Import").': '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES)." ".i18n("Is Not Readable"));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_fileinfo',$dirname.'/'.$filename." Is Not Readable");
		$file_type = 9999999;
	}
	/**
	 * no need for is_link as links are detected correctly 
	 * with is_file, is_dir and is_readable !
	 * elseif( is_link ($dirname.'/'.$value ) )
	{
		linSysLog('link');
	}*/
	elseif( is_dir( $dirname.'/'.$filename ) )
	{
		$file_type = 0;
		$md5sum = md5($dirname.'/'.$filename);
	}
	elseif( is_file( $dirname.'/'.$filename ) )
	{
	    list($org_width,$org_height,$file_type) = LinIdentify::linGetImageSize($dirname.'/'.$filename);
	    $filesize = filesize($dirname.'/'.$filename);
	
	    if( LinIdentify::isVideo($file_type) OR LinIdentify::isSupportedImage($file_type) )
	    {
		    /**
		     * calculate md5sum
		     * we dont have memory limit for videos anymore with the builtin md5_file() (tested !)
		     * 
		     * the calculation of a over 200MB file is done in less than 2 seconds on a newer computer
		     */
		    $md5sum = md5_file($dirname.'/'.$filename);
	    	$stats_number = 1;
	    }
	    
	    if( LinIdentify::isSupportedImage($file_type) )
	    {
			/**
			 * exif date and exif rotate stuff
			 * globally turn off using 'sys_import_exif' if having problems with pjmt
			 * 
			 * @todo  index all exif tags if we already have read the meta data!
			 *        but should we also read iptc at this point?
			 */
			if( $GLOBALS['linpha']->sql->config->value['sys_import_exif'] )
			{
			 	include_once( LINPHA_DIR.'/lib/classes/pjmt/EXIF.php' );
				$meta = get_EXIF_JPEG($dirname.'/'.$filename);
				
				/**
				 * get date
				 */
				if( isset($meta[0][34665]['Data'][0][36867]['Data'][0]) ) {
					$time_exif = LinMetaDataIndex::convertExifDateToUnix( $meta[0][34665]['Data'][0][36867]['Data'][0] );
				}

				/**
				 * get rotate information
				 * turn off exif autorotate using 'sys_import_exif_autorot'
				 */
				if( $GLOBALS['linpha']->sql->config->value['sys_import_exif_autorot'] && isset($meta[0][274]['Data'][0]) )
				{
					$rotate = LinMetaDataIndex::getRotationByExifTag( $meta[0][274]['Data'][0] );
				}
			}
	    }		
	}
	else
	{
		linSysLog(i18n("Import").': Error: what am i? '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES));
		linLog(LOG_TYPE_IMPORT,LOG_NOTICE,'import_fileinfo','Error: what am i? '.htmlspecialchars($dirname.'/'.$filename,ENT_QUOTES));
	}

	return array($md5sum,$file_type,$org_width,$org_height,$stats_number,$time_exif,$rotate,$filesize);
}

/**
* get an array with ignored filenames
*
* @author  flo
* @return  Array  array with filenames
*/
function getIgnoreFilelist()
{
	return Array(
		'CVS',				// CVS
		'Thumbs.db',		// Windows
		'ZbThumbnail.info',	// ?
		'_vti_cnf',			// IIS
		'_derived',			// IIS
		'lost+found'		// Linux
	);

}


/**
* get an array with ignored file extensions
*
* @author  flo
* @return  Array  array with file extensions
*/
function getIgnoreFileext()
{
	return Array(
	'thm'	// jpg thumbnails of videos on Canon cameras 
			//(this thumbnail is automatically taken as thumbnail of the video itself)
	);
}

} // end class linImport
?>
