<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class handles all the EXIF and IPTC stuff used in LinPHA
 * @package Metadata
 * @author  flo, bzrudi
 */
 
if(!defined('LINPHA_DIR')) { exit(1); }

/**
 * Functions to read MetaData from the database
 */
class LinMetaDataRead extends LinMetaData
{
	public $cachedFieldNames, $cachedFieldIds, $cachedSelectedFieldNames;
	
	/**
	 * read image information from database
	 * and escape data with htmlspecialchars() if necessary
	 * 
	 * @param array array_src_infos  array containing needed source file informations, same as LIN_SELECT_FIELDS
	 * 
	 */
	function readInformations($array_src_infos, $flag_nr)
	{
		/**
		 * cache field names if not already done
		 * readInformations() is executed for each thumbnail on a folder
		 * save a lot of time if do it only once!
		 */
		if( !isset($this->cachedSelectedFieldNames) OR !is_array($this->cachedSelectedFieldNames))
		{
			$query = $GLOBALS['linpha']->db->Execute("SELECT name " .
					"FROM ".LIN_PREFIX."meta_fields " .
					"WHERE flags = '".$flag_nr."' ORDER by id");
					
			while($fields_data = $query->FetchRow(ADODB_FETCH_NUM))
			{
				$this->cachedSelectedFieldNames[] = $fields_data['0'];
			}
		}
		
		foreach($this->cachedSelectedFieldNames AS $field_name)
		{
			$name = $this->getNameOfField( $field_name, false );
			$value = '';
	
			$pos = strpos($field_name,'_');
			$first_part = substr($field_name,0,$pos);
			$last_part = substr($field_name,$pos+1);
					
			switch($first_part)
			{
			case 'builtin':
	
				switch($last_part)
				{
					case 'filename':
						$value = $array_src_infos['name'];		// is already escaped
						break;
					case 'imagesize':
						$value = LinFilesys::niceFilesize( $array_src_infos['filesize'], 2 );
						break;
					case 'dimension':
						$value = $array_src_infos['width'].'x'.$array_src_infos['height'];
						break;
					case 'thumbnail':
						$value = 'thumbnail';
						break;
					case 'time_add':
						$name = i18n("Date Added");
						$value = linStrftime($array_src_infos['time_add']);
						break;
					case 'time_mod':
						$name = i18n("Date Modified");
						$value = linStrftime($array_src_infos['time_mod']);
						break;
					case 'time_exif':
						$name = i18n("Date Taken");
						$value = linStrftime($array_src_infos['time_exif']);
						break;
					case 'stats_numbers':
						$name = i18n("Photos");
						$value = $array_src_infos['stats_numbers'];
						break;
					case 'stats_views':
						$name = i18n("Nr Of Views");
						$value = $array_src_infos['stats_views'];
						break;
					case 'stats_downloads':
						$name = i18n("Nr Of Downloads");
						$value = $array_src_infos['stats_downloads'];
						break;
					case 'first_comment':
						$name = i18n("First Comment");
						$value = linHtmlTag(
							$GLOBALS['linpha']->db->GetOne(
							"SELECT meta_comment FROM ".LIN_PREFIX."meta_comments " .
							"WHERE md5sum = '".LinSql::linAddslashes($array_src_infos['md5sum'])."' " .
							"ORDER by meta_time ASC")
						);
						break;
	
					default:
						/**
						 * description, category, persons, date	(field entries which are stored in linpha_meta_data)
						 */
							/**
							 * get field id, and cache it
							 */
							if( !isset($this->cachedFieldIds[$last_part]) )
							{
								$this->cachedFieldIds[$last_part] = $GLOBALS['linpha']->db->GetOne("SELECT id " .
									"FROM ".LIN_PREFIX."meta_fields WHERE name = '".$last_part."'");
							}
							
							/**
							 * get data
							 */
							$meta_data = $GLOBALS['linpha']->db->GetOne("SELECT meta_data " .
								"FROM ".LIN_PREFIX."meta_data ".
								"WHERE field_id = '".$this->cachedFieldIds[$last_part]."' " .
								"AND md5sum = '".$array_src_infos['md5sum']."'");
							
							if( !empty($meta_data) ) {
								$value = htmlspecialchars($meta_data,ENT_QUOTES);
							}
						break;
				}	// end switch($last_part)
				break;	// end case 'builtin'

			case 'id':	// own fields

				/**
				 * get data
				 */
				$meta_data = $GLOBALS['linpha']->db->GetOne("SELECT meta_data " .
					"FROM ".LIN_PREFIX."meta_data " .
					"WHERE field_id = '".$last_part."' " .
					"AND md5sum = '".$array_src_infos['md5sum']."'");
					
				if(isset($meta_data) && !empty($meta_data)) {
					$value = htmlspecialchars($meta_data,ENT_QUOTES);
				}
				break;

			case 'exif':
				if($GLOBALS['linpha']->sql->config->value['sys_image_exif']) {
					$exif_data = $GLOBALS['linpha']->db->GetRow("SELECT ".$last_part." " .
							"FROM ".LIN_PREFIX."meta_exif WHERE md5sum = '".$array_src_infos['md5sum']."'");
					
					if(isset($exif_data[$last_part])) {
						$value = htmlspecialchars($exif_data[$last_part],ENT_QUOTES);
					}
				}
				break;
			case 'iptc':
				if($GLOBALS['linpha']->sql->config->value['sys_image_iptc']) {
					$iptc_data = $GLOBALS['linpha']->db->GetRow("SELECT ".$last_part." " .
							"FROM ".LIN_PREFIX."meta_iptc WHERE md5sum = '".$array_src_infos['md5sum']."'");
					
					if(isset($iptc_data[$last_part])) {
						$value = htmlspecialchars($iptc_data[$last_part],ENT_QUOTES);
					}
				}
				break;
			}
	
			if($value != '')
			{
				/**
				 * get category names if the data looks like ';5;4;44;' or at least ';2;'
				 */
				if( preg_match( '/^;[0-9;]*;$/', $value) )
				{
					$array_catids = linExplodeAndSlice(';',$value);
					$str_catnames = '';
					$isValidCategory = true;
					for($i=1; list($key,$catid) = each($array_catids) ; $i++)
					{
						$str_catnames .= $GLOBALS['linpha']->db->GetOne("SELECT name FROM ".LIN_PREFIX."meta_category WHERE id = '".LinSql::linAddslashes($catid)."'");
						
						/**
						 * was this a valid category id?
						 * maybe someone just added ;nice; as a description
						 * 
						 * this will only check the first category id, but thats okay
						 */
						if(empty($str_catnames))
						{
							$isValidCategory = false;
							break;
						}
						
						if( $i < count($array_catids) )
						{
							$str_catnames .= ', ';
						}
					}
					
					if($isValidCategory)
					{
						$value = $str_catnames;
					}
				}
				
				$output_array[$field_name] = Array('name' => $name, 'value' => $value);
			}
		}
	
		return $output_array;
	}
	
} // end class LinMetaData


/**
 * Admin functions to select the tags which should be indexed
 */
class LinMetaDataAdmin extends LinMetaData
{
	function createExifTable($array='')
	{
	    if(empty($array))
	    {
	    	$array = $this->getDefaultExifFields();
	    }
	    
	    if(LIN_DB_TYPE == "mysql")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_exif (".
	                    "md5sum VARCHAR(32) NOT NULL, ";
	        foreach($array AS $key=>$value)
	        {
	            $str .= LinSql::linAddslashes(strtolower($value))." VARCHAR(255), ";
	        }
	        $str .= "PRIMARY KEY  (md5sum), ";
	        $str .= "KEY (md5sum) )";
	    }
	    elseif(LIN_DB_TYPE == "sqlite")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_exif (".
	                    "md5sum VARCHAR(32) PRIMARY KEY NOT NULL, ";
	        foreach($array AS $key=>$value)
	        {
	            $str .= strtolower($value)." VARCHAR(255), ";
	        }
	        /**
	         * remove last 2 signs (the comma and the space)
	         */
	        $str = substr($str,0,strlen($str)-2).")";
	    }
	    elseif(LIN_DB_TYPE == "postgres")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_exif (".
	                    "md5sum VARCHAR(32) NOT NULL, ";
	
	        foreach($array AS $key=>$value)
	        {
	            $str .= strtolower($value)." VARCHAR(255), ";
	        }
	        
	        /**
	         * remove last 2 signs (the comma and the space)
	         */
	        $str = substr($str,0,strlen($str)-2).")";
	    }
	    return $str;
	}
	
	function getDefaultExifFields()
	{
		return Array
		(
	    	'datetimeoriginal',
			'make',
			'model',
			'artist',
			'copyright',
			'aperturevalue',
			'shutterspeedvalue',
			'exposuretime',
			'isospeedratings',
			'flash',
			'focallength',
			'fnumber'
		);
	}

	function createIptcTable($array='')
	{
	    if(empty($array))
	    {
	    	$array = Array();
	    }
	
	    if(LIN_DB_TYPE == "mysql")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_iptc (".
	                    "md5sum VARCHAR(32) NOT NULL, ";
	        
	        foreach($array AS $key=>$value)
	        {
	            $str .= LinSql::linAddslashes(strtolower($value))." VARCHAR(255), ";
	        }
	        $str .= "PRIMARY KEY  (md5sum), ";
	        $str .= "KEY (md5sum) )";
	    }
	    elseif(LIN_DB_TYPE == "sqlite")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_iptc (".
	                    "md5sum VARCHAR(32) PRIMARY KEY NOT NULL, ";
	        foreach($array AS $key=>$value)
	        {
	            $str .= strtolower($value)." VARCHAR(255), ";
	        }
	        /**
	         * remove last 2 signs (the comma and the space)
	         */
	        $str = substr($str,0,strlen($str)-2).")";
	    }
	    elseif(LIN_DB_TYPE == "postgres")
	    {
	        $str = "CREATE TABLE ".LIN_PREFIX."meta_iptc (".
	                    "md5sum VARCHAR(32) NOT NULL, ";
	
	        foreach($array AS $key=>$value)
	        {
	            $str .= strtolower($value)." VARCHAR(255), ";
	        }
	        
	        /**
	         * remove last 2 signs (the comma and the space)
	         */
	        $str = substr($str,0,strlen($str)-2).")";
	    }
	    return $str;
	}	

} // end class LinMetaDataAdmin


/**
 * Index functions (save data in DB)
 */
class LinMetaDataIndex extends LinMetaData
{
	/**
	 * wrapper function for saveExifData and saveIptcDate for easier access
	 */
	function saveData($what,$filename,$md5sum)
	{
		/**
		 * no double entries
		 * do this check at very first position to improve performance!
		 * also do not setFields in the __contruct() because they may never be used
		 */        
	    $query = $GLOBALS['linpha']->db->Execute("SELECT md5sum FROM ".LIN_PREFIX."meta_".$what." ".
	    	"WHERE md5sum = '".$md5sum."'");

		if( $query->EOF) // not a single value returned
		{
			/**
			 * define tags to index if not already done
			 */
			if( !isset($this->defined_fields[$what]))
			{
				$this->setFields($what);
			}

			switch($what)
			{
			case 'exif':
				$this->saveExifData($filename,$md5sum);
				break;
			case 'iptc':
				$this->saveIptcData($filename,$md5sum);
				break;
			}
		}
	}
	
	/**
	 * store EXIF data in DB
	 * 
	 * create an entry in the db even if we haven't found anything
	 * to prevent img_view.class.php to call this function every time
	 * the image is viewed
	 */
	function saveExifData($filename,$md5sum)
	{
    	include_once(LINPHA_DIR.'/lib/classes/pjmt/JPEG.php'); // used for jpeg comment
        include_once(LINPHA_DIR.'/lib/classes/pjmt/EXIF.php');

	    $str_columns = "md5sum, ";
	    $str_values = "'".$md5sum."', ";

	    /**
	     * get exif info from file
	     */
	    $exif_data =  get_EXIF_JPEG( $filename );
	    if( $exif_data)
	    {
		    /**
		     * special tags
		     */
		    // Canon Owner Name append to Artist
		    if(isset($exif_data['Makernote_Tag']['Decoded Data'])
		     && is_array($exif_data['Makernote_Tag']['Decoded Data']) // sometimes 'Decoded Data' is an empty string
		     && isset($exif_data['Makernote_Tag']['Decoded Data'][0][9]['Text Value']))
		    {
		        $owner = $exif_data['Makernote_Tag']['Decoded Data'][0][9]['Text Value']; 
		        if(isset($exif_data[0][315]['Text Value']))
		        {
		            $exif_data[0][315]['Text Value'] .= $owner;
		        }
		        else
		        {
		            $exif_data[0][315]['Text Value'] = $owner;
		            }
		             
		        }
		 
		    /**
		     * search for valid tags 
		     */       
		    foreach($this->defined_fields['exif'] AS $key=>$value)
		    //foreach(LinMetaData::$Tags['exif'] AS $key=>$value)
		    {
		        if( !empty(LinMetaData::$Tags['exif'][$key]['pathvalue']) )
		        {
		        	/**
		        	 * there are currently only entries at level deep 2 and 5 in the array
		        	 */
		            $array_pieces = explode('/',LinMetaData::$Tags['exif'][$key]['pathvalue']);
		            switch(count($array_pieces))
		            {
		            case 2:
		                if(isset($exif_data[ $array_pieces[0] ][ $array_pieces[1] ]['Text Value']))
		                {
		                    $str_columns .= $key.', ';
		                    
		                    $exif_value = $exif_data[ $array_pieces[0] ][ $array_pieces[1] ]['Text Value'];
		                    $str_values .= "'".LinSql::linAddslashes(trim( $exif_value ))."', ";
		                }
		            break;
		            case 5:
		                if(isset($exif_data[ $array_pieces[0] ][ $array_pieces[1] ][ $array_pieces[2] ][ $array_pieces[3] ][ $array_pieces[4] ]['Text Value']))
		                {
		                    $str_columns .= $key.', ';
		                    
		                    $exif_value = $exif_data[ $array_pieces[0] ][ $array_pieces[1] ][ $array_pieces[2] ][ $array_pieces[3] ][ $array_pieces[4] ]['Text Value'];
		                    $str_values .= "'".LinSql::linAddslashes(trim( $exif_value ))."', ";
		                }
		            break;
		            default:
		                echo "Error no valid path for key: ".$key." value: ".LinMetaData::$Tags['exif'][$key]['pathvalue']."<br />";
		            break;
		            }
		        }
		    }
		    
			/**
			 * add jpegcomment
			 */
			if(isset($this->defined_fields['exif']['jpegcomment']))
			{
				$jpeg_header_data = get_jpeg_header_data( $filename );
				$comment = get_jpeg_Comment( $jpeg_header_data );
				if(!empty($comment))
				{
					$str_columns .= 'jpegcomment, ';
					$str_values .= "'".LinSql::linAddslashes(trim( $comment ))."', ";
				}
			}
	    }
	    
	    /**
	     * remove last two signs (the comma and the space) which we added to much
	     */
	    $str_columns = substr($str_columns,0,strlen($str_columns)-2);
	    $str_values = substr($str_values,0,strlen($str_values)-2);
    

        $GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."meta_exif (".$str_columns.") ".
            "VALUES (".$str_values.")");
	}
	
	/**
	 * store IPTC data in DB
	 */
	function saveIptcData($filename, $md5sum)
	{
        include_once(LINPHA_DIR.'/lib/classes/pjmt/Photoshop_IRB.php'); 
        include_once(LINPHA_DIR.'/lib/classes/pjmt/JPEG.php');  

	    $str_columns = "md5sum, ";
	    $str_values = "'".$md5sum."', ";

	    // Retrieve the header information
	    $jpeg_header_data = get_jpeg_header_data($filename);

	    if( $jpeg_header_data )
	    {
	        // get array with all available IPTC data
	        $IPTC_Info = get_Photoshop_IPTC(get_Photoshop_IRB($jpeg_header_data));
	
			if(is_array($IPTC_Info))
			{
			    /**
			     * search for valid tags 
			     */       
			    foreach($this->defined_fields['iptc'] AS $key=>$value)
			    {
			        if( !empty(LinMetaData::$Tags['iptc'][$key]['path']) )
			        {
						$str_columns .= $key.', ';
						$str_values .= "'";

						foreach($IPTC_Info as $IPTC_Record)
						{
							if( $IPTC_Record['IPTC_Type'] == LinMetaData::$Tags['iptc'][$key]['path'] )
							{
			                    $str_values .= "".LinSql::linAddslashes(trim( $IPTC_Record['RecData'] ))." ";
								
							}
						}
						
						$str_values .= "', ";
			        }
			    }
			}
	    }
	    
	    /**
	     * remove last two signs (the comma and the space) which we added to much
	     */
	    $str_columns = substr($str_columns,0,strlen($str_columns)-2);
	    $str_values = substr($str_values,0,strlen($str_values)-2);
    

        $GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."meta_iptc (".$str_columns.") ".
            "VALUES (".$str_values.")");
	}

	/**
	 * Autorotate Images via EXIF tag
	 * Most Images contain an orientation tag which shows if and how to rotate 
	 * the images
	 * @param int exif rotation tag
	 * @return degrees to rotate
	 * @author bzrudi
	 */	
	function getRotationByExifTag($rotation)
	{
	    if(is_numeric($rotation))
	    {
	    	switch($rotation)
	    	{
	    		//No Rotation, No Flip Row 0 is at the visual top of the 
	    		//image, and column 0 is the visual left-hand side
	    		case "1":
	    		$rotate = "0";
	    		break;
	    		//No Rotation, Flipped Horizontally Row 0 is at the visual 
	    		//top of the image,and column 0 is the visual right-hand side
	    		case "2":
	    		$rotate = "0";
	    		break;
	    		//Rotated 180 degrees, No Flip Row 0 is at the visual 
	    		//bottom of the image, and column 0 is the visual right-hand side
	    		case "3":
	    		$rotate = "180";
	    		break;
	    		//No Rotation, Flipped Vertically Row 0 is at the visual 
	    		//bottom of the image, and column 0 is the visual left-hand side
	    		case "4":
	    		$rotate = "0";
	    		break;
	    		//Flipped Horizontally, Rotated 90 degrees counter clockwise 
	    		//Row 0 is at the visual left-hand side of of the image, 
	    		//and column 0 is the visual top
	    		case "5":
	    		$rotate = "0";
	    		break;
	    		//No Flip, Rotated 90 degrees clockwise Row 0 is at the visual 
	    		//right-hand side of of the image, and column 0 is the visual top
	    		case "6":
	    		$rotate = "90";
	    		break;
	    		//Flipped Horizontally, Rotated 90 degrees clockwise Row 0 is 
	    		//at the visual right-hand side of of the image, and 
	    		//column 0 is the visual bottom
	    		case "7":
	    		$rotate = "0";
	    		break;
	    		//No Flip, Rotated 90 degrees counter clockwise Row 0 is at 
	    		//the visual left-hand side of of the image, and column 0 is 
	    		//the visual bottom
	    		case "8":
	    		$rotate = "-90";
	    		break;
	    		default:
	    		$rotate = "0";
	    		break;
	
	    	}
	    }
		else // no rotation data is found, default to "0"
		{
			$rotate = "0";
		}
	   	return $rotate;
	} // end function getRotationByExifTag()
	
	/**
	 * converts a exif date to a unix timestamp
	 * example: 2004:02:14 18:24:19 -> 11167594589
	 * 
	 * @author flo
	 * @param string $date
	 * @return int unix timestamp
	 */
	function convertExifDateToUnix( $date )
	{
		$year=substr($date,0,4);
		$month=substr($date,5,2);
		$day=substr($date,8,2);
		$hour=substr($date,11,2);
		$min=substr($date,14,2);
		$sec=substr($date,17,2);
		return mktime($hour,$min,$sec,$month,$day,$year);		
	}

} // end class LinMetaDataIndex


/**
 * Modificate metadata and save the information back to the file (iptc and xmp)
 */
class LinMetaDataModificate extends LinMetaData
{
	function showEditMetaData()
	{
		/**
		 * set meta linpha
		 */
			$GLOBALS['linpha']->template->output['meta_linpha'] = '';
		
			$query = $GLOBALS['linpha']->db->Execute("SELECT id, name, field_type, flags " .
					"FROM ".LIN_PREFIX."meta_fields " .
					"WHERE flags = '5' OR flags = '7' ORDER by id");
			while($data = $query->FetchRow())
			{
				$GLOBALS['linpha']->template->output['meta_linpha'] .= '<tr><td width="150" valign="top">';
	
				if( $data['flags']==5 )
				{
					$GLOBALS['linpha']->template->output['meta_linpha'] .= LinMetaData::getNameOfField('builtin_'.$data['name'],false);
				}
				elseif( $data['flags']==7 )
				{
					$GLOBALS['linpha']->template->output['meta_linpha'] .= LinMetaData::getNameOfField('id_'.$data['id'],false);
				}
				$GLOBALS['linpha']->template->output['meta_linpha'] .= '</td><td>';
	
				$meta_data = $GLOBALS['linpha']->db->GetRow("SELECT meta_data FROM ".LIN_PREFIX."meta_data " .
					"WHERE md5sum = '".$GLOBALS['linpha']->imgview->md5sum."'" .
					"AND field_id = '".$data['id']."'");
		
				switch($data['field_type'])
				{
				case 1:
					if(isset($meta_data['meta_data'])) {
						$value = htmlspecialchars($meta_data['meta_data'],ENT_QUOTES);
					} else {
						$value = '';
					}
					$GLOBALS['linpha']->template->output['meta_linpha'] .= '<input type="text" class="linForms" name="linMetaField['.$data['id'].']" value="'.$value.'"  style="width: 250px;" maxlength="255" />';
					break;
				case 3:
					if(isset($meta_data['meta_data'])) {
						$value = htmlspecialchars($meta_data['meta_data'],ENT_QUOTES);
					} else {
						$value = '';
					}
					$GLOBALS['linpha']->template->output['meta_linpha'] .=
						'<input type="text" class="linForms" name="linMetaField['.$data['id'].']" id="linField'.$data['id'].'" value="'.$value.'"  style="width: 150px;" maxlength="255" />'
						.' <a href="javascript:void(0);" onclick="var loginWin = window.open(\''.LINPHA_CLIENT.'/lib/include/calendar.php?form=linField'.$data['id'].'\''.
							',\'Calendar\',\'height=250,width=300,scrollbars=no,menubar=no,status=no\');" target="_top">'.i18n("Calendar").'</a>';
	
					break;
				case 2:
					if(isset($meta_data['meta_data'])) {
						$array_pieces = linExplodeAndSlice(';',$meta_data['meta_data']);
					} else {
						$array_pieces = Array();
					}
					
					$GLOBALS['linpha']->template->output['meta_linpha'] .= '<select name="linMetaField['.$data['id'].'][]" class="linForms" size="7" multiple="multiple" style="width: 250px;">';
		
					$cat_query = $GLOBALS['linpha']->db->Execute("SELECT id, name " .
							"FROM ".LIN_PREFIX."meta_category " .
							"WHERE field_id = '".$data['id']."'");
					while($cat_data = $cat_query->FetchRow())
					{
						if(in_array($cat_data['id'],$array_pieces)) {
							$checked = ' selected="selected"';
						} else {
							$checked = '';
						}
						
						$GLOBALS['linpha']->template->output['meta_linpha'] .= '<option value="'.$cat_data['id'].'"'.$checked.'>'.htmlspecialchars($cat_data['name'],ENT_QUOTES).'</option>'."\n";
					}
					$GLOBALS['linpha']->template->output['meta_linpha'] .= '</select>'."\n".
						'<br /><span style="font-size: smaller;">'.i18n("For Multiple Select Use CTRL").'</span>';
					
					if($GLOBALS['linpha']->sql->isAdmin())
					{
						$GLOBALS['linpha']->template->output['meta_linpha'] .= '<br /><span style="font-size: smaller;"><a href="'.LINPHA_CLIENT.'/admin/?cat=metadata_define_adddata">'.i18n("Add More Options").'</a><br /><br /></span>';
					}
	
					break;
				}
	
				$GLOBALS['linpha']->template->output['meta_linpha'] .= '</td></tr>';
			}
			
		/**
		 * set meta iptc
		 */
			if($GLOBALS['linpha']->sql->config->value['sys_image_iptc'])
			{
				if( isset($GLOBALS['linpha']->imgview->src_file) && file_exists($GLOBALS['linpha']->imgview->src_file))
				{
					if( is_writable($GLOBALS['linpha']->imgview->src_file))
					{
						$edit_possible = true;
						$filename = $GLOBALS['linpha']->imgview->src_file;		// used in IPTC Edit_File_Info.php
					}
					else
					{
						$edit_possible = false;
					}
				}
				else
				{
					$edit_possible = false;
				}
			}
		
		/**
		 * build the meta data view
		 */
			$fragment = 'edit_meta';
			global $linTpl;
			include(LINPHA_DIR.'/templates/'.$this->template_name.'/fragments.php');

	} // end function showEditMetaData()
	
	/**
	 * save meta linpha and meta iptc separetaly
	 */
	function saveModifiedMetaData()
	{
		if($GLOBALS['linpha']->sql->checkPermission('metadata_edit'))
		{
			/**
			 * write linpha meta data
			 */
			if(isset($_POST['linCmd']) && $_POST['linCmd']=='linInsertMetaLinpha')
			{
				LinMetaDataModificate::saveModifiedMetaDataLinpha();
				linSysLog( i18n("Successfully updated data.") );
			}

			/**
			 * write iptc data
			 */
			if( isset($_POST['linCmd'])
				&& $_POST['linCmd']=='linInsertMetaIptc'
				&& $GLOBALS['linpha']->sql->config->value['sys_image_iptc']
				&& isset($GLOBALS['linpha']->imgview->src_file)
				&& file_exists($GLOBALS['linpha']->imgview->src_file) )
			{
				global $new_ps_file_info_array;
				$new_ps_file_info_array = $GLOBALS['HTTP_POST_VARS'];
				
				/**
				 * important!
				 * remove other POST data from this array, because pjmt just takes all values and this will generate unexpected results!!!
				 * we got problems because $_POST['linMetaField'] is an array and pjmt couldnt handle this
				 */
				unset($new_ps_file_info_array['linCmd']);
				unset($new_ps_file_info_array['linMetaField']);
				unset($new_ps_file_info_array['submit']);
				
				/**
				 * if the backup went wrong, the saveModifiedMetaDataIptc() would most probably also fail
				 */
				if( LinFilesys::backupFile($GLOBALS['linpha']->imgview->src_file))
				{
					if( LinMetaDataModificate::saveModifiedMetaDataIptc( $new_ps_file_info_array, $GLOBALS['linpha']->imgview->src_file ) )	// error handling already done
					{
						/**
						 * force read data from file again if write was successful
						 */
						unset($new_ps_file_info_array);
						LinImport::updateEntry(
							$GLOBALS['linpha']->template->idCurrent ,
							dirname($GLOBALS['linpha']->imgview->src_file) ,
							basename($GLOBALS['linpha']->imgview->src_file)
						);
					}
				}
				else
				{
					linSysLog( sprintf( i18n("Error: Cannot create backup of: %s"), $GLOBALS['linpha']->imgview->src_file) );
					linLog( LOG_TYPE_META, LOG_ERR, 'meta_iptc', "Error: Cannot create backup of:" .' '. $GLOBALS['linpha']->imgview->src_file );
				}
			}
		}
	}
	
	/**
	 * save modified linpha meta data to database
	 */
	function saveModifiedMetaDataLinpha()
	{
		if( isset($_POST['linMetaField']) && is_array($_POST['linMetaField']) )
		{
			/**
			 * delete existing entries
			 */
			$query = $GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."meta_data " .
				"WHERE md5sum = '".$GLOBALS['linpha']->imgview->md5sum."'");

			/**
			 * insert new entries
			 */
			foreach($_POST['linMetaField'] AS $key=>$value)
			{
				if(is_array($value))	// coming from a select (categories etc.)
				{
					$value = ';'.implode(';',$value).';';
				}
				
				if(! empty($value) )
				{
					$GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."meta_data ".
						"(field_id, md5sum, meta_data) VALUES " .
						"('".LinSql::linAddslashes($key)."','".$GLOBALS['linpha']->imgview->md5sum."','".LinSql::linAddslashes($value)."')");
				}
			}
		}
	}
	
	/**
	 * save iptc metadata back to file
	 * 
	 * copied from pjmt/Write_File_Info.php
	 */
	function saveModifiedMetaDataIptc( & $new_ps_file_info_array, $filename )
	{
		/** // Turn off Error Reporting
		error_reporting ( 0 );          // Change: changed to no reporting -  as of version 1.11
		
		include 'Toolkit_Version.php';  // Change: added as of version 1.11*/
		
		// Include the required files for reading and writing Photoshop File Info
		include_once(LINPHA_DIR.'/lib/classes/pjmt/JPEG.php');
		include_once(LINPHA_DIR.'/lib/classes/pjmt/XMP.php');
		include_once(LINPHA_DIR.'/lib/classes/pjmt/Photoshop_IRB.php');
		include_once(LINPHA_DIR.'/lib/classes/pjmt/EXIF.php');
		include_once(LINPHA_DIR.'/lib/classes/pjmt/Photoshop_File_Info.php');
		
		$GLOBALS[ "Software Name" ] = "LinPHA v".LINPHA_VERSION.", powered by PHP JPEG Metadata Toolkit v" . $GLOBALS['Toolkit_Version'] ;
		
		// Copy all of the HTML Posted variables into an array
		/**
		 * $new_ps_file_info_array = $GLOBALS['HTTP_POST_VARS'];
		 */
		
		// Some characters are escaped with backslashes in HTML Posted variable
		// Cycle through each of the HTML Posted variables, and strip out the slashes
		/**
		 * already done in /include/common.php !!
		 * foreach( $new_ps_file_info_array as $var_key => $var_val )
		{
		        $new_ps_file_info_array[ $var_key ] = stripslashes( $var_val );
		}*/
		
		// Keywords should be an array - explode it on newline boundarys
		$new_ps_file_info_array[ 'keywords' ] = explode( "\n", trim( $new_ps_file_info_array[ 'keywords' ] ) );
		
		// Supplemental Categories should be an array - explode it on newline boundarys
		$new_ps_file_info_array[ 'supplementalcategories' ] = explode( "\n", trim( $new_ps_file_info_array[ 'supplementalcategories' ] ) );
		
		/**
		// Make the filename easier to access
		$filename = $new_ps_file_info_array[ 'filename' ];
		
		// Protect against hackers editing other files
		$path_parts = pathinfo( $filename );
		if ( strcasecmp( $path_parts["extension"], "jpg" ) != 0 )
		{
		        echo "Incorrect File Type - JPEG Only\n";
		        exit( );
		}
		// Change: removed limitation on file being in current directory - as of version 1.11
		 * 
		 */
		
		// Retrieve the header information
		$jpeg_header_data = get_jpeg_header_data( $filename );
		
		// Retreive the EXIF, XMP and Photoshop IRB information from
		// the existing file, so that it can be updated
		$Exif_array = get_EXIF_JPEG( $filename );
		$XMP_array = read_XMP_array_from_text( get_XMP_text( $jpeg_header_data ) );
		$IRB_array = get_Photoshop_IRB( $jpeg_header_data );
		
		// Update the JPEG header information with the new Photoshop File Info
		$jpeg_header_data = put_photoshop_file_info( $jpeg_header_data, $new_ps_file_info_array, $Exif_array, $XMP_array, $IRB_array );
		
		// Check if the Update worked
		if ( $jpeg_header_data == FALSE )
		{
		        // Update of file info didn't work - output error message
		        /**
		         * echo "Error - Failure update Photoshop File Info : $filename <br>\n";
		         */
		        linSysLog( i18n("Error: Failure update Photoshop File Info:" .' '. $filename) );
		        linLog( LOG_TYPE_META, LOG_ERR, 'meta_iptc', "Error: Failure update Photoshop File Info:" .' '. $filename );
		
		        // Output HTML with the form and data which was
		        // sent, to allow the user to fix it
		
		        /**$outputfilename = $filename;
		        include "Edit_File_info.php";
		        echo "</body>\n";
		        echo "</html>\n";
		
		        // Abort processing
		        exit( );*/
		        
		        return false;
		}
		
		// Attempt to write the new JPEG file
		elseif ( FALSE == put_jpeg_header_data( $filename, $filename, $jpeg_header_data ) )
		{
		        // Writing of the new file didn't work - output error message
		        /**
		         * echo "Error - Failure to write new JPEG : $filename <br>\n";
		         */
		        linSysLog( i18n("Error: Failure to write new JPEG:" .' '. $filename) );
		        linLog( LOG_TYPE_META, LOG_ERR, 'meta_iptc', "Error: Failure to write new JPEG:" .' '. $filename );
		
		        // Output HTML with the form and data which was
		        // sent, to allow the user to fix it
		
		        /**$outputfilename = $filename;
		        include "Edit_File_info.php";
		        echo "</body>\n";
		        echo "</html>\n";
		
		        // Abort processing
		        exit( );*/
		        
		        return false;
		}
		else
		{
			// Writing of new JPEG succeeded
			
			// Output information about new file
			
			linSysLog( i18n("DONE! Updated:" .' '. $filename) );
			linLog( LOG_TYPE_META, LOG_NOTICE, 'meta_iptc', "Updated:" .' '. $filename );
			/**
			echo "<h1>DONE! - $filename updated</h1>\n";
			echo "<p><a href=\"Example.php?jpeg_fname=$filename\" >View Full Metatdata Information</a></p>\n";
			echo "<p><a href=\"Edit_File_Info_Example.php?jpeg_fname=$filename\" >Re-Edit Photoshop File Info</a></p>\n";
			echo "<br><br>\n";
			echo "<p>Below is the updated image, you can save it and look at the changed metadata in your favorite image editor</p>\n";
			echo "<p><img src=\"$filename\" ></p>\n";*/
			
			return true;
		}
	}

} // end class LinMetaDataModificate


/**
 * class LinMetaData
 * 
 * some common definitions which are used in more than one child class of LinMetaData
 */
class LinMetaData
{
	public $available_fields, $defined_fields;

	static $Tags = array(

	'exif' => array(
	    // TIFF Rev. 6.0 Attribute Information
	        // A. Tags relating to image data structure
	    'imagewidth'					=> array('name' => 'Image Width',						'pathkey' => 'ImageWidth',					'pathvalue' => ''), 
	    'imagelength'					=> array('name' => 'Image Length',						'pathkey' => 'ImageLength',					'pathvalue' => ''),
	    'bitspersample'					=> array('name' => 'Bits Per Sample',					'pathkey' => 'BitsPerSample',				'pathvalue' => ''),
	    'compression'					=> array('name' => 'Compression',						'pathkey' => 'Compression',					'pathvalue' => ''),
	    'photometricinterpretation'		=> array('name' => 'Photometric Interpretation',		'pathkey' => 'PhotometricInterpretation',	'pathvalue' => ''),
	    'orientation'					=> array('name' => 'Orientation',						'pathkey' => 'Orientation',					'pathvalue' => '0/274'),
	    'samplesperpixel'				=> array('name' => 'Samples Per Pixel',					'pathkey' => 'SamplesPerPixel',				'pathvalue' => ''),
	    'planarconfiguration'			=> array('name' => 'Planar Configuration',				'pathkey' => 'PlanarConfiguration',			'pathvalue' => ''),
	    'ycbcrsubsampling'				=> array('name' => 'YCbCr Sub Sampling',				'pathkey' => 'YCbCrSubSampling',			'pathvalue' => ''),
	    'ycbcrpositioning'				=> array('name' => 'YCbCr Positioning',					'pathkey' => 'YCbCrPositioning',			'pathvalue' => '0/531'),
	    'xresolution'					=> array('name' => 'X Resolution',						'pathkey' => 'XResolution',					'pathvalue' => '0/282'),
	    'yresolution'					=> array('name' => 'Y Resolution',						'pathkey' => 'YResolution',					'pathvalue' => '0/283'),
	    'resolutionunit'				=> array('name' => 'Resolution Unit',					'pathkey' => 'ResolutionUnit',				'pathvalue' => '0/296'),
	    
	    // B. Tags relating to recording offset
	    'stripoffsets'					=> array('name' => 'Strip Offsets',						'pathkey' => 'StripOffsets',				'pathvalue' => ''),
	    'rowsperstrip'					=> array('name' => 'Rows Per Strip',					'pathkey' => 'RowsPerStrip',				'pathvalue' => ''),
	    'stripbytecounts'				=> array('name' => 'Strip Byte Counts',					'pathkey' => 'StripByteCounts',				'pathvalue' => ''),
	    'jpeginterchangeformat'			=> array('name' => 'JPEG Interchange Format',			'pathkey' => 'JPEGInterchangeFormat',		'pathvalue' => ''),
	    'jpeginterchangeformatlength'	=> array('name' => 'JPEG Interchange Format Length',	'pathkey' => 'JPEGInterchangeFormatLength',	'pathvalue' => ''),
	    
	    // C. Tags relating to image data characteristics
	    'transferfunction'				=> array('name' => 'Transfer Function',					'pathkey' => 'TransferFunction',			'pathvalue' => ''),
	    'whitepoint'					=> array('name' => 'White Point',						'pathkey' => 'WhitePoint',					'pathvalue' => ''),
	    'primarychromaticities'			=> array('name' => 'Primary Chromaticities',			'pathkey' => 'PrimaryChromaticities',		'pathvalue' => ''),
	    'ycbcrcoefficients'				=> array('name' => 'YCbCr Coefficients',				'pathkey' => 'YCbCrCoefficients',			'pathvalue' => ''),
	    'referenceblackwhite'			=> array('name' => 'Reference Black White',				'pathkey' => 'ReferenceBlackWhite',			'pathvalue' => ''),
	    
	    // D. Other tags
	    'datetime'						=> array('name' => 'Date Time',							'pathkey' => 'DateTime',					'pathvalue' => '0/306'),
	    'imagedescription'				=> array('name' => 'Image Description',					'pathkey' => 'ImageDescription',			'pathvalue' => '0/270'),
	    'make'							=> array('name' => 'Make',								'pathkey' => 'Make',						'pathvalue' => '0/271'),
	    'model'							=> array('name' => 'Model',								'pathkey' => 'Model',						'pathvalue' => '0/272'),
	    'software'						=> array('name' => 'Software',							'pathkey' => 'Software',					'pathvalue' => '0/305'),
    	'artist'						=> array('name' => 'Artist',							'pathkey' => 'Artist',						'pathvalue' => '0/315'),
	    'copyright'						=> array('name' => 'Copyright',							'pathkey' => 'Copyright',					'pathvalue' => '0/33432'),
	    
	// Exif IFD Attribute Information
	    // A. Tags Relating to Version
	    'exifversion'					=> array('name' => 'Exif Version',						'pathkey' => 'ExifVersion',					'pathvalue' => '0/34665/Data/0/36864'),
	    'flashpixversion'				=> array('name' => 'FlashPix Version',					'pathkey' => 'FlashpixVersion',				'pathvalue' => '0/34665/Data/0/40960'),
	    
	    // B. Tag Relating to Image Data Characteristics
	    'colorspace'					=> array('name' => 'Color Space',						'pathkey' => 'ColorSpace',					'pathvalue' => '0/34665/Data/0/40961'),
	    
	    // C. Tags Relating to Image Configuration
	    'componentsconfiguration'		=> array('name' => 'Components Configuration',			'pathkey' => 'ComponentsConfiguration',		'pathvalue' => '0/34665/Data/0/37121'),
	    'compressedbitsperpixel'		=> array('name' => 'Compressed Bits Per Pixel',			'pathkey' => 'CompressedBitsPerPixel',		'pathvalue' => '0/34665/Data/0/37122'),
	    'pixelxdimension'				=> array('name' => 'Pixel X Dimension',					'pathkey' => 'PixelXDimension',				'pathvalue' => '0/34665/Data/0/40962'),
	    'pixelydimension'				=> array('name' => 'Pixel Y Dimension',					'pathkey' => 'PixelYDimension',				'pathvalue' => '0/34665/Data/0/40963'),
	    
	    // D. Tags Relating to User Information
	    //'makernote'					=> array('name' => 'MakerNote',							'pathkey' => 'MakerNote',					'pathvalue' => ''), too big to store!!
	    'usercomment'					=> array('name' => 'User Comment',						'pathkey' => 'UserComment',					'pathvalue' => '0/34665/Data/0/37510'),
	    
	    // E. Tag Relating to Related File Information
	    'relatedsoundfile'				=> array('name' => 'Related Sound File',				'pathkey' => 'RelatedSoundFile',			'pathvalue' => ''),
	    
	    // F. Tags Relating to Date and Time
	    'datetimeoriginal'				=> array('name' => 'Date Time Original',				'pathkey' => 'DateTimeOriginal',			'pathvalue' => '0/34665/Data/0/36867'),
	    'datetimedigitized'				=> array('name' => 'Date Time Digitized',				'pathkey' => 'DateTimeDigitized',			'pathvalue' => '0/34665/Data/0/36868'),
	    'subsectime'					=> array('name' => 'Sub Sec Time',						'pathkey' => 'SubSecTime',					'pathvalue' => ''),
	    'subsectimeoriginal'			=> array('name' => 'Sub Sec Time Original',				'pathkey' => 'SubSecTimeOriginal',			'pathvalue' => ''),
	    'subsectimedigitized'			=> array('name' => 'Sub Sec Time Digitized',			'pathkey' => 'SubSecTimeDigitized',			'pathvalue' => ''),
	    
	    // G. Tags Relating to Picture-Taking Conditions
	    'exposuretime'					=> array('name' => 'Exposure Time',						'pathkey' => 'ExposureTime',				'pathvalue' => '0/34665/Data/0/33434'),
	    'fnumber'						=> array('name' => 'F Number',							'pathkey' => 'FNumber',						'pathvalue' => '0/34665/Data/0/33437'),
	    'exposureprogram'				=> array('name' => 'Exposure Program',					'pathkey' => 'ExposureProgram',				'pathvalue' => '0/34665/Data/0/34850'),
	    'spectralsensitivity'			=> array('name' => 'Spectral Sensitivity',				'pathkey' => 'SpectralSensitivity',			'pathvalue' => ''),
	    'isospeedratings'				=> array('name' => 'ISO Speed Ratings',					'pathkey' => 'ISOSpeedRatings',				'pathvalue' => '0/34665/Data/0/34855'),
	    'oecf'							=> array('name' => 'OECF',								'pathkey' => 'OECF',						'pathvalue' => ''),
	    'shutterspeedvalue'				=> array('name' => 'Shutter Speed Value',				'pathkey' => 'ShutterSpeedValue',			'pathvalue' => '0/34665/Data/0/37377'),
	    'aperturevalue'					=> array('name' => 'Aperture Value',					'pathkey' => 'ApertureValue',				'pathvalue' => '0/34665/Data/0/37378'),
	    'brightnessvalue'				=> array('name' => 'Brightness Value',					'pathkey' => 'BrightnessValue',				'pathvalue' => '0/34665/Data/0/37379'),
	    'exposurebiasvalue'				=> array('name' => 'Exposure Bias Value',				'pathkey' => 'ExposureBiasValue',			'pathvalue' => '0/34665/Data/0/37380'),
	    'maxaperturevalue'				=> array('name' => 'MaxAperture Value',					'pathkey' => 'MaxApertureValue',			'pathvalue' => '0/34665/Data/0/37381'),
	    'subjectdistance'				=> array('name' => 'Subject Distance',					'pathkey' => 'SubjectDistance',				'pathvalue' => '0/34665/Data/0/37382'),
	    'meteringmode'					=> array('name' => 'Metering Mode',						'pathkey' => 'MeteringMode',				'pathvalue' => '0/34665/Data/0/37383'),
	    'lightsource'					=> array('name' => 'Light Source',						'pathkey' => 'LightSource',					'pathvalue' => '0/34665/Data/0/37384'),
	    'flash'							=> array('name' => 'Flash',								'pathkey' => 'Flash',						'pathvalue' => '0/34665/Data/0/37385'),
	    'focallength'					=> array('name' => 'Focal Length',						'pathkey' => 'FocalLength',					'pathvalue' => '0/34665/Data/0/37386'),
	    'subjectarea'					=> array('name' => 'Subject Area',						'pathkey' => 'SubjectArea',					'pathvalue' => ''),
	    'flashenergy'					=> array('name' => 'Flash Energy',						'pathkey' => 'FlashEnergy',					'pathvalue' => ''),
	    'spatialfrequencyresponse'		=> array('name' => 'Spatial Frequency Response',		'pathkey' => 'SpatialFrequencyResponse',	'pathvalue' => ''),
	    'focalplanexresolution'			=> array('name' => 'Focal Plane X Resolution',			'pathkey' => 'FocalPlaneXResolution',		'pathvalue' => '0/34665/Data/0/41486'),
	    'focalplaneyresolution'			=> array('name' => 'Foca Plane Y Resolution',			'pathkey' => 'FocalPlaneYResolution',		'pathvalue' => '0/34665/Data/0/41487'),
	    'focalplaneresolutionunit'		=> array('name' => 'Focal Plane Resolution Unit',		'pathkey' => 'FocalPlaneResolutionUnit',	'pathvalue' => '0/34665/Data/0/41488'),
	    'subjectlocation'				=> array('name' => 'Subject Location',					'pathkey' => 'SubjectLocation',				'pathvalue' => ''),
	    'exposureindex'					=> array('name' => 'Exposure Index',					'pathkey' => 'ExposureIndex',				'pathvalue' => ''),
	    'sensingmethod'					=> array('name' => 'Sensing Method',					'pathkey' => 'SensingMethod',				'pathvalue' => '0/34665/Data/0/41495'),
	    'filesource'					=> array('name' => 'File Source',						'pathkey' => 'FileSource',					'pathvalue' => '0/34665/Data/0/41728'),
	    'scenetype'						=> array('name' => 'Scene Type',						'pathkey' => 'SceneType',					'pathvalue' => '0/34665/Data/0/41729'),
	    'cfapattern'					=> array('name' => 'CFA Pattern',						'pathkey' => 'CFAPattern',					'pathvalue' => ''),
	    'customrendered'				=> array('name' => 'Custom Rendered',					'pathkey' => 'CustomRendered',				'pathvalue' => '0/34665/Data/0/41985'),
	    'exposuremode'					=> array('name' => 'Exposure Mode',						'pathkey' => 'ExposureMode',				'pathvalue' => '0/34665/Data/0/41986'),
	    'whitebalance'					=> array('name' => 'White Balance',						'pathkey' => 'WhiteBalance',				'pathvalue' => '0/34665/Data/0/41987'),
	    'digitalzoomratio'				=> array('name' => 'Digital Zoom Ratio',				'pathkey' => 'DigitalZoomRatio',			'pathvalue' => '0/34665/Data/0/41988'),
	    'focallengthin35mmfilm'			=> array('name' => 'Focal Length In 35mm Film',			'pathkey' => 'FocalLengthIn35mmFilm',		'pathvalue' => ''),
	    'scenecapturetype'				=> array('name' => 'Scene Capture Type',				'pathkey' => 'SceneCaptureType',			'pathvalue' => '0/34665/Data/0/41990'),
	    'gaincontrol'					=> array('name' => 'Gain Control',						'pathkey' => 'GainControl',					'pathvalue' => '0/34665/Data/0/41991'),
	    'contrast'						=> array('name' => 'Contrast',							'pathkey' => 'Contrast',					'pathvalue' => '0/34665/Data/0/41992'),
	    'saturation'					=> array('name' => 'Saturation',						'pathkey' => 'Saturation',					'pathvalue' => '0/34665/Data/0/41993'),
	    'sharpness'						=> array('name' => 'Sharpness',							'pathkey' => 'Sharpness',					'pathvalue' => '0/34665/Data/0/41994'),
	    'devicesettingdescription'		=> array('name' => 'Device Setting Description',		'pathkey' => 'DeviceSettingDescription',	'pathvalue' => ''),
	    'subjectdistancerange'			=> array('name' => 'Subject Distance Range',			'pathkey' => 'SubjectDistanceRange',		'pathvalue' => ''),
	    
	    // H. Other Tags
	    'imageuniqueid'					=> array('name' => 'Image Unique ID',					'pathkey' => 'ImageUniqueID',				'pathvalue' => ''),
	    
	// GPS Attribute Information
	    // A. Tags Relating to GPS
	    'gpsversionid'					=> array('name' => 'GPS Version ID',					'pathkey' => 'GPSVersionID',				'pathvalue' => '0/34853/Data/0/0'),
	    'gpslatituderef'				=> array('name' => 'GPS Latitude Ref',					'pathkey' => 'GPSLatitudeRef',				'pathvalue' => '0/34853/Data/0/1'),
	    'gpslatitude'					=> array('name' => 'GPS Latitude',						'pathkey' => 'GPSLatitude',					'pathvalue' => '0/34853/Data/0/2'),
	    'gpslongituderef'				=> array('name' => 'GPS Longitude Ref',					'pathkey' => 'GPSLongitudeRef',				'pathvalue' => '0/34853/Data/0/3'),
	    'gpslongitude'					=> array('name' => 'GPS Longitude',						'pathkey' => 'GPSLongitude',				'pathvalue' => '0/34853/Data/0/4'),
	    'gpsaltituderef'				=> array('name' => 'GPS Altitude Ref',					'pathkey' => 'GPSAltitudeRef',				'pathvalue' => '0/34853/Data/0/5'),
	    'gpsaltitude'					=> array('name' => 'GPS Altitude',						'pathkey' => 'GPSAltitude',					'pathvalue' => '0/34853/Data/0/6'),
	    'gpstimestamp'					=> array('name' => 'GPS Timestamp',						'pathkey' => 'GPSTimeStamp',				'pathvalue' => '0/34853/Data/0/7'),
	    'gpssatellites'					=> array('name' => 'GPS Satellites',					'pathkey' => 'GPSSatellites',				'pathvalue' => '0/34853/Data/0/8'),
	    'gpsstatus'						=> array('name' => 'GPS Status',						'pathkey' => 'GPSStatus',					'pathvalue' => '0/34853/Data/0/9'),
	    'gpsmeasuremode'				=> array('name' => 'GPS Measure Mode',					'pathkey' => 'GPSMeasureMode',				'pathvalue' => '0/34853/Data/0/10'),
	    'gpsdop'						=> array('name' => 'GPS DOP',							'pathkey' => 'GPSDOP',						'pathvalue' => '0/34853/Data/0/11'),
	    'gpsspeedref'					=> array('name' => 'GPS Speed Ref',						'pathkey' => 'GPSSpeedRef',					'pathvalue' => '0/34853/Data/0/12'),
	    'gpsspeed'						=> array('name' => 'GPS Speed',							'pathkey' => 'GPSSpeed',					'pathvalue' => '0/34853/Data/0/13'),
	    'gpstrackref'					=> array('name' => 'GPS Track Ref',						'pathkey' => 'GPSTrackRef',					'pathvalue' => '0/34853/Data/0/14'),
	    'gpstrack'						=> array('name' => 'GPS Track',							'pathkey' => 'GPSTrack',					'pathvalue' => '0/34853/Data/0/15'),
	    'gpsimgdirectionref'			=> array('name' => 'GPS Img Direction Ref',				'pathkey' => 'GPSImgDirectionRef',			'pathvalue' => '0/34853/Data/0/16'),
	    'gpsimgdirection'				=> array('name' => 'GPS Img Direction',					'pathkey' => 'GPSImgDirection',				'pathvalue' => '0/34853/Data/0/17'),
	    'gpsmapdatum'					=> array('name' => 'GPS Map Datum',						'pathkey' => 'GPSMapDatum',					'pathvalue' => '0/34853/Data/0/18'),
	    'gpsdestlatituderef'			=> array('name' => 'GPS Dest Latitude Ref',				'pathkey' => 'GPSDestLatitudeRef',			'pathvalue' => '0/34853/Data/0/19'),
	    'gpsdestlatitude'				=> array('name' => 'GPS Dest Latitude',					'pathkey' => 'GPSDestLatitude',				'pathvalue' => '0/34853/Data/0/20'),
	    'gpsdestlongituderef'			=> array('name' => 'GPS Dest Longitude Ref',			'pathkey' => 'GPSDestLongitudeRef',			'pathvalue' => '0/34853/Data/0/21'),
	    'gpsdestlongitude'				=> array('name' => 'GPS Dest Longitude',				'pathkey' => 'GPSDestLongitude',			'pathvalue' => '0/34853/Data/0/22'),
	    'gpsdestbearingref'				=> array('name' => 'GPS Dest Bearing Ref',				'pathkey' => 'GPSDestBearingRef',			'pathvalue' => '0/34853/Data/0/23'),
	    'gpsdestbearing'				=> array('name' => 'GPS Dest Bearing',					'pathkey' => 'GPSDestBearing',				'pathvalue' => '0/34853/Data/0/24'),
	    'gpsdestdistanceref'			=> array('name' => 'GPS Dest Distance Ref',				'pathkey' => 'GPSDestDistanceRef',			'pathvalue' => '0/34853/Data/0/25'),
	    'gpsdestdistance'				=> array('name' => 'GPS Dest Distance',					'pathkey' => 'GPSDestDistance',				'pathvalue' => '0/34853/Data/0/26'),
	    'gpsprocessingmethod'			=> array('name' => 'GPS Processing Method',				'pathkey' => 'GPSProcessingMethod',			'pathvalue' => '0/34853/Data/0/27'),
	    'gpsareainformation'			=> array('name' => 'GPS Area Information',				'pathkey' => 'GPSAreaInformation',			'pathvalue' => '0/34853/Data/0/28'),
	    'gpsdatestamp'					=> array('name' => 'GPS DateStamp',						'pathkey' => 'GPSDateStamp',				'pathvalue' => '0/34853/Data/0/29'),
	    'gpsdifferential'				=> array('name' => 'GPS Differential',					'pathkey' => 'GPSDifferential',				'pathvalue' => '0/34853/Data/0/30'),
	    
	// Custom Makernotes http://www.ozhiker.com/electronics/pjmt/jpeg_info/makernotes.html
	    // there are several tags: camera settings etc. might be usefull
	
	    // Canon
	    // overwrite Artist => 0/315
	    //'OwnerName' => 'Makernote_Tag/Decoded Data/0/9'
	    
	    // Casio
	    
	    // Fujifilm
	    
	    // Konica/Minolta
	    
	    // Nikon
	
	    // Olympus
	    
	    // Panasonic
	    
	    // Ricoh
		'jpegcomment'					=> array('name' => 'JPEG Comment',						'pathkey' => 'JpegComment',					'pathvalue' => ''),
	),

	'iptc' => array(
		'caption'						=> array('name' => 'Caption',							'path' => '2:120'),
		'caption_writer'				=> array('name' => 'Caption Writer',					'path' => '2:122'),
		'headline'						=> array('name' => 'Headline',							'path' => '2:105'),
		'instructions'					=> array('name' => 'Special Instructions',				'path' => '2:40'),
		'keywords'						=> array('name' => 'Keywords',							'path' => '2:25'),
		'category'						=> array('name' => 'Category',							'path' => '2:15'),
		'supplemental_categorie'		=> array('name' => 'Supplemental Category',				'path' => '2:20'),
		'copyright'						=> array('name' => 'Copyright Notice',					'path' => '2:116'),
		'byline'						=> array('name' => 'By-Line (Author)',					'path' => '2:80'),
		'byline_title'					=> array('name' => 'By-Line Title',						'path' => '2:85'),		// Not used in Photoshop 7
		'credit'						=> array('name' => 'Credit',							'path' => '2:110'),
		'source'						=> array('name' => 'Source',							'path' => '2:115'),
		'edit_status'					=> array('name' => 'Edit Status',						'path' => '2:07'),
		'priority'						=> array('name' => 'Priority',							'path' => '2:10'),
		'object_cycle'					=> array('name' => 'Object Cycle',						'path' => '2:75'),
		'job_id'						=> array('name' => 'Fixture Identifier',				'path' => '2:22'),
		'program'						=> array('name' => 'Originating Program',				'path' => '2:65'),
		'object_name'					=> array('name' => 'Object Name (Title)',				'path' => '2:05'),
		'date_created'					=> array('name' => 'Date Created',						'path' => '2:55'),
		'date_released'					=> array('name' => 'Release Date',						'path' => '2:30'),
		'time_created'					=> array('name' => 'Time Created',						'path' => '2:60'),
		'time_released'					=> array('name' => 'Release Time',						'path' => '2:25'),
		'city'							=> array('name' => 'City',								'path' => '2:90'),
		'sublocation'					=> array('name' => 'Sub-Location',						'path' => '2:92'),
		'state'							=> array('name' => 'Province/State',					'path' => '2:95'),
		'country'						=> array('name' => 'Country Name',						'path' => '2:101'),
		'country_code'					=> array('name' => 'Country Code',						'path' => '2:100'),
		'trans_reference'				=> array('name' => 'Transmission Reference',			'path' => '2:103'),
	)
	);

	/**
	 * define tags to index and set available fields
	 * @param  string  $what  [exif,iptc]
	 */
	function setFields($what)
	{
		/**
		 * create defined_fields
		 * use a new array with lowercase keys without the md5sum entry
		 * MetaColumns() returns the keys always uppercase independent of ADODB_ASSOC_CASE
		 */
			$array = $GLOBALS['linpha']->db->MetaColumns(LIN_PREFIX."meta_".$what);
			$this->defined_fields[$what] = array();
			foreach($array AS $key=>$value)
			{
				if( $key != 'MD5SUM')
				{
					$new_key = strtolower($key);
					$this->defined_fields[$what][$new_key] = LinMetaData::$Tags[$what][$new_key]['name'];
				}
			}
	
		/**
		 * create available_fields
		 */
			foreach(LinMetaData::$Tags[$what] AS $key=>$value)
			{
				if( ! isset( $this->defined_fields[$what][$key] ))
				{
					$this->available_fields[$what][$key] = $value['name'];
				}
			}
	}

	/**
	 * show nice name for exif, iptc, builtin and own fields, for example: convert "exif_datetimeoriginal" to "DateTime Original"
	 */
	function getNameOfField( $key , $show_exifiptc_in_brackets)
	{
		$pos = strpos($key,'_');
		$first_part = substr($key,0,$pos);
		$last_part = substr($key,$pos+1);
		switch($first_part)
		{
		case 'builtin': // builtin and special fields like filename, imagesize and dimension
			$value = i18n( ucfirst( $last_part ) );
			break;
	
		case 'id': // own fields
			
			/**
			 * enable cache (only possible in object context)
			 * we dont want have to read this for each thumbnail
			 */
			if(isset($this->cachedFieldNames[$last_part]))
			{
				$value = i18n( ucfirst( $this->cachedFieldNames[$last_part] ) );
			}
			else
			{
				$data = $GLOBALS['linpha']->db->GetRow("SELECT name FROM ".LIN_PREFIX."meta_fields WHERE id = '".$last_part."'");
				$value = i18n( ucfirst( $data['name'] ) );

				if(isset($this))	// cache entry, but only in object context
				{
					$this->cachedFieldNames[$last_part] = $data['name'];
				}
			}
			break;
		case 'exif':
			$value = LinMetaData::$Tags['exif'][$last_part]['name'];
			if($show_exifiptc_in_brackets) {
				$value .= ' (exif)';
			}
			break;
		case 'iptc':
			$value = LinMetaData::$Tags['iptc'][$last_part]['name'];
			if($show_exifiptc_in_brackets) {
				$value .= ' (iptc)';
			}
			break;
		}
		
		return $value;
	}

} // end class LinMetaData

?>