<?php
/*
* Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
*                    Florian Angehrn 
* 
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
 * This class holds database related methods
 * @package Database
 */

class LinSql
{
private $isLoggedIn, $isAdmin;
public $config;	// sub class $linpha->sql->config

/**
 * constructor
 */
function __construct()
{
	
}

/**
 * connect to database
 */
function dbConnect()
{

	if(isset($_SESSION['installmode1']))
	{
		$db_type = $_SESSION['sql_dbtype'];
		$db_hostname = $_SESSION['sql_hostname'];
		$db_username = $_SESSION['sql_username'];
		$db_password = $_SESSION['sql_password'];
		$db_port = $_SESSION['sql_dbport'];
		$db_name = ($db_type == "sqlite" ? LINPHA_DIR.'/'.$_SESSION['sql_dir'] : '').$_SESSION['sql_dbname'];
		$db_prefix = $_SESSION['sql_prefix'];

	}
	else
	{
		include_once(LINPHA_DIR.'/lib/adodb/adodb-errorhandler.inc.php');
		
		$sql_dir = '';
		include(LINPHA_DIR.'/var/config.dir.php');	// no include_once if we reconnect from another directory...

		$include_file = LinFilesys::getFullPath( $sql_dir ) . '/config.sql.php';
		if( file_exists($include_file) )
		{
			include_once( $include_file );
		}
		else
		{
			linLog(LOG_TYPE_DB,LOG_EMERG,'db_connect',"Cannot connect to the database !!! Cannot get config file ./var/config.sql.php");
			
			echo "Cannot connect to the database, please contact admin!!!<br />";
			echo "Cannot get config file './var/config.sql.php'<br />";
			
			include(LINPHA_DIR."/include/db_connect_error.html");
			
			exit(1);
		}
	}
	
	define('LIN_PREFIX', $db_prefix);
	define('LIN_DB_TYPE', $db_type);

	/**
	 * set adodb global default to fetch ASSOC
	 */
	$ADODB_FETCH_MODE = 'ADODB_FETCH_ASSOC';
	
	/**
	 * force oracle to fetch lowercase
	 * 
	 * Note that for large recordsets, it might be better to disable
	 * RecordCount() emulation because substantial amounts of memory are
	 * required to cache the recordset for counting. Also there is a speed
	 * penalty of 40-50% if emulation is required. This is emulated in most
	 * databases except for PostgreSQL and MySQL. This variable is checked
	 * every time a query is executed, so you can selectively choose which
	 * recordsets to count.
	 */
	if(LIN_DB_TYPE == 'oci8po')
	{
		define('ADODB_ASSOC_CASE', 0);
		$ADODB_COUNTRECS = false;
	}
	define('ADODB_ASSOC_CASE', 0);

	/**
	* connect to db and check if connection was succesfull
	*/
	include_once(LINPHA_DIR.'/lib/adodb/adodb.inc.php');
	$GLOBALS['linpha']->db = ADONewConnection(LIN_DB_TYPE);
	switch(LIN_DB_TYPE)
	{
		case "sqlite":
			if(isset($_SESSION['installmode1']))
			{			
				$sql_dir = LinFilesys::getFullPath($_SESSION['sql_dir']);
				$GLOBALS['linpha']->db->Connect($sql_dir.'/database.sqlite');
			}
			else
			{
				$GLOBALS['linpha']->db->Connect(LinFilesys::getFullPath( $sql_dir ).'/database.sqlite');		
			}
		break;
		case "mysql":
			$GLOBALS['linpha']->db->Connect("$db_hostname:$db_port", $db_username, $db_password, $db_name);
		break;
		case "postgres":
			if(isset($db_password))
			{
				$GLOBALS['linpha']->db->Connect("host=$db_hostname port=$db_port user=$db_username password=$db_password dbname=$db_name");
			}
			else
			{
				$GLOBALS['linpha']->db->Connect("host=$db_hostname port=$db_port user=$db_username dbname=$db_name");
			}			
		break;
		case "oci8po":
			$GLOBALS['linpha']->db->PConnect($db_hostname, $db_username, $db_password, $db_name);
		break;
	}
	
	if( !isset($GLOBALS['linpha']->db) OR !$GLOBALS['linpha']->db->isConnected() )
	{
	    if( isset($_SESSION['installmode1']) )
	    {
	    	echo failed_msg().'<br />';
	    }
	
	    if( isset($GLOBALS['linpha']->db) )
	    {
	        echo $GLOBALS['linpha']->db->ErrorMsg().'<br />';
	    }
	    
	    if( isset($_SESSION['installmode2']) )
	    {
			echo "</form>";
			$show_next_button = false;
			include_once(LINPHA_DIR."/install/footer.php");
	    }
	    else
	    {
	    	if( isset($GLOBALS['linpha']->db) )
	    	{
	    		linLog(LOG_TYPE_DB,LOG_EMERG,'db_connect',"Cannot connect to database: ".$GLOBALS['linpha']->db->ErrorMsg());
	    	}
	    	else
	    	{
	    		linLog(LOG_TYPE_DB,LOG_EMERG,'db_connect',"Cannot connect to database!");
	    	}

	    	include(LINPHA_DIR."/include/db_connect_error.html");
	    }
	    exit(1);
	}
	
	
	/**
	 * register sub class LinSqlConfig
	 * (not during install, because we dont have any config table yet)
	 */
	if( ! isset($_SESSION['installmode2']) )
	{
		$this->config = new LinSqlConfig();
	}
	
} // end function dbConnect

/**
 * start the linpha session and check login information
 */
function startSession()
{
	$this->isLoggedIn = false;
	
	/**
	 * connect to db if not already done
	 */
	if( !isset($GLOBALS['linpha']->db) OR !$GLOBALS['linpha']->db->isConnected() )
	{
		$this->dbConnect();
	}
	
	/**
	 * start template
	 * this must be at execatly this place, because:
	 * 1. it needs to be after the database is connected, because
	 *    in the constructor of linTemplate() are configs from the db read
	 * 2. we need the template already in startSession(), because of the
	 *    syslog after loggin in
	 */
	$GLOBALS['linpha']->template = new linTemplate();
	global $linTpl;
	$linTpl = $GLOBALS['linpha']->template;


	/**
	 * set idCurrent
	 * add support for static links
	 * linpha2/index.php/album1/file.jpg
	 * will set the correct id
	 */
	if(isset($_GET['linId']))
	{
		$GLOBALS['linpha']->template->idCurrent = intval($_GET['linId']);
	}
	else
	{
		if( isset($_SERVER['PATH_INFO']) )
		{
			$id = LinSql::getIdFromFullFilename( substr($_SERVER['PATH_INFO'],1) );		// PATH_INFO starts with a '/' -> remove this
			
			if($id === false)
			{
				$GLOBALS['linpha']->template->idCurrent = 0;
			}
			else
			{
				$GLOBALS['linpha']->template->idCurrent = $id;
			}		
		}
		else
		{
			$GLOBALS['linpha']->template->idCurrent = 0;
		}
	}

	/**
	 * maybe the session is already started
	 * this should be no problem, hopefully
	 */
	if (session_id() == '')
	{
		session_name('linpha2');
		session_start();
	}

	/**
	 * read cookie
	 */
	if($this->config->value['sys_user_autologin']) {
		if(!isset($_SESSION['user_name']) && 
		isset($_COOKIE['linpha_userid'])&&$_COOKIE['linpha_userid']!=""&&
		isset($_COOKIE['linpha_password'])&&$_COOKIE['linpha_password']!="")
		{
			$query = $GLOBALS['linpha']->db->Execute("SELECT username, display_name FROM ".LIN_PREFIX."users ".
				"WHERE id = '".LinSql::linAddslashes($_COOKIE['linpha_userid'])."' ".
				"AND password = '".LinSql::linAddslashes($_COOKIE['linpha_password'])."'");
			if($data = $query->FetchRow())
			{
		        linLog(LOG_TYPE_USER,LOG_NOTICE,'login','User '.$_POST['username'].': successfully logged in.');
		        linSysLog(i18n("Successfully Logged In!"));
				
				$this->setUserInformation($data['username'],$_COOKIE['linpha_userid'],$data['display_name']);
			}
			else	// wrong cookie
			{
				setcookie('linpha_userid');	// delete cookie linpha_userid
				setcookie('linpha_password');	// delete cookie linpha_password
			}
		}
	}
	
	/**
	 * check login and fill session
	 */
	if( isset($_POST['cmd']) )
	{
		if($_POST['cmd']=='logout')
		{
			linLog(LOG_TYPE_USER,LOG_NOTICE,'logout','User '.$_SESSION['user_name'].': logged out');
			
			/**
			 * do not destroy the session because we save a lot of other informations in the session
			 * like: ref_url, use_ajax, ...
			 * $_SESSION = array();	// delete all session data
			 * session_destroy();		// destroy session
			 */
			unset($_SESSION['user_name']);
			unset($_SESSION['user_id']);
			unset($_SESSION['user_groups']);
			unset($_SESSION['user_display_name']);
			
			if(isset($_COOKIE['linpha_userid'])) {
				setcookie('linpha_userid');	// delete cookie linpha_userid
				setcookie('linpha_password');	// delete cookie linpha_password
			}
			
			linSysLog(i18n("Logged out."));
		}
		elseif($_POST['cmd']=='login')
		{
			if( !isset($_POST['username']) OR !isset($_POST['password']) OR empty($_POST['username']) OR empty($_POST['password']))
			{
				linSysLog(i18n("Please Fill In All Fields!"));
			}
			else
			{
				/**
				 * get user from DB
				 */
				$data = $GLOBALS['linpha']->db->GetRow("SELECT id, username, password, id, display_name FROM ".LIN_PREFIX."users ".
									"WHERE username='".LinSql::linAddslashes($_POST['username'])."'");
				
				if( isset($data['username']) && isset($data['password']) && $_POST['username'] == $data['username'] && md5($_POST['password']) == $data['password'])
				{
				    /**
			         * sucessfull login
				     */

			        linLog(LOG_TYPE_USER,LOG_NOTICE,'login','User '.$_POST['username'].': successfully logged in.');
			        linSysLog(i18n("Successfully Logged In!"));
					
					$this->setUserInformation($_POST['username'],$data['id'],$data['display_name']);
					
					if(isset($_POST['rememberme'])) {
						$this->setLinphaCookie($data['id'], md5($_POST['password']));
					}
	
					/**
					 * some security stuff
					 */
					unset($_POST['password']);
					unset($_REQUEST['password']);
				}
				else
				{
					/**
					 * failed login
					 */

			        linLog(LOG_TYPE_USER,LOG_WARNING,'login','User '.$_POST['username'].': login failed!');
					linSysLog(i18n("Login Failed!"));
				}
			}
		}
	}

	/**
	 * set isLoggedIn and isAdmin, check REMOTE_ADDR
	 * if $_SERVER['REMOTE_ADDR'] doesnt exists, we cant do this -> disable ip check
	 */
		$this->isAdmin = false;
		if(isset($_SESSION['user_id']) && isset($_SESSION['user_name']) && !empty($_SESSION['user_id']) && !empty($_SESSION['user_name']) )
		{
			if( isset($_SERVER['REMOTE_ADDR']) && ($_SESSION['REMOTE_ADDR'] != $_SERVER['REMOTE_ADDR']) )
			{
				/**
				 * ip check wrong!
				 */
				$this->isLoggedIn=false;
		
				$_SESSION = array();
				session_destroy();
			}
			else
			{
				$this->isLoggedIn=true;
				
				/**
				 * set isAdmin
				 */
				if(isset($_SESSION['user_groups']) && is_array($_SESSION['user_groups']))
				{
					$data = $GLOBALS['linpha']->db->GetRow("SELECT id FROM ".LIN_PREFIX."groups WHERE group_name = 'admin'");
					if(in_array($data['id'],$_SESSION['user_groups']))
					{
						$this->isAdmin = true;
					}
				}
			}
		}
		else
		{
			$this->isLoggedIn=false;
		}

	/**
	 * initialize some session variables
	 */
		if( ! isset( $_SESSION['user_groups']) )
		{
			$_SESSION['user_groups'] = Array();
		}

		/** 
		 * it's much nicer if we can later just use count($_SESSION['basket_ids']) etc.
		 * without additional isset() and so on..
		 */
		if( ! isset( $_SESSION['basket_ids'] ) )
		{
			$_SESSION['basket_ids'] = Array();
		}

	/**
	 * apply user overridden config values
	 * this has also to be here because the session isn't started yet in reloadConfig()
	 * but also has to be in reloadConfig() if it will be called during the script 
	 */
	if(isset($_SESSION['user_id']))
	{
		$query = $GLOBALS['linpha']->db->Execute("SELECT option_name, option_value " .
				"FROM ".LIN_PREFIX."config WHERE user_id = '".LinSql::linAddslashes($_SESSION['user_id'])."'");
		
		while($data = $query->FetchRow(ADODB_FETCH_NUM))
		{
			$GLOBALS['linpha']->sql->config->value[$data['0']] = $data['1'];
		}
	}
} // end function startSession

/**
 * set user infos after correct login
 */
function setUserInformation($username,$userid,$displayname)
{
	/**
	 * store user stuff
	 */
	$_SESSION['REMOTE_ADDR'] = $_SERVER['REMOTE_ADDR'];
	$_SESSION['user_name'] = $username;
	$_SESSION['user_id'] = $userid;
	
	/**
	 * Store users display name. If display name is blank - store username.
	 */
	$_SESSION['user_displayname'] = (empty($displayname)) ? $_SESSION['user_name'] : $displayname;

	/**
	 * get user_groups
	 */
	$query = $GLOBALS['linpha']->db->Execute("SELECT group_id FROM ".LIN_PREFIX."user_group WHERE user_id = '".$userid."'");
	while($data = $query->FetchRow())
	{
		$_SESSION['user_groups'][] = $data['group_id'];
	}

}



/**
 * permission stuff
 */

/**
 * get permission string from photo id
 * 
 * @param  photo id
 * @return  array  array(stage, permission) stage=0 if permission is not inherited
 */
function getPerm( $photos_id, $perm_type )
{
	/**
	 * special behaviour for top permission
	 * the top permission is only for inheritation
	 * for example:
	 *  top perm = ;admin;
	 *  perm folder1 = ;public;
	 *  perm folder2 = ;admin;
	 *  users must be able to see folder1
	 */
	if($photos_id == 0 && $perm_type == 'read')
	{
		return array(0, ';public;');
	}
	
	for($i = 0 ; true ; $i++)
	{
		$data = $GLOBALS['linpha']->db->GetRow("SELECT permission FROM ".LIN_PREFIX."permissions " .
				"WHERE perm_type = '".LinSql::linAddslashes($perm_type)."' " .
				"AND photos_id = '".LinSql::linAddslashes($photos_id)."'");
		
		if( isset( $data[0] ) && !empty( $data[0] ) )
		{
			return array($i, $data[0]);
		}
		else
		{
			$data = $GLOBALS['linpha']->db->GetRow("SELECT parent_id FROM ".LIN_PREFIX."photos " .
				"WHERE id = '".LinSql::linAddslashes($photos_id)."'");

			if( isset( $data[0] ) )
			{
				$photos_id = $data[0];
			}
			else
			{
				/**
				 * no parent_id found -> we're on top
				 */
				$photos_id = 0;
			}
		}
	}
} // end function getPerm

/**
 * returns true if we're allowed to see this photo
 */
function photoIsAllowed( $photo_id, $sql_perm_type = 'read' )
{
	/**
	 * admin sees always all photos
	 */
	if($GLOBALS['linpha']->sql->isAdmin())
	{
		return true;
	}

	list($stage,$perm) = LinSql::getPerm( $photo_id, $sql_perm_type );
	$array_photo_perm = linExplodeAndSlice(';',$perm);
	
	foreach($array_photo_perm AS $value)
	{
		if( $value == 'public' OR in_array($value, $_SESSION['user_groups']) )
		{
			return true;
		}
	}
	return false;
}

/**
 * method to check if we are logged in
 */
function isLoggedIn()
{
	return $this->isLoggedIn;
}

/**
 * method to check if we are admin
 * 
 * just returns the content of $isAdmin which has status private
 * to prevent modification from external
 */
function isAdmin()
{
	return $this->isAdmin;
}

/**
 * check permissions, basket_*, metadata_*, download, statistics, watermark, ...
 */
function checkPermission($perm_type)
{
	if($GLOBALS['linpha']->sql->isAdmin())
	{
		return true;
	}

	$data = $GLOBALS['linpha']->db->GetRow("SELECT permission FROM ".LIN_PREFIX."permissions " .
			"WHERE perm_type = '".LinSql::linAddslashes($perm_type)."'");
	$array_permissions = linExplodeAndSlice(';',$data['permission']);

	if(in_array('public',$array_permissions))
	{
		return true;
	}

    $array = array_intersect($_SESSION['user_groups'],$array_permissions);
    if(count($array) > 0)
    {
        return true;
    }
    else
    {
    	return false;
    }
}

/**
 * other stuff
 */

/**
 * add slashes for sql data
 * 
 * @author  flo
 * @param  string  $str  string to escape
 * @return  string  escaped string
 */
function linAddslashes($str)
{
	if(LIN_DB_TYPE == 'sqlite')
	{
		$str = str_replace("'","''",$str);
	}
	else
	{
		$str = addslashes($str);
	}
	return $str;
} // end method linAddslashes

/**
* addslashes to all values of an array
* used for posted data from <select>'s
*/
function linAddslashesArray(&$array)
{
	if(is_array($array))
	{
		foreach($array AS $key=>$value)
		{
			$array2[$key] = LinSql::linAddslashes($value);
		}
	}
	else
	{
		$array2 = Array();
	}
	
	return $array2;
}

/**
 * returns the id of the given full filename
 */
function getIdFromFullFilename( $full_filename )
{
	if( empty($full_filename))
	{
		return false;
	}
	
	$array_filenames = explode('/', $full_filename );
	$parent_id = 0;
	
	foreach($array_filenames AS $value)
	{
		$query = $GLOBALS['linpha']->db->Execute("SELECT id FROM ".LIN_PREFIX."photos " .
				"WHERE parent_id = '".LinSql::linAddslashes($parent_id)."' AND " .
					  "name = '".LinSql::linAddslashes($value)."'");
		$data = $query->FetchRow();
		
		if( !isset($data['id']) OR empty($data['id']))
		{
			return false;
		}
		
		$parent_id = $data['id'];
	}
	
	return $data['id'];
}

/**
 * returns an array with the full pathname of the given id
 */
function getFullFilenameFromId( $photos_id )
{
	$query = $GLOBALS['linpha']->db->Execute("SELECT id, name, parent_id FROM ".LIN_PREFIX."photos " .
			"WHERE id = '".LinSql::linAddslashes($photos_id)."'");
	$data = $query->FetchRow();
	
	if( !isset( $data['id']) OR empty($data['id']) )
	{
		return array();
	}
	
	$array_names[ $photos_id ] = $data['name'];
	$parent_id = $data['parent_id'];
	
	for($i = 0; $parent_id != 0 ; $i++)
	{
		$query = $GLOBALS['linpha']->db->Execute("SELECT id, name, parent_id FROM ".LIN_PREFIX."photos " .
				"WHERE id = '".$parent_id."'");
		$data = $query->FetchRow();

		$array_names[ $data['id'] ] = $data['name'];
		$parent_id = $data['parent_id'];
		
		/**
		 * really need this hard limit
		 * because if we try to get infos from an id which doesn't exist anymore
		 * we would run in an endless loop
		 */
		
		if($i == 1000)
		{
			linSysLog(i18n("Error: cannot get informations, you want to run the database cleanup script"));
			linLog(LOG_TYPE_IMPORT,LOG_WARNING,'integrity',"cannot get file informations for file with id '".$photos_id."', you want to run the database cleanup script");

			/**
			 * don't show this file/folder, go back to parent view
			 * @todo  what if we're on search/new_images/etc. ?? maybe set a "reset" variable which resets
			 * each view to top folder
			 * better: use an error handling after each call of getFullFilenameFromId()
			 * example: check with if( count( $array_names ) == 0 )
			 */
			$GLOBALS['parent_id'] = 0;
			$GLOBALS['view'] = 'thumb_view';

			/**
			 * stop the loop
			 */
			return array();
		}
	}
	
	$array_names = array_reverse($array_names, true);

	return $array_names;
}

/**
 * extends getFullFilenameFromId with album path and implode for direct use in file operations
 */
function getFullImagePath( $photo_id )
{
	$album_dir = LinSql::getPath('album');
	
	if($photo_id == 0)
	{
		return $album_dir;	// return album_dir without ending slash
	}
	
	$array_path = LinSql::getFullFilenameFromId( $photo_id );
	
	if( count($array_path) == 0 )
	{
		return false;
	}
	else
	{
		return $album_dir .'/'. implode('/',$array_path);
	}
}

/**
 * returns the correct path to the album, cache or tmp directory
 * pay attention to use this getPath and not access directly 'sys_path_album_dir' !
 */
function getPath($what)
{
	$path = $GLOBALS['linpha']->sql->config->value['sys_path_'.$what.'_dir'];
	return LinFilesys::getFullPath( $path );
}

/**
* Set autologin-cookie
*
* can only be set once per request
* must be set before a header is already sent
*
* @uses  LinSql::startSession
*/
function setLinphaCookie($userid, $md5pw)
{
    /**
    * calculate correct path for cookie
    * needs always to be the linpha root folder
    * for example: http://localhost/linpha/index.php -> must be /linpha/
    * 
    * @todo  what if linpha is included from another directory?
    */
        /**
        * get directory and append needed ending slash
        */
        $php_self_dir = dirname($_SERVER['PHP_SELF']);
        $php_self_dir .= '/';
    
        if(LINPHA_DIR != '.')
        {
            if(LINPHA_DIR == '..')
            {
                $num = 1;
            }
            else
            {
                $num = 1 + substr_count(LINPHA_DIR, '/');
            }
            
            /**
            * remove directories
            */
            for( ; $num > 0 ; $num--)
            {
                /**
                * remove last slash
                */
                $php_self_dir = substr($php_self_dir,0,strlen($php_self_dir)-1);
                
                /**
                * get last slash
                */
                $pos = strrpos($php_self_dir,'/');
                
                /**
                * remove last directory
                */
                $php_self_dir = substr($php_self_dir,0,$pos+1);
            }
        }
    
    /**
    * set the cookie
    */
	setcookie("linpha_userid", $userid, (time() + 60 * 60 * 24 * 31), $php_self_dir);
	setcookie("linpha_password", $md5pw, (time() + 60 * 60 * 24 * 31), $php_self_dir);
}

} // end class LinSql


/**
 * sub-class LinSqlConfig
 */
class LinSqlConfig
{
	public $value;
	
/**
 * constructor
 * 
 * read all config values in $linpha->sql->config->*
 */
function LinSqlConfig()
{
	$this->reloadConfig();
}

/**
 * (re-)load config from db to array
 * the overriden user configs will be load in startSession() 
 */
function reloadConfig()
{
	unset($this->value);

	/**
	 * load system values
	 */
	$query = $GLOBALS['linpha']->db->Execute("SELECT option_name, option_value " .
			"FROM ".LIN_PREFIX."config WHERE user_id = '0'");
	while($data = $query->FetchRow(ADODB_FETCH_NUM))
	{
		$this->value[$data['0']] = $data['1'];
	}

	/**
	 * apply user overridden config values
	 * this has also to be in startSession() because the session isn't started yet in reloadConfig()
	 * but also has to be in reloadConfig() if it will be called during the script 
	 */
	if(isset($_SESSION['user_id']) && !empty($_SESSION['user_id']))
	{
		$query = $GLOBALS['linpha']->db->Execute("SELECT option_name, option_value " .
				"FROM ".LIN_PREFIX."config WHERE user_id = '".$_SESSION['user_id']."'");
		
		while($data = $query->FetchRow(ADODB_FETCH_NUM))
		{
			$GLOBALS['linpha']->sql->config->value[$data['0']] = $data['1'];
		}
	}
}

function writeConfig($option_name,$option_value)
{
	$GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."config (option_name, option_value) VALUES (" .
			"'".$GLOBALS['linpha']->sql->linAddslashes($option_name)."'," .
			"'".$GLOBALS['linpha']->sql->linAddslashes($option_value)."'" .
		")");
}

function updateConfig($option_name,$option_value,$user_id=0)
{
	if($user_id == 0)
	{
		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."config SET option_value = ".
				"'".LinSql::linAddslashes($option_value)."'".
			"WHERE option_name = '".LinSql::linAddslashes($option_name)."' " .
			"AND user_id = '0'");
	}
	else
	{
		/**
		 * if the value is the same like the system value, delete it
		 */
		$data = $GLOBALS['linpha']->db->GetRow("SELECT option_value FROM ".LIN_PREFIX."config " .
				"WHERE option_name = '".LinSql::linAddslashes($option_name)."' AND user_id = '0'");
		if($option_value == $data['option_value'])
		{
			$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."config " .
				"WHERE option_name = '".LinSql::linAddslashes($option_name)."' " .
				"AND user_id = '".$user_id."'");
		}
		else
		{
			/**
			 * insert or update?
			 */
			$query = $GLOBALS['linpha']->db->Execute("SELECT id FROM ".LIN_PREFIX."config " .
				"WHERE option_name = '".$option_name."' AND user_id = '".$user_id."'");
			if( ! $query->EOF ) // update
			{
				$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."config SET option_value = ".
						"'".LinSql::linAddslashes($option_value)."'".
					"WHERE option_name = '".LinSql::linAddslashes($option_name)."' " .
					"AND user_id = '".$user_id."'");
			}
			else // insert
			{
				$GLOBALS['linpha']->db->Execute("INSERT into ".LIN_PREFIX."config " .
						"(option_name, option_value, user_id) " .
						"VALUES ('".LinSql::linAddslashes($option_name)."', " .
								"'".LinSql::linAddslashes($option_value)."', " .
								"'".$user_id."')");
				
			}			
		}
	}	
}

} // end sub-class LinSqlConfig

?>