<?php
/*
* Copyright (c) 2004 Heiko Rutenbeck <bzrudi@tuxpower.de>
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/**
* File to create mail with image attachementes
* 
* with the mail() function we have to read the whole email with all images in a
* variable and passed to mail() -> not possible to send big mails (memory limit)
* maybe we should use not mail(), but a script which use fsockopen()
*/

if(!defined('LINPHA_DIR')) { exit(1); }

/**
* overhead of memory used more than only the filesize of the images
* overhead of base64 is 33%, need at least 50% because we have some other data too...
*/
$overhead = 1.50;
$overhead_base64 = 4/3;
$tot_filesize = 0;
$at_least_one_image = false;
$max_mail_size = $GLOBALS['linpha']->sql->config->value['sys_basket_mail_max_size'];

if(($mem_limit = linGetMemoryLimit()) === false)
{
    $no_mem_limit = true;
}
else
{
    $no_mem_limit = false;
    $mem_limit = $mem_limit*1024*1024;
}

function get_available_mem()
{
	global $mem_limit;

	if(function_exists('memory_get_usage'))	// we need php > 4.3.2
	{
		$available_mem = $mem_limit - memory_get_usage();
	}
	else
	{
		$available_mem = $mem_limit - 1024*1024*2;	// average value
	}
	
	return $available_mem;
}


try
{
	if( empty($_POST['mail_address']) OR empty($_POST['sender_address']) OR empty($_POST['subject']) )
	{
		throw new Exception(i18n("Please Fill In All Fields!"));
	}


	$tmp_folder = LinFilesys::createTmpFolder('mail');

	/**
	* create temporary folder containing all images
	*/
	reset($_SESSION['basket_ids']);
	while(list($key, $image_id) = each($_SESSION['basket_ids']))
	{
		$data = $GLOBALS['linpha']->db->GetRow("SELECT img_type FROM ".LIN_PREFIX."photos WHERE id = '".LinSql::linAddslashes( $image_id )."'");
		if(isset($data['img_type']) && LinIdentify::isSupportedImage( $data['img_type'] ) )
		{
			$at_least_one_image = true;
			
			$full_filename = LinSql::getFullImagePath( $image_id );
			$output = $tmp_folder.'/'.$image_id.'.jpg';
		
		    /**
		     * warning: session will be closed!
		     */
			$image = new LinImage();
			if( ! $image->createImage( $image_id, $overrideSettings=Array() ) )
			{
				linSysLog( i18n("Warning: Image couldn't be created!").': '.$full_filename );
				//$error = true; no error, try to continue...
			}
			else
			{
				if(file_exists($image->output_file))
				{
					copy($image->output_file,$output);
					$tot_filesize += filesize($output);
				}
				else
				{
					linSysLog( i18n("Warning: Created File doesn't exists!") );
					//$error = true; no error, try to continue...
				}
			}
		}
	}
	$real_mail_size = round($tot_filesize*$overhead);
	
	/**
	* check if there were only videos or other files
	*/
	if(!$at_least_one_image)
	{
		throw new Exception(i18n("No Images Available! Videos are currently not supported."));
	}
	

	/**
	 * check if mail is getting to big
	 */	
	if(
		($real_mail_size > $max_mail_size ) // only overhead of base64 (33%)
        OR 
        ( !$no_mem_limit AND $real_mail_size > get_available_mem())
	)	
	{
		/**
		* but still delete images
		*/
		$d = dir($tmp_folder);
		while (false !== ($entry = $d->read()))
		{
			if($entry != '.' && $entry != '..')
			{
				unlink($tmp_folder.'/'.$entry);
			}
		}
		
		/**
		* delete temporary folder
		*/
		rmdir($tmp_folder);

		throw new Exception(
			i18n("Error: The E-Mail is too big.").
			"<br /><br />".
			sprintf(i18n("Allowed size: %sBytes. Your selected images use %sBytes."),$max_mail_size,$real_mail_size).
			"<br /><br />".
			i18n("Remove some images or use the download zipped albums feature!")
		);
	}
	
	include_once(LINPHA_DIR.'/lib/classes/phpmailer.class.php');
	
	$mail = new PHPMailer();

	$mail->IsSmtp();
	$mail->Host = $GLOBALS['linpha']->sql->config->value['sys_basket_mail_smpthost'];		
	$mail->From = $_POST['sender_address'];
	$mail->FromName = $_POST['sender_address'];	// otherwise, a default name like "Root User" is taken
	$mail->AddAddress($_POST['mail_address']);		
	$mail->Subject = $_POST['subject'];
	$mail->Body = $_POST['text_message'];
	
	$d = dir($tmp_folder);
	while (false !== ($entry = $d->read()))
	{
		if($entry != '.' && $entry != '..')
		{
			if($_POST['mail_mode'] == "txt")
			{
				$handle = fopen($tmp_folder.'/'.$entry,"rb");
				$file_content = fread( $handle, filesize( $tmp_folder.'/'.$entry ) );
				fclose($handle);
	
				$mail->AddStringAttachment("$file_content", "$entry");

				unlink($tmp_folder.'/'.$entry);
			}
			else
			{
				$image_filename = "$tmp_folder/$entry";
				
				$mail->AddAttachment("$image_filename", "$entry", "base64", "image/jpeg");
				
				/**
				 * We have to delete tmp files later, as AddAttachment() 
				 * just adds a reference to the images and unlink would delete 
				 * them before they were send out... 
				 */
				$array_files[] = $image_filename; 
			}
		}
	}
	$d->close();

	if($_POST['mail_mode'] == "html")
	{
		$mail->IsHTML(true);
	}
	
	if(!$mail->Send())
	{
		/**
		 * no exception, because we still need to delete the images
		 */
		linSysLog(i18n("Error: E-Mail couldn't be sent. Contact the Administrator."));
	}

	$mail->ClearAddresses();	
	$mail->ClearAttachments();
	
	/**
	* delete temporary folder
	*/
	if($_POST['mail_mode'] == "html")
	{		
		foreach($array_files as $filename)
		{
			unlink($filename);
		}
	}		
	rmdir($tmp_folder);

	linSysLog(
		i18n("Email Sent To").': '.$_POST['mail_address'].
		'<br />'.i18n("Size Of E-Mail").": ".niceFilesize($real_mail_size,1)
	);	
}
catch(Exception $error)
{
	linSysLog("Error: ".$error -> getMessage());
}
?>