<?php
/*
 * Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
 *                    Florian Angehrn 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * Filemanager
 * @package Modules
 */
 
/**
 * Short Permission Description
 * If a user has write permission to folder A, it means that 
 * he can upload images to this folder, rename content, delete content
 * But he cannot rename or delete the folder itself
 * 
 * 
 * it doesnt make sense to give write permissions to the files itself
 * 
 * 
 * @todo  make an option with allowed filetypes to upload: images, images and videos, all
 * default setting: images
 * its very dangerous if users may upload php files etc.!! or even video files may be
 * prepared with malicious code (there are known security holes in some video players!)
 */

/**
 * @todo  update md5sum while renaming or moving a folder
 * on a folder, the md5sum is based on $md5sum = md5($dirname.'/'.$filename);
 */

if(!defined('LINPHA_DIR')) { exit(1); }

/**
 * some double used i18n entries
 * @todo  what should we do with double used entries??
 */
	$str_checkpermissions = "Check the permissions in the linpha albums folder. (This means NOT the write permissions page in the admin section)";
	$str_autoimportdisabled = "Autoimport is disabled, you need now to start the manual import to make your changes appear, but you maybe don't have permissions to do that...";
	$str_nopermission = "You don't have permissions to this folder!";
	$str_filealreadyexists = "A File/Folder with the same name already exists!";

/**
 * init stuff
 */
	$parent_id = $linpha->template->idCurrent;
	
	/**
	 * check global permission
	 */
		if(! $linpha->sql->config->value['plugins_filemanager_enable']
			&& !$linpha->sql->isAdmin())
		{
			echo i18n("No Access!");
			exit();
		}
		
		if( ! $linpha->sql->photoIsAllowed( $parent_id, 'read' ) )
		{
			echo i18n("No Access!");
			exit();
		}
		
		/**
		 * use this for child objects
		 */
		if( $linpha->sql->photoIsAllowed( $parent_id, 'write' ) )
		{
			$isWriteAble = true;
		}
		else
		{
			$isWriteAble = false;
		}


	$i = 0;
	$sum['size']  = 0;
	$sum['files'] = 0;
	
	$use_posix = checkPosix();
	
	if($use_posix) {
		$uid = @posix_getuid(); // get user id of current process
	} else {
		$uid = 0;
	}
	
	/**
	 * upload stuff
	 */
	include_once(LINPHA_DIR.'/lib/classes/archiver.class.php');
	$apps = new Archive_Applications();
	$apps->searchApps();
	
	$appselect  = '<option value="-">'. i18n("Extract Archive With"). '</option>';
	foreach($apps->found_apps AS $key=>$value)
	{
		$value = $key.' (.'.$apps->apps[$key]['file_ext'].')';
		$appselect .= '<option value="'.$key.'">'.$value.'</option>';
	}

/**
 * output buffering, to use in template
 */
ob_start();

/**
 * parsing POST data
 * 
 * cmd = file_action -> show forms
 * cmd = do_file_action -> do actions
 */
if(isset($_REQUEST['cmd']))
{
	try
	{
		switch($_REQUEST['cmd'])
		{
	case 'file_action':
			if(!isset($_REQUEST['checkedfiles']))
			{
				throw new Exception(i18n("No Files Selected"));
			}
			if(!isset($_REQUEST['file_action']))
			{
				throw new Exception(i18n("No Action Selected"));
			}
			
			?>
			<form method="POST" action="<?php echo LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$parent_id; ?>">
			<?php
			switch($_REQUEST['file_action'])
			{
			case 'rename':
				$data = $linpha->db->GetRow("SELECT name, parent_id FROM ".LIN_PREFIX."photos " .
						"WHERE id = '".LinSql::linAddslashes($_REQUEST['checkedfiles'][0])."'");
				
				echo '<b>'.i18n("Rename File/Folder").':</b><br />';
				?>
				<input type="text" name="new_name" value="<?php echo htmlspecialchars($data['name'],ENT_QUOTES); ?>" class="linForms" style="width: 300px;" />
				<input type="hidden" name="file_id" value="<?php echo $_REQUEST['checkedfiles'][0]; ?>" />
				<?php
				break;
	
			case 'move':
			case 'copy':
				if( $_REQUEST['file_action'] == 'move')
				{
					echo i18n("Move These Files/Folders");
				}
				elseif( $_REQUEST['file_action'] == 'copy')
				{
					echo i18n("Copy These Files/Folders");
				}
				echo ':<br />';

				foreach($_REQUEST['checkedfiles'] AS $value)
				{
					$data = $linpha->db->GetRow("SELECT name FROM ".LIN_PREFIX."photos " .
						"WHERE id = '".LinSql::linAddslashes($value)."'");
					echo htmlspecialchars($data['name'],ENT_QUOTES).', ';
				}
				echo '<br /><br />';
				echo i18n("To").':';
				echo '<br />';
				?>
				<select name="movecopy_to" class="linForms">
				<option value="0">/</option>
				<?php
				linBuildAlbumSelect( $with_all_albs_entry=false );
				?>
				</selet>
				<input type="hidden" name="checkedfiles" value="<?php echo implode(';',$_REQUEST['checkedfiles']); ?>" />
				<?php
				break;
			case 'delete':
				echo i18n("Delete all the following files/folders including contents:").'<br />';
				foreach($_REQUEST['checkedfiles'] AS $value)
				{
					$data = $linpha->db->GetRow("SELECT name FROM ".LIN_PREFIX."photos " .
						"WHERE id = '".LinSql::linAddslashes($value)."'");
					echo htmlspecialchars($data['name'],ENT_QUOTES).', ';
				}
				echo '<br /><br />';
				echo i18n("Really Sure?");
				?>
				<input type="hidden" name="checkedfiles" value="<?php echo implode(';',$_REQUEST['checkedfiles']); ?>" />
				<?php
				break;
			case 'changeperm':
				$full_filename = LinSql::getFullImagePath( $_REQUEST['checkedfiles'][0]);

				if( $full_filename == false )
				{
					throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
				}

				$perm = getReadableFileperms($full_filename);
				
				(substr($perm, 1, 1)!="-" ? $ur = 1 : '');
				(substr($perm, 2, 1)!="-" ? $uw = 1 : '');
				(substr($perm, 3, 1)!="-" ? $ux = 1 : '');
				(substr($perm, 4, 1)!="-" ? $gr = 1 : '');
				(substr($perm, 5, 1)!="-" ? $gw = 1 : '');
				(substr($perm, 6, 1)!="-" ? $gx = 1 : '');
				(substr($perm, 7, 1)!="-" ? $or = 1 : '');
				(substr($perm, 8, 1)!="-" ? $ow = 1 : '');
				(substr($perm, 9, 1)!="-" ? $ox = 1 : '');

				echo i18n("Change Permission Of:").' '.htmlspecialchars($full_filename,ENT_QUOTES);
				?>
				<br />
				<table border="1" cellpadding="0" cellspacing="0">
					<tr>
						<td width="100">&nbsp;</td>
						<td class='admintable' width="75"><?php echo i18n("Read"); ?></td>
						<td class='admintable' width="75"><?php echo i18n("Write"); ?></td>
						<td class='admintable' width="75"><?php echo i18n("Execute"); ?></td>
					</tr>
					<tr>
						<td class='admintable'><?php echo i18n("Owner"); ?></td>
						<td><input name="ur" type="checkbox" value="4"<?=(isset($ur) ? ' checked' : '')?>></td>
						<td><input name="uw" type="checkbox" value="2"<?=(isset($uw) ? ' checked' : '')?>></td>
						<td><input name="ux" type="checkbox" value="1"<?=(isset($ux) ? ' checked' : '')?>></td>
					</tr>
					<tr>
						<td class='admintable'><?php echo i18n("Group"); ?></td>
						<td><input name="gr" type="checkbox" value="4"<?=(isset($gr) ? ' checked' : '')?>></td>
						<td><input name="gw" type="checkbox" value="2"<?=(isset($gw) ? ' checked' : '')?>></td>
						<td><input name="gx" type="checkbox" value="1"<?=(isset($gx) ? ' checked' : '')?>></td>
					</tr>
					<tr>
						<td class='admintable'><?php echo i18n("All Others"); ?></td>
						<td><input name="or" type="checkbox" value="4"<?=(isset($or) ? ' checked' : '')?>></td>
						<td><input name="ow" type="checkbox" value="2"<?=(isset($ow) ? ' checked' : '')?>></td>
						<td><input name="ox" type="checkbox" value="1"<?=(isset($ox) ? ' checked' : '')?>></td>
					</tr>
				</table>
				<input type="hidden" name="file_id" value="<?php echo $_REQUEST['checkedfiles'][0]; ?>" />
				<?php
				break;
			}
			?>
			<input type="hidden" name="cmd" value="do_file_action" />
			<input type="hidden" name="file_action" value="<?php echo $_REQUEST['file_action']; ?>" />
			<input type="submit" name="submit" value="<?php echo i18n("Submit"); ?>" class="linButton" />
			</form>
			<hr /><br />
			<?php
		break;

	case 'do_file_action':
			
		switch($_POST['file_action'])
		{
		case 'rename':
			$new_name = removeUnWantedChars( $_POST['new_name'] );
			
			$full_filename = LinSql::getFullImagePath( $_POST['file_id'] );

			if( $full_filename == false )
			{
				throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
			}

			$parent_dirname = dirname($full_filename);
			
			/**
			 * check write permission of parent folder
			 */
			$data = $linpha->db->GetRow("SELECT parent_id FROM ".LIN_PREFIX."photos " .
					"WHERE id = '".LinSql::linAddslashes($_POST['file_id'])."'");
			if( ! $linpha->sql->photoIsAllowed( $data['parent_id'], 'write') )
			{
				throw new Exception(i18n($str_nopermission));
			}				

			if( nameAlreadyExists( $new_name, $data['parent_id'] ))
			{
				throw new Exception(i18n($str_filealreadyexists));
			}

			if( ! @rename( $full_filename , $parent_dirname."/".$new_name ) )
			{
				throw new Exception(i18n("Rename File/Folder Failed!").
					htmlspecialchars($full_filename,ENT_QUOTES).' '.i18n("to").' '.htmlspecialchars($parent_dirname.'/'.$new_name,ENT_QUOTES).
					LIN_NL.i18n($str_checkpermissions)
				);
			}

			/**
			 * rename on filesys was successful
			 * -> rename also in db
			 */
			$linpha->db->Execute("UPDATE ".LIN_PREFIX."photos SET name = '".LinSql::linAddslashes($new_name)."' " .
					"WHERE id = '".LinSql::linAddslashes($_POST['file_id'])."'");

			linSysLog(i18n("Successfully Renamed!"));
			linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_rename',"renamed file (".$full_filename." to ".$parent_dirname."/".$new_name.")");
		break;

		case 'move':
		case 'copy':
			/**
			 * check write permission of target folder
			 * AND it will also check if it is a valid id (?)
			 */
			if( ! $linpha->sql->photoIsAllowed( $_POST['movecopy_to'], 'write') )
			{
				throw new Exception(i18n($str_nopermission));
			}

			/**
			 * get target name
			 */
			$full_destparentname = LinSql::getFullImagePath( $_POST['movecopy_to'] );

			if( $full_destparentname == false )
			{
				throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_destparentname);
			}

			/**
			 * check write permission of destination parent folder
			 */
			if( ! is_writable($full_destparentname) )
			{
				throw new Exception(
					sprintf(
						i18n("Cannot Copy/Move to Folder \"%s\" because its not writable!"),
						htmlspecialchars($full_destparentname,ENT_QUOTES)
					).
					LIN_NL.
					i18n($str_checkpermissions)
				);
			}


			/**
			 * go through each folder
			 */
			$checkedfiles = explode(';',$_POST['checkedfiles']);
			foreach($checkedfiles AS $value)
			{
				/**
				 * get full source name 
				 */
				$full_sourcename = LinSql::getFullImagePath( $value );
				
				if( $full_sourcename == false )
				{
					throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
				}

				/** 
				 * check read permission of source folder (not parent! ...) and content
				 */
				if( ! $linpha->sql->photoIsAllowed( $value, 'read') )
				{
					throw new Exception(i18n($str_nopermission));
				}

				/**
				 * check if destination is empty
				 */
				if( nameAlreadyExists( basename($full_sourcename), $_POST['movecopy_to'] ))
				{
					throw new Exception(i18n($str_filealreadyexists));
				}	

				/**
				 * move
				 */
				if( $_POST['file_action'] == 'move')
				{
					/**
					 * check write permission of parent source folder
					 */
					$data = $linpha->db->GetRow("SELECT parent_id FROM ".LIN_PREFIX."photos " .
						"WHERE id = '".LinSql::linAddslashes($value)."'");
					if($_POST['file_action'] == 'move')
					{
						if( ! $linpha->sql->photoIsAllowed( $data['parent_id'], 'write') )
						{
							throw new Exception(i18n($str_nopermission));
						}
					}

					if( ! rename($full_sourcename, $full_destparentname.'/'.basename($full_sourcename) ) )
					{
						linSysLog(
							sprintf(
								i18n("Error While Moving Files/Folders \"%s\" to \"%s\"!"),
								htmlspecialchars($full_sourcename,ENT_QUOTES),
								htmlspecialchars($full_destparentname.'/'.basename($full_sourcename),ENT_QUOTES)
							)
						);
						linSysLog(i18n($str_checkpermissions));
						
						linLog(LOG_TYPE_FILEMANAGER,LOG_ERR,'fm_move',"cannot moved file/folder (".$full_sourcename." to ".$full_destparentname.'/'.basename($full_sourcename).")");
					}
					else
					{
						$linpha->db->Execute("UPDATE ".LIN_PREFIX."photos " .
							"SET parent_id = '".LinSql::linAddslashes($_POST['movecopy_to'])."' " .
							"WHERE id = '".LinSql::linAddslashes($value)."'");

						linSysLog(
							sprintf(
								i18n("Successfully Moved \"%s\" to \"%s\"!"),
								htmlspecialchars($full_sourcename,ENT_QUOTES),
								htmlspecialchars($full_destparentname.'/'.basename($full_sourcename),ENT_QUOTES)
							)
						);
						
						linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_move',"moved file/folder (".$full_sourcename." to ".$full_destparentname.'/'.basename($full_sourcename).")");
					}			
				}
				/**
				 * copy
				 */
				elseif( $_POST['file_action'] == 'copy')
				{
					/**
					 * copy files/folders (only which we have read permission)
					 */
					copy_r( $value, $full_sourcename, $full_destparentname);
					
					if( ! file_exists( $full_destparentname.'/'.basename($full_sourcename) ))
					{
						linSysLog(
							sprintf(
								i18n("Error While Copying Files/Folders \"%s\" to \"%s\"!"),
								htmlspecialchars($full_sourcename,ENT_QUOTES),
								htmlspecialchars($full_destparentname.'/'.basename($full_sourcename),ENT_QUOTES)
							)
						);
						linSysLog(i18n($str_checkpermissions));
						
						linLog(LOG_TYPE_FILEMANAGER,LOG_ERR,'fm_copy',"Cannot copy file/folder (".$full_sourcename." to ".$full_destparentname.'/'.basename($full_sourcename).")");
					}
					else
					{
						linSysLog(
							sprintf(
								i18n("Successfully Copied \"%s\" to \"%s\"!"),
								$full_sourcename,
								$full_destparentname.'/'.basename($full_sourcename)
							)
						);
						
						linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_copy',"Copied file/folder (".$full_sourcename." to ".$full_destparentname.'/'.basename($full_sourcename).")");
					}
				}
			}

		break;

		case 'delete':
			$checkedfiles = explode(';',$_POST['checkedfiles']);
			foreach($checkedfiles AS $value)
			{
				if( $value == 0 )
				{
					throw new Exception(i18n("For security reasons, its not possible to delete the whole albums folder!"));
				}

				$data = $linpha->db->GetRow("SELECT parent_id FROM ".LIN_PREFIX."photos " .
					"WHERE id = '".LinSql::linAddslashes($value)."'");

				if( !isset($data['parent_id']) )
				{
					throw new Exception(i18n("This file doesn't exists anymore!").' file id: '.$value);
				}

				/**
				 * check write permission of parent folder
				 */
				if( ! $linpha->sql->photoIsAllowed( $data['parent_id'], 'write') )
				{
					throw new Exception(i18n($str_nopermission));
				}
				
				$full_filename = LinSql::getFullImagePath( $value );

				if( $full_filename == false )
				{
					throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
				}

				LinFilesys::rm_rf($full_filename);
				
				if(file_exists($full_filename))
				{
					throw new Exception(i18n("Deleting Files/Folders Failed").": \"".htmlspecialchars($full_filename,ENT_QUOTES)."\"".LIN_NL.i18n($str_checkpermissions));
				}

				linSysLog(i18n("File/Folder deleted successfully!"));
				LinImport::deleteEntry( $value, $full_filename);
				
				linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_delete',"Deleted file/folder (".$full_filename.")");
			}
		break;

		case 'create_folder':
			/**
			 * check write permission of parent folder
			 */
			if( ! $linpha->sql->photoIsAllowed( $linpha->template->idCurrent, 'write') )
			{
				throw new Exception(i18n($str_nopermission));
			}
			
			$new_name = removeUnWantedChars( $_POST['folder_name'] );
			
			if( nameAlreadyExists( $new_name, $linpha->template->idCurrent ))
			{
				throw new Exception(i18n($str_filealreadyexists));
			}
			
			$full_filename = LinSql::getFullImagePath( $linpha->template->idCurrent );

			if( $full_filename == false )
			{
				throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
			}

			if( empty( $new_name ) ) //catch empty folder create submit
			{
				throw new Exception(i18n("Create Folder Failed, No Folder Name given"));			
			}
			elseif(! @mkdir( $full_filename .'/'. $new_name ) )
			{
				throw new Exception(i18n("Create Folder Failed").": \"".htmlspecialchars($new_name,ENT_QUOTES)."\"".LIN_NL.i18n($str_checkpermissions));
			}

			linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_create_folder',"Created folder (".$full_filename.'/'.$new_name.")");
			linSysLog(i18n("Folder Created Successfully!"));

			if( ! $linpha->sql->config->value['sys_import_autoimport'] )
			{
				linSysLog(i18n($str_autoimportdisabled));
			}

		break;
		case 'changeperm':
			$full_filename = LinSql::getFullImagePath( $_POST['file_id'] );

			if( $full_filename == false )
			{
				throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
			}

			isset($_POST['ur']) ? $ur = $_POST['ur'] : $ur = 0;
			isset($_POST['uw']) ? $uw = $_POST['uw'] : $uw = 0;
			isset($_POST['ux']) ? $ux = $_POST['ux'] : $ux = 0;
			isset($_POST['gr']) ? $gr = $_POST['gr'] : $gr = 0;
			isset($_POST['gw']) ? $gw = $_POST['gw'] : $gw = 0;
			isset($_POST['gx']) ? $gx = $_POST['gx'] : $gx = 0;
			isset($_POST['or']) ? $or = $_POST['or'] : $or = 0;
			isset($_POST['ow']) ? $ow = $_POST['ow'] : $ow = 0;
			isset($_POST['ox']) ? $ox = $_POST['ox'] : $ox = 0;
		
			$u = $ur+$uw+$ux;
			$g = $gr+$gw+$gx;
			$o = $or+$ow+$ox;
			
			$mode = $u.$g.$o;
			
			if(! @chmod($full_filename,intval($mode,8)))
			{
				throw new Exception(i18n("Change Permissions Failed").": \"".htmlspecialchars($full_filename,ENT_QUOTES)."\" to ".$mode.LIN_NL.i18n($str_checkpermissions));
			}

			linSysLog(i18n("Permissions Changed Successfully!"));
			linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_perm',"changed permission of ".$full_filename." to ".$mode);

		break;
		case 'upload':
			$targetdir = LinSql::getFullImagePath( $linpha->template->idCurrent );
			
			$full_filename = LinSql::getFullImagePath( $linpha->template->idCurrent );
			if( $full_filename == false )
			{
				throw new Exception(i18n("This file doesn't exists anymore!").' '.$full_filename);
			}
	
			for($i = 0; $i < $linpha->sql->config->value['plugins_filemanager_nruploads']; $i++ )
			{
				if( isset( $_FILES[ 'file' . $i ] ) && $_FILES[ 'file' . $i]['name'] )	// break loop as sson as 'name' is not set, to prevent a couple of warnings
				{
					$thisfile = $_FILES[ 'file' . $i ];
							
					/**
					 * check errors
					 */
					if (isset($thisfile['error']) && $thisfile['error'] != 0 )	// ['error']  was added in PHP 4.2.0
					{
						switch ( $thisfile['error'] )
						{
						case 1: 
		                    throw new Exception( sprintf(i18n("Error: The uploaded file %s exceeds the upload_max_filesize directive in php.ini - %s"), 
		                    	htmlspecialchars($thisfile['name'],ENT_QUOTES), ini_get('upload_max_filesize') ) );
		                    break;				
		                case 2:
		                    throw new Exception( sprintf(i18n("size of uploaded file %s exceeds the HTML FORM settings"), htmlspecialchars($thisfile['name'],ENT_QUOTES) ) );
		                    break;				
		                case 3:
		                    throw new Exception( sprintf(i18n("The uploaded file %s was only partially uploaded"), htmlspecialchars($thisfile['name'],ENT_QUOTES) ) );
		                    break;
						}
					}
	
					if ( $_POST[ 'app' . $i ] == '-' )
					{
						list($org_width, $org_height, $org_type) = LinIdentify::linGetImageSize( $thisfile['tmp_name'] );
						if( !LinIdentify::isSupportedImage( $org_type ) && !LinIdentify::isVideo( $org_type ) )
						{
							throw new Exception( i18n("This File Is Wether An Image Nor A Video").' ('.htmlspecialchars($thisfile['name'],ENT_QUOTES).')' );
						}
						
						if ( !move_uploaded_file( $thisfile['tmp_name'], "".$targetdir.'/'.$thisfile['name']."" ) )
						{
	                        throw new Exception( sprintf(i18n("Unable To Move File (%s) To Directory (%s)"),htmlspecialchars($thisfile['name'],ENT_QUOTES),htmlspecialchars($targetdir,ENT_QUOTES ) ) ); 
						}
						
						$somethingUploaded = true;
					}
					/**
					 * extract archives
					 */
					else	// $_POST[ 'app' . $i ] != '-'
					{
						$tmpfolder = LinFilesys::createTmpFolder('filemanager');
	
						if ( !move_uploaded_file( $thisfile['tmp_name'], $tmpfolder . '/' . $thisfile['name'] ) )
						{
	                        throw new Exception(sprintf( i18n("Unable To Move File (%s) To Directory (%s)"), htmlspecialchars($thisfile[ 'name' ],ENT_QUOTES), htmlspecialchars($targetdir,ENT_QUOTES) ) ); 
						}
	
						
						$command = $apps->apps[$_POST['app'.$i]]['command_extract'];
						$apps->searchApp($_POST['app'.$i]);
	
						$executable = $apps->found_apps[$_POST['app'.$i]].$apps->apps[$_POST['app'.$i]]['executable_extract'];
	
						$command = str_replace('{executable}',$executable,$command);
						$command = str_replace('{archive_name}',linEscapeString($thisfile['name']),$command);
						
						$oldpwd = getcwd();
						if ( !chdir( $tmpfolder ) )
						{
	                        throw new Exception( sprintf( i18n("Unable to switch (chdir) to %s directory. File being processed: %s"), $tmpfolder, $thisfile[ 'name' ] ) );
						}
	
						$output=array(); $return_value='';	// do not this directly in exec() !
						exec( $command, $output, $return_value );
						chdir( $oldpwd );
	                    
						
						echo '<textarea cols="80" rows="4"">' .
							htmlspecialchars( $command . "\n". implode( "\n", $output ), ENT_QUOTES ) .
							'</textarea><br />';
						
						unset($output);
	
						/**
						 * move valid images and videos to target dir
						 * this will delete the archive file automatically...
						 * and also delete the tmpfolder
						 */
						moveValidEntries( $tmpfolder, $targetdir );
	
	                    $somethingUploaded = true;
					}
	
	            linSysLog(i18n("File uploaded").": ".$thisfile['tmp_name']." ".$targetdir.'/'.$thisfile['name']);
	            linLog(LOG_TYPE_FILEMANAGER,LOG_NOTICE,'fm_upload',"Uploaded file (".$thisfile['tmp_name']." ".$targetdir.'/'.$thisfile['name'].")");
				}
			}
			
			if(isset($somethingUploaded))
			{
				if( ! $linpha->sql->config->value['sys_import_autoimport'] )
				{
					linSysLog(i18n($str_autoimportdisabled));
				}
			}
		break;
		}	// end switch($_POST['file_action'])
	break;	// end case 'do_file_action':

		} // end switch($_REQUEST['cmd'])
	} // end try()
	catch(Exception $error)
	{
		linSysLog("Error: ".str_replace(LIN_NL,"<br />",$error -> getMessage()));
        linLog(LOG_TYPE_FILEMANAGER,LOG_ERR,'fm_'.$_REQUEST['file_action'],$error->getMessage());
	}

	/**
	 * print Syslog
	 */
	/*if( isset($linpha->template->output['sys_log']) )
	{
		echo $linpha->template->output['sys_log'];
		echo '<hr />';
		unset($linpha->template->output['sys_log']);
	}*/
}

/**
 * do import
 */
	if( $linpha->sql->config->value['sys_import_autoimport'] )
	{
		/**
		 * empty syslog
		 */
		//unset($linpha->template->output['sys_log']);
		
		LinImport::updateDir( $parent_id, $recursive = false, $dryrun = false );

		/**
		 * print Syslog
		 */
		/*if( isset($linpha->template->output['sys_log']) )
		{
			echo i18n("LinPHA Import:").'<br />';
			echo $linpha->template->output['sys_log'];
			echo '<hr />';
			unset($linpha->template->output['sys_log']);
		}*/
	}

echo i18n("Current Folder:").' '.linSetNavigationLine( $parent_id, LINPHA_LINK.'&amp;linCat=filemanager&amp;linId=').'<br />';
?>

<form method="POST" action="<?php echo LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$parent_id; ?>">
<div id="linDivFilemanager">
<table>
	<tr>
		<td width="20">&nbsp;</td>
		<td width="500">
		<?php
		if($parent_id == 0)
		{
			echo '&nbsp;';
		}
		else
		{
			$data = $linpha->db->GetRow("SELECT name, parent_id FROM ".LIN_PREFIX."photos WHERE id = '".LinSql::linAddslashes($parent_id)."'");
			?>
			<a href="<?php echo LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$data['parent_id']; ?>">..</a>
			<?php
		}
		?>
		</td>
		<td width="100">&nbsp;</td>
		<td width="150">&nbsp;</td>
		<td width="100">&nbsp;</td>
	</tr>

<?php

$query = $linpha->db->Execute("SELECT id, name, img_type FROM ".LIN_PREFIX."photos WHERE parent_id = '".LinSql::linAddslashes($parent_id)."' ORDER by name");
while($data = $query->FetchRow())
{
	$full_filename = LinSql::getFullImagePath( $data['id'] );

	/**
	 * show only if file exists and we have READ permissions
	 */	
	if( file_exists( $full_filename ) && $linpha->sql->photoIsAllowed( $data['id'], 'read' ) )
	{
		/**
		 * set file informations
		 */
		$filestat = stat($full_filename);
	
		/**
		 * size and nr files
		 */
	    if( $data['img_type'] != 0 && $data['img_type'] != 9999999 )
	    {
			$sum['size']  += $filestat['size'];
			$sum['files']++;
	    }

		/**
		 * owner and group name
		 */
		if($use_posix) {
			$owner_info = @posix_getpwuid($filestat['uid']);  // use @ because check_posix() can return true even if it fails...
			$group_info = @posix_getgrgid($filestat['gid']);
		} else {
			$owner_info['name'] = '';
			$group_info['name'] = '';
		}
	
		/**
		 * file permissions
		 */
		if( LIN_OS == 'win' )
		{
			$file_permissions    =
				'[' .
				(is_readable($full_filename) ? '<font color="green">r</font>' : '&nbsp;') .
				(is_writable($full_filename) ? '<font color="red">w</font>' : '&nbsp;') .
				(function_exists('is_executable') ?
					(is_executable($full_filename) ? '<font color="blue">x</font>' : '&nbsp;')
					:
					'&nbsp;'
					) .
				']';
		} else {
			$perm = '['.getReadableFileperms($full_filename).']';
			
			$perm = str_replace('r','<font color="green">r</font>',$perm);
			$perm = str_replace('w','<font color="red">w</font>',$perm);
			$perm = str_replace('x','<font color="blue">x</font>',$perm);
			
			if($uid == $filestat['uid'])
			{
				$file_permissions = '<a href="'.LINPHA_LINK.'&amp;linCat=filemanager&amp;cmd=file_action&amp;file_action=changeperm&amp;checkedfiles[]='.$data['id'].'">'. $perm . '</a>';
			}
			else
			{
				$file_permissions = $perm;
			}
		}
	
		?>
		<tr>
			<td>
				<?php
				if($isWriteAble)
				{
					if( isset($_REQUEST['checkedfiles']) 
						&& is_array($_REQUEST['checkedfiles'])
						&& in_array($data['id'], $_REQUEST['checkedfiles']) )
					{
						$str_checked = ' checked="checked"';
					}
					else
					{
						$str_checked = '';
					}
				?>
					<input type="checkbox" name="checkedfiles[]" value="<?php echo $data['id']; ?>"<?php echo $str_checked; ?> />
				<?php } else { ?>
					&nbsp;
				<?php } ?>
			</td>
			<td>
			<?php
			if($data['img_type']==0)
			{
				echo '<a href="'.LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$data['id'].'">'.htmlspecialchars($data['name'],ENT_QUOTES).'</a>';
			}
			else
			{
				echo htmlspecialchars($data['name'],ENT_QUOTES);
			}
			?>
			</td>
			<td align="right"><?php echo $data['img_type']==0 ? '&nbsp;' : LinFilesys::niceFilesize($filestat['size'],1); ?></td>
			<td align="center"><?php echo strftime("%x %X",$filestat['mtime']); ?></td>
			<td align="center" title="<?php echo $owner_info['name'].' '.$group_info['name']; ?>"><?php echo $file_permissions; ?></td>
		</tr>
		<?php
	}
}
?>
</table>

<?php
/**
 * set checked radio buttons
 */
	$array_values = Array('rename','move','copy','delete');
	foreach($array_values AS $value)
	{
		if(isset($_POST['file_action']) && $_POST['file_action'] == $value)
		{
			$arr_checked[$value] = ' checked="checked"';
			$somtething_checked = true;
		}
		else
		{
			$arr_checked[$value] = '';
		}
	}
	
	/**
	 * default checked
	 */
	if(!isset($something_checked))
	{
		$arr_checked['rename'] = ' checked="checked"';		
	}

if($isWriteAble)
{
?>
&nbsp;&nbsp;^&nbsp;<a href="javascript:select_all()"><?php echo i18n("Select All"); ?></a>
<input type="radio" id="file_rename" name="file_action" value="rename"<?php echo $arr_checked['rename']; ?> /><label for="file_rename"> <?php echo i18n("Rename"); ?></label>
<input type="radio" id="file_move" name="file_action" value="move"<?php echo $arr_checked['move']; ?> /><label for="file_move"> <?php echo i18n("Move"); ?></label>
<input type="radio" id="file_copy" name="file_action" value="copy"<?php echo $arr_checked['copy']; ?> /><label for="file_copy"> <?php echo i18n("Copy"); ?></label>
<input type="radio" id="file_delete" name="file_action" value="delete"<?php echo $arr_checked['delete']; ?> /><label for="file_delete"> <?php echo i18n("Delete"); ?></label>
<input type="hidden" name="cmd" value="file_action" />
<input type="submit" class="linButton" name="submit" value="<?php echo i18n("Submit"); ?>" />
</form>

<br /><br />
<form method="POST" action="<?php echo LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$parent_id; ?>">
<input type="text" class="linForms" name="folder_name" value="" />
<input type="hidden" name="cmd" value="do_file_action" />
<input type="hidden" name="file_action" value="create_folder" />
<input type="submit" class="linButton" name="submit" value="<?php echo i18n("Create Folder"); ?>" />
</form>

<br />
<h2 class="linStyle"><?php echo i18n("Upload Files"); ?></h2>
<form method="POST" enctype="multipart/form-data" action="<?php echo LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$parent_id; ?>">
<?php
echo i18n("Choose files you would like to upload. Choose desired action to accomplish upon succesful upload.");
echo '<br />';
echo i18n("Allowed filesize (each file!) maximum currently set in php.ini is:").' '.ini_get('upload_max_filesize');
echo '<br />';
if( $linpha->sql->isAdmin() )
{
	echo i18n("Note: You should set post_max_size to a higher value than upload_max_filesize (currently set to:".' '.ini_get('post_max_size').")");
	echo '<br />';
	echo i18n("Note: You don't have to upload your images through this form. You can use whatever you want (ftp,scp,nfs,local,...). Just copy them to the albums folder.");
	echo '<br />';
}

echo '<br />';

for($i = 0; $i < $linpha->sql->config->value['plugins_filemanager_nruploads']; $i++)
{
	echo '<input type="file" name="file'.$i.'" value="" class="linForms" />&nbsp;';
	echo '<select name="app'.$i.'"class="linForms">'.$appselect.'</select><br />';
}
?>

<input type="hidden" name="cmd" value="do_file_action" />
<input type="hidden" name="file_action" value="upload" />
<input type="submit" class="linButton" name="submit" value="<?php echo i18n("Upload Files"); ?>"/>
</form>
<?php
}
?>
</div>

<script language="JavaScript" type="text/javascript">
var is_checked = false;
function select_all()
{
	var input_forms = document.getElementsByTagName('input');

	// loop through all anchor tags
	for (var i=0; i < input_forms.length; i++)
	{
		var anchor = input_forms[i];
		
		if(anchor.type == 'checkbox')
		{
			if( is_checked )
			{
				anchor.checked = false;
			}
			else
			{
				anchor.checked = true;
			}			
		}
	}

	if( is_checked )
	{
		is_checked = false;		
	}
	else
	{
		is_checked = true;
	}			
}
</script>
  

<?php
$linpha->template->output['default'] = ob_get_clean();

$linpha->template->setModuleName('default');
$linpha->template->overrideModule('css','filemanager');
$linpha->template->URL_full = LINPHA_LINK.'&amp;linCat=filemanager&amp;linId='.$parent_id;
$linpha->template->output['title'] = i18n("Filemanager");
include_once(LINPHA_DIR.'/templates/'.$linpha->template->template_name.'/global.html.php');
?>


<?php
/**
* found on http://ch.php.net/manual/en/function.fileperms.php
*/
function getReadableFileperms($file)
{
	$perms = fileperms($file);
	
	$info = '';
	
	if (($perms & 0xC000) == 0xC000) {
	// Socket
	$info .= 's';
	} elseif (($perms & 0xA000) == 0xA000) {
	// Symbolic Link
	$info .= 'l';
	} elseif (($perms & 0x8000) == 0x8000) {
	// Regular
	$info .= '-';
	} elseif (($perms & 0x6000) == 0x6000) {
	// Block special
	$info .= 'b';
	} elseif (($perms & 0x4000) == 0x4000) {
	// Directory
	$info .= 'd';
	} elseif (($perms & 0x2000) == 0x2000) {
	// Character special
	$info .= 'c';
	} elseif (($perms & 0x1000) == 0x1000) {
	// FIFO pipe
	$info .= 'p';
	} else {
	// Unknown
	$info .= 'u';
	}
	
	// Owner
	$info .= (($perms & 0x0100) ? 'r' : '-');
	$info .= (($perms & 0x0080) ? 'w' : '-');
	$info .= (($perms & 0x0040) ?
			(($perms & 0x0800) ? 's' : 'x' ) :
			(($perms & 0x0800) ? 'S' : '-'));
	
	// Group
	$info .= (($perms & 0x0020) ? 'r' : '-');
	$info .= (($perms & 0x0010) ? 'w' : '-');
	$info .= (($perms & 0x0008) ?
			(($perms & 0x0400) ? 's' : 'x' ) :
			(($perms & 0x0400) ? 'S' : '-'));
	
	// World
	$info .= (($perms & 0x0004) ? 'r' : '-');
	$info .= (($perms & 0x0002) ? 'w' : '-');
	$info .= (($perms & 0x0001) ?
			(($perms & 0x0200) ? 't' : 'x' ) :
			(($perms & 0x0200) ? 'T' : '-'));
	

	return $info;
}

/**
* this function checks if the posix functions are available
*
* @author  flo
* @return  bool  true if posix enabled, false if not
* @package  requirements
*/
function checkPosix()
{
	$array_needed_functions = Array(
		'posix_getuid',
		'posix_getpwuid',
		'posix_getgrgid'
	);
	
	foreach($array_needed_functions AS $value)
	{
		if( !function_exists($value) )
		{
			return false;
		}
	}
	
    /**
    * can return true even if it fails...!!! seen on sf.net
    */
	if($uid = @posix_getuid())
	{
		if(@posix_getpwuid($uid))
		{
			return true;
		}
	}
    
    return false;
}

/**
 * make sure to keep in current directory
 * @todo  is this enough?
 * @uses  do_file_action (rename and createfolder)
 * @author flo
 */
function removeUnWantedChars( $str )
{
	$str = str_replace('/','',$str);
	$str = str_replace('\\','',$str);
	
	return $str;
}

/**
 * checks if a file/folder with the same name already exists in the same folder
 */
function nameAlreadyExists( $name, $parent_id )
{
	$query = $GLOBALS['linpha']->db->Execute("SELECT id FROM ".LIN_PREFIX."photos " .
		"WHERE name = '".LinSql::linAddslashes($name)."' " .
		"AND parent_id = '".LinSql::linAddslashes($parent_id)."'");
	
	if( $query->EOF) // not a single value returned
	{
		return false;
	}
	else
	{
		return true;
	}
}

/**
 * recursively copy files/folders
 * check read permission of source folder
 * -> check each subentry!
 */
function copy_r( $source_id, $source, $dest )
{
	if( $GLOBALS['linpha']->sql->photoIsAllowed( $source_id, 'read' ) )
	{
		$data = $GLOBALS['linpha']->db->GetRow("SELECT img_type FROM ".LIN_PREFIX."photos " .
			"WHERE id = '".LinSql::linAddslashes($source_id)."'");

		if($data['img_type']!=0)
		{
			@copy( $source, $dest.'/'.basename($source));
		}
		else
		{
			mkdir( $dest.'/'.basename($source) );
			
			$query = $GLOBALS['linpha']->db->Execute("SELECT id, name, img_type FROM ".LIN_PREFIX."photos " .
				"WHERE parent_id = '".LinSql::linAddslashes($source_id)."'");
			while($data = $query->FetchRow())
			{
				if( file_exists($source.'/'.$data['name'] ) )
				{
					copy_r( $data['id'], $source.'/'.$data['name'], $dest.'/'.basename($source) );
				}
			}
		}
	}
}

/**
 * recursively move files/folders
 * check that only valid images/videos will be uploaded
 * and not any malicious data (like php scripts etc.)
 */
function moveValidEntries( $source, $dest )
{
	$dirHandle = dir( $source );
	while (false !== ($entry = $dirHandle->read()))
	{
		if($entry != '.' && $entry != '..')
		{
			if(is_dir($source .'/'. $entry))
			{
				mkdir( $dest.'/'.$entry );
				moveValidEntries( $source .'/'. $entry, $dest .'/'. $entry );
			}
			else
			{
				list($org_width, $org_height, $org_type) = LinIdentify::linGetImageSize( $source .'/'. $entry );
				if( LinIdentify::isSupportedImage( $org_type ) OR LinIdentify::isVideo( $org_type ) )
				{
					rename( $source .'/'. $entry, $dest .'/'. $entry);
				}
				else
				{
					unlink( $source .'/'. $entry );
				}
			}
		}
	}
	$dirHandle->close();
	rmdir( $source );
}
?>