<?php
/*
 * Copyright (c) 2005 Heiko Rutenbeck <bzrudi@tuxpower.de>
 *                    Florian Angehrn 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/**
 * Settings
 * @package Modules
 */
 
if(!defined('LINPHA_DIR')) { exit(1); }

include_once(LINPHA_DIR.'/lib/classes/linpha.admin.class.php');
$LinAdmin = new LinAdmin();

/**
 * check permissions
 */
	if( ! $GLOBALS['linpha']->sql->isLoggedIn())
	{
		linSysLog(i18n("Please Login!"));
		$_GET['linCat'] = 'alb';
		include_once(LINPHA_DIR.'/lib/modules/module.albums.php');
		exit();
	}
	
/**
 * get user data
 */
	$user_data = $GLOBALS['linpha']->db->GetRow("SELECT username, password, display_name, user_email ".
		"FROM ".LIN_PREFIX."users ".
		"WHERE id = '".$_SESSION['user_id']."'");

/**
 * get user settings
 */	
	$options = Array();
	$query = $GLOBALS['linpha']->db->Execute("SELECT option_name FROM ".LIN_PREFIX."config WHERE override = 1 ORDER by option_name");
	while($data = $query->FetchRow())
	{
		$options[] = $data['option_name'];
	}
	$radio_options = LinAdmin::getRadioOptions();

/**
 * output buffering, to use in template
 */
ob_start();

/**
 * parse POST data
 */
if(isset($_POST['cmd']))
{
	try
	{
	switch($_POST['cmd'])
	{
	case 'saveuserinfos':

		/**
		 * check if username is empty
		 */
		if( empty($_POST['username']) )
		{
			throw new Exception(i18n("The username may not be empty."));
		}

		/**
		 * check if username already exists
		 * (only if it will be changed)
		 */
		if( $user_data['username'] != $_POST['username'] )
		{
			$query = $GLOBALS['linpha']->db->Execute("SELECT username FROM ".LIN_PREFIX."users " .
				"WHERE username = '".LinSql::linAddslashes($_POST['username'])."'");
			if( ! $query->EOF )
			{
				throw new Exception(i18n("A user with the same username already exists."));
			}
		}
		
		/**
		 * check if it is a valid email
		 */
		if( ! LinAdmin::checkValidEmailAddress($_POST['user_email']) )
		{
			throw new Exception(i18n("This is not a valid email address."));
		}
		
		/**
		 * update data in db
		 */
		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."users SET " .
			"username = '".LinSql::linAddslashes($_POST['username'])."', " .
			"user_email = '".LinSql::linAddslashes($_POST['user_email'])."', " .
			"display_name = '".LinSql::linAddslashes($_POST['display_name'])."' " .
			"WHERE id = '".$_SESSION['user_id']."'");
			
		/**
		 * update data also for show in html form
		 */
		$user_data['username'] = $_POST['username'];
		$user_data['user_email'] = $_POST['user_email'];
		$user_data['display_name'] = $_POST['display_name'];
		
		/**
		 * update session variables
		 */
        $_SESSION['user_name'] = $_POST['username'];
        $_SESSION['user_displayname'] = (empty($_POST['display_name'])) ? $_POST['username'] : $_POST['display_name'];  //Store user fullname. If fullname is blank - store username.


		linSysLog(i18n("Successfully updated data."));

		break;
	case 'changepw':
		/**
		 * check if old password is correct
		 */
		if( md5($_POST['old_password']) != $user_data['password'] )
		{
			throw new Exception(i18n("Old password is not correct."));
		}
		
		/**
		 * check length of new password
		 */
		if( strlen($_POST['new_password'])<3 )
		{
			throw new Exception(i18n("Password To Short"));
		}
		
		/**
		 * check correct repeating passwords
		 */
		if( $_POST['new_password'] != $_POST['new_password2'] )
		{
			throw new Exception(i18n("New password doesn't match with retyped password"));
		}
		
		/**
		 * update password
		 */
		$GLOBALS['linpha']->db->Execute("UPDATE ".LIN_PREFIX."users SET " .
			"password = '".md5($_POST['new_password'])."' WHERE id = '".$_SESSION['user_id']."'");
		
		linSysLog(i18n("Password successfully updated."));
		
		break;
	case 'saveconfig':
		/**
		 * save and reload config
		 */
		$LinAdmin->saveConfig($options,$_SESSION['user_id']);
		linSysLog(i18n("Successfully updated data."));
		break;
	case 'loaddefault':
		$GLOBALS['linpha']->db->Execute("DELETE FROM ".LIN_PREFIX."config " .
			"WHERE user_id = '".$_SESSION['user_id']."'");
		$GLOBALS['linpha']->sql->config->reloadConfig();
		linSysLog(i18n("Successfully updated data."));
		break;
	}

	}
	catch(Exception $error)
	{
		linSysLog("Error: ".$error -> getMessage());
	}
}

?>
<h2 class="linStyle"><?php echo i18n("Change User Information"); ?></h2>
<form method="POST" action="./?linCat=settings">
<?php
$LinAdmin->printAdminConfig('text',i18n("Username"),'username',$user_data['username']);
$LinAdmin->printAdminConfig('text',i18n("Display Name"),'display_name',$user_data['display_name']);
$LinAdmin->printAdminConfig('text',i18n("Email"),'user_email',$user_data['user_email']);
?>
<input type="hidden" name="cmd" value="saveuserinfos" />
<input type="submit" name="submit" value="<?php echo i18n("Submit"); ?>" class="linButton" />
</form>
<hr />

<h2 class="linStyle"><?php echo i18n("Change Password"); ?></h2>
<form method="POST" name="change_password" action="./?linCat=settings">
<?php
$LinAdmin->printAdminConfig('password',i18n("Old Password"),'old_password','');
$LinAdmin->printAdminConfig('password',i18n("New Password"),'new_password','');
$LinAdmin->printAdminConfig('password',i18n("Repeat New Password"),'new_password2','');
?>
<input type="hidden" name="cmd" value="changepw" />
<input type="submit" name="submit" value="<?php echo i18n("Submit"); ?>" class="linButton" onclick="return checkPassLength();" />
</form>
<script language="JavaScript" type="text/javascript">
function checkPassLength()
{
	if(document.change_password.new_password.value.length <3  )
	{
		alert ('<?php echo i18n("Password To Short"); ?>');
		document.change_password.new_password.focus();
		return false;
	}
}
</script>
<hr />

<h2 class="linStyle"><?php echo i18n("Change User Settings"); ?></h2>
<form method="POST" action="./?linCat=settings">
<?php
foreach($options AS $value)
{
	if(in_array($value,$radio_options))
	{
		$type = 'radio';
	}
	else
	{
		$type = 'text';
	}
	$LinAdmin->printAdminConfig($type,$LinAdmin->getDescriptionByOptionName($value),$value,$GLOBALS['linpha']->sql->config->value[$value]);
}
?>
<input type="hidden" name="cmd" value="saveconfig" />
<input type="submit" name="submit" value="<?php echo i18n("Submit"); ?>" class="linButton" />
</form>

<form method="POST" action="./?linCat=settings">
<input type="hidden" name="cmd" value="loaddefault" />
<input type="submit" name="submit" value="<?php echo i18n("Load default values"); ?>" class="linButton" />
</form>
<hr />

<?php
$linpha->template->output['default'] = ob_get_clean();
$linpha->template->setModuleName('default');
$linpha->template->URL_full = LINPHA_LINK.'&amp;linCat=settings';
$linpha->template->output['title'] = i18n("Settings");
include_once(LINPHA_DIR.'/templates/'.$linpha->template->template_name.'/global.html.php');
?>