
/**
 * This method parses an XML file (currently in Garmin GPX format) and saves 
 * all available information in the DB
 * 
 * @param string filename
 * @author bzrudi
 */
function parseXMLDataFile($filename)
{
	
$gpxparser = new GpxParser($filename);
list($xmldata, $filetype) = $gpxparser->getGpxFileType();

	switch($filetype)
	{
		case "trkpt":
		{
			foreach( $xmldata->gpx->trk->trkseg->trkpt as $trkinfo)
			{
				$latlon = $trkinfo->attributes(); //array(lat,lon)
		
				$lat = $latlon['lat'];
				$lon = $latlon['lon'];
				$ele = $trkinfo->ele->CDATA();
				$time = $trkinfo->time->CDATA();
				
				//echo"LAT:".$lat."LON:".$lon."ELE:".$ele."<br>";
				
				$location = new Location($lat, $lon, '<div class="someCssClass">' .
						'LAT: '.$lat.'<br>' .
						'LON: '.$lon.'<br>' .
						'H&Ouml;HE: '.$ele.'m<br>' .
						'DATE: '.$time.'</div>');
				$this->cmap->addLocation($location);
			}
		break;		
		}
	
		case "rtept":
		{
			foreach( $xmldata->gpx->rte->rtept as $pointinfo)
			{
				$latlon = $pointinfo->attributes(); //array(lat,lon)
	
				$lat = $latlon['lat'];
				$lon = $latlon['lon'];
				//$ele = $pointinfo->ele->CDATA();
				//$name = $pointinfo->name->CDATA();
				
				//echo"LAT:".$lat."LON:".$lon."ELE:".$ele."NAME:".$name."<br>";
				
				$location = new Location($lat, $lon, '<div class="someCssClass">' .
						'LAT: '.$lat.'<br>' .
						'LON: '.$lon.'<br>' .
						'H&Ouml;HE: na<br>' .
						'Name: na</div>');
				$this->cmap->addLocation($location);
			}
		break;		
		}
	
		case "wpt":
		{
			foreach( $xmldata->gpx->wpt as $wpointinfo)
			{
				$latlon = $wpointinfo->attributes(); //array(lat,lon)
	
				$lat = $latlon['lat'];
				$lon = $latlon['lon'];
				$ele = $wpointinfo->ele->CDATA();
				$name = $wpointinfo->name->CDATA();
				$cmt = $wpointinfo->cmt->CDATA();
				//$desc = $wpointinfo->desc->CDATA();
				$sym = $wpointinfo->sym->CDATA();
							
				//echo"LAT: ".$lat."LON: ".$lon."ELE: ".$ele."NAME: ".$name." " .
				//		"CMT: ".$cmt."SYMBOL: ".$sym."<br>";
				
				$location = new Location($lat, $lon, '<div class="someCssClass">' .
						'LAT: '.$lat.'<br>' .
						'LON: '.$lon.'<br>' .
						'ELE: '.$ele.'m<br>' .
						'NAM: '.$name.'<br>' .
						'CMT: '.$cmt.'<br>' .
						'SYM: '.$sym.'<br></div>');
				$this->cmap->addLocation($location);
			}
		break;		
		}
		case "failed":
		{
			echo "CANNOT DETECT INPUT FILE FORMAT";
		}
	}
}



